@@ -26,8 +26,6 @@
 #include "config.h"
 #include "applix.h"
 #include "application.h"
-#include "command-context.h"
-#include "workbook.h"
 #include "expr.h"
 #include "value.h"
 #include "sheet.h"
@@ -38,6 +36,9 @@
 #include "selection.h"
 #include "position.h"
 #include "ranges.h"
+#include "io-context.h"
+#include "workbook-view.h"
+#include "workbook.h"
 
 #include <ctype.h>
 #include <string.h>
@@ -47,7 +48,8 @@ static int debug_applix_read = 1;
 
 typedef struct {
 	FILE		*file;
-	CommandContext	*context;
+	IOContext	*context;
+	WorkbookView	*wb_view;
 	Workbook	*wb;
 	GHashTable	*exprs, *styles;
 	GPtrArray	*colours;
@@ -67,7 +69,7 @@ static int
 applix_parse_error (ApplixReadState *state, char const *msg)
 {
 	gchar *tmp = g_strconcat ("APPLIX : ", msg, NULL);
-	gnumeric_error_read (state->context, msg);
+	gnumeric_io_error_read (state->context, msg);
 	g_free (tmp);
 	return -1;
 }
@@ -311,6 +313,11 @@ applix_parse_style (ApplixReadState *state, char **buffer)
 		/* Formating and alignment */
 		for (; *sep && *sep != '|' && *sep != ')' ; ) {
 
+			if (*sep == ',') {
+				++sep;
+				continue;
+			}
+
 			if (isdigit ((unsigned char)*sep)) {
 				StyleHAlignFlags a;
 				switch (*sep) {
@@ -393,6 +400,10 @@ applix_parse_style (ApplixReadState *state, char **buffer)
 					sep += 2;
 					break;
 				}
+				case 'G' : /* general */
+					mstyle_set_format_text (style, "General");
+					break;
+
 				case 'C' : /* currency or comma */
 					/* comma 'CO' */
 					if (sep[1] == 'O') {
@@ -411,7 +422,6 @@ applix_parse_style (ApplixReadState *state, char **buffer)
 					if (!format_suffix)
 						format_suffix = "%";
 
-				case 'G' : /* general FIXME */
 				case 'F' : /* fixed */
 				{
 					static char const *zeros = "000000000";
@@ -456,9 +466,6 @@ applix_parse_style (ApplixReadState *state, char **buffer)
 				if (needs_free)
 					g_free (format);
 			}
-
-			if (*sep == ',')
-				++sep;
 		}
 
 		/* Font spec */
@@ -702,10 +709,10 @@ applix_get_sheet (ApplixReadState *state, char **buffer,
 	}
 
 	*tmp = '\0';
-	sheet = workbook_sheet_lookup (state->wb, *buffer);
+	sheet = workbook_sheet_by_name (state->wb, *buffer);
 	if (sheet == NULL) {
 		sheet = sheet_new (state->wb, *buffer);
-		workbook_attach_sheet (state->wb, sheet);
+		workbook_sheet_attach (state->wb, sheet, NULL);
 		sheet_set_zoom_factor (sheet, (double )(state->zoom) / 100., FALSE, FALSE);
 	}
 
@@ -1030,7 +1037,7 @@ applix_read_cells (ApplixReadState *state)
 								expr, FALSE);
 					cell_assign_value (cell, val, NULL);
 				} else
-					cell_set_expr_and_value (cell, expr, val);
+					cell_set_expr_and_value (cell, expr, val, NULL);
 
 				if (!applix_get_line (state) ||
 				    strncmp (state->buffer, "Formula: ", 9)) {
@@ -1049,7 +1056,7 @@ applix_read_cells (ApplixReadState *state)
 				printf ("shared '%s'\n", expr_string);
 #endif
 				expr = g_hash_table_lookup (state->exprs, expr_string);
-				cell_set_expr_and_value (cell, expr, val);
+				cell_set_expr_and_value (cell, expr, val, NULL);
 			}
 			break;
 		}
@@ -1112,13 +1119,15 @@ applix_read_impl (ApplixReadState *state)
 		return applix_parse_error (state, "Missing dump revision");
 	else if (state->line_len < 0 || 65535 < state->line_len) /* magic sanity check */
 		return applix_parse_error (state, "Invalid line length");
-	else {
-		state->buffer_size = 2 * state->line_len + MAX_END_OF_LINE_SLOP;
-		state->buffer = g_new (gchar, state->buffer_size);
-	}
 
-	if (1 != fscanf (state->file, "Current Doc Real Name: %127s\n", real_name))
-		return applix_parse_error (state, "Official workbook name");
+	state->buffer_size = 2 * state->line_len + MAX_END_OF_LINE_SLOP;
+	state->buffer = g_new (gchar, state->buffer_size);
+
+	/* It appears some docs do not have this */
+	if (1 != fscanf (state->file, "Current Doc Real Name: %127s\n", real_name)) {
+		strncpy (real_name, "Unspecified", sizeof (real_name)-1);
+		ungetc ((unsigned char)'C', state->file); /* restore the leading 'C' */
+	}
 
 #ifndef NO_APPLIX_DEBUG
 	if (debug_applix_read > 0)
@@ -1289,7 +1298,7 @@ applix_read_impl (ApplixReadState *state)
 
 	/* We only need the sheet, the visible cell, and edit pos are already set */
 	if (applix_parse_cellref (state, cur_cell_addr, &sheet, &col, &row, ':'))
-		workbook_focus_sheet (sheet);
+		wb_view_sheet_focus (state->wb_view, sheet);
 
 	return 0;
 }
@@ -1310,7 +1319,8 @@ cb_remove_style (gpointer key, gpointer value, gpointer user_data)
 }
 
 int
-applix_read (CommandContext *context, Workbook *wb, FILE *file)
+applix_read (IOContext *context, WorkbookView *wb_view,
+	     FILE *file)
 {
 	int i;
 	int res;
@@ -1319,7 +1329,8 @@ applix_read (CommandContext *context, Workbook *wb, FILE *file)
 	ApplixReadState	state;
 	state.file	= file;
 	state.context	= context;
-	state.wb	= wb;
+	state.wb_view	= wb_view;
+	state.wb	= wb_view_workbook (wb_view);
 	state.exprs	= g_hash_table_new (&g_int_hash, &g_int_equal);
 	state.styles	= g_hash_table_new (&g_str_hash, &g_str_equal);
 	state.colours	= g_ptr_array_new ();
