@@ -19,6 +19,7 @@
 #include "style.h"
 #include "format.h"
 #include "expr.h"
+#include "gutils.h"
 #include "gnumeric-chart.h"
 
 /* #define NO_DEBUG_EXCEL */
@@ -1833,7 +1834,7 @@ conditional_get_double (gboolean flag, guint8 const *data,
 			gchar const *name)
 {
 	if (!flag) {
-		double const val = BIFF_GETDOUBLE (data);
+		double const val = gnumeric_get_le_double (data);
 		printf ("%s = %f\n", name, val);
 		return TRUE;
 	}
@@ -1845,7 +1846,7 @@ static gboolean
 BC_R(valuerange)(ExcelChartHandler const *handle,
 		 ExcelChart *s, BiffQuery *q)
 {
-	guint16 const flags = BIFF_GETDOUBLE (q->data+40);
+	guint16 const flags = gnumeric_get_le_double (q->data+40);
 
 	conditional_get_double (flags&0x01, q->data+ 0, "Min Value");
 	conditional_get_double (flags&0x02, q->data+ 8, "Max Value");
@@ -2019,7 +2020,7 @@ BC(register_handler)(ExcelChartHandler const *const handle)
 }
 
 void
-ms_excel_chart (BiffQuery *q, MSContainer *container, MsBiffBofData *bof)
+ms_excel_chart (BiffQuery *q, MSContainer *container, MsBiffVersion ver)
 {
 	int const num_handler = sizeof(chart_biff_handler) /
 		sizeof(ExcelChartHandler *);
@@ -2030,12 +2031,10 @@ ms_excel_chart (BiffQuery *q, MSContainer *container, MsBiffBofData *bof)
 	/* Register the handlers if this is the 1sttime through */
 	BC(register_handlers)();
 
-	g_return_if_fail (bof->type == MS_BIFF_TYPE_Chart);
-
 	/* FIXME : create an anchor parser for charts */
 	ms_container_init (&state.container, NULL);
 
-	state.container.ver = bof->version;
+	state.container.ver = ver;
 	state.depth = 0;
 	state.sheet = NULL;
 	state.prev_opcode = 0xdead; /* Invalid */
@@ -2098,7 +2097,7 @@ ms_excel_chart (BiffQuery *q, MSContainer *container, MsBiffBofData *bof)
 			case BIFF_NUMBER:	/* Should figure out what these are associated with */
 			{
 				if (ms_excel_chart_debug > 0)
-					printf ("%f\n", BIFF_GETDOUBLE (q->data + 6));
+					printf ("%f\n", gnumeric_get_le_double (q->data + 6));
 				break;
 			}
 
@@ -2161,7 +2160,11 @@ ms_excel_read_chart (BiffQuery *q, MSContainer *container)
 	/* 1st record must be a valid BOF record */
 	g_return_if_fail (ms_biff_query_next (q));
 	bof = ms_biff_bof_data_new (q);
+
+	g_return_if_fail (bof != NULL);
+	g_return_if_fail (bof->type == MS_BIFF_TYPE_Chart);
+
 	if (bof->version != MS_BIFF_V_UNKNOWN)
-		ms_excel_chart (q, container, bof);
+		ms_excel_chart (q, container, bof->version);
 	ms_biff_bof_data_destroy (bof);
 }
