@@ -4,7 +4,7 @@
  * Author:
  *    Jon K Hellan (hellan@acm.org)
  *
- * (C) 1999 Jon K Hellan
+ * (C) 1999, 2000 Jon K Hellan
  **/
 #ifndef GNUMERIC_MS_EXCEL_UTIL_H
 #define GNUMERIC_MS_EXCEL_UTIL_H
@@ -46,12 +46,10 @@ two_way_table_key_to_idx (const TwoWayTable *table, gconstpointer key);
 gpointer
 two_way_table_idx_to_key (const TwoWayTable *table, gint idx);
 
-
-#define EXCEL_DEFAULT_CHAR_WIDTH 12
-
 /* Measures base character width for column sizing. Returns width. */
+/* A new version based on hard coded tables to match XL */
 double
-lookup_font_base_char_width (StyleFont *font, gboolean logging_condition);
+lookup_font_base_char_width_new (char const * const name, double size_pts,
+				 gboolean const is_default);
 
 #endif
-
