@@ -35,15 +35,18 @@
 #include "position.h"
 #include "color.h"
 #include "cell.h"
+#include "cell-comment.h"
 #include "sheet-object.h"
 #include "style.h"
 #include "format.h"
 #include "main.h"
 #include "parse-util.h"
 #include "print-info.h"
-#include "command-context.h"
+#include "workbook-view.h"
 #include "workbook.h"
+#include "io-context.h"
 #include "expr.h"
+#include "gutils.h"
 
 #include <libole2/ms-ole.h>
 #include "ms-biff.h"
@@ -2302,13 +2305,13 @@ write_value (BiffPut *bp, Value *v, MsBiffVersion ver,
 			EX_SETROW(data, row);
 			EX_SETCOL(data, col);
 			EX_SETXF (data, xf);
-			BIFF_SETDOUBLE (data + 6, val);
+			gnumeric_set_le_double (data + 6, val);
 			ms_biff_put_commit (bp);
 		} else { /* Nasty RK thing S59DDA.HTM */
 			guint8 data[16];
 
 			ms_biff_put_var_next   (bp, (0x200 | BIFF_RK));
-			BIFF_SETDOUBLE (data+6-4, val);
+			gnumeric_set_le_double (data+6-4, val);
 			EX_SETROW(data, row);
 			EX_SETCOL(data, col);
 			EX_SETXF (data, xf);
@@ -2380,7 +2383,7 @@ write_formula (BiffPut *bp, ExcelSheet *sheet, const Cell *cell, gint16 xf)
 	switch (v->type) {
 	case VALUE_INTEGER :
 	case VALUE_FLOAT :
-		BIFF_SETDOUBLE (data + 6, value_get_as_float (v));
+		gnumeric_set_le_double (data + 6, value_get_as_float (v));
 		break;
 
 	case VALUE_STRING :
@@ -2434,6 +2437,57 @@ write_formula (BiffPut *bp, ExcelSheet *sheet, const Cell *cell, gint16 xf)
 	}
 }
 
+#define MAX_BIFF_NOTE_CHUNK	2048
+
+/* biff7 an earlier */
+static void
+write_biff7_comments (BiffPut *bp, ExcelSheet *sheet)
+{
+	guint8 data[6];
+	GList *l;
+	Sheet *gnum_sheet = sheet->gnum_sheet;
+	MsBiffVersion ver = sheet->wb->ver;
+
+	 for (l = gnum_sheet->comment_list; l; l = l->next) {
+		Cell *cell = l->data;
+		char *comment = cell_get_comment (cell);
+		guint16 row = cell->pos.row;
+		guint16 col = cell->pos.col;
+		guint16 len = strlen (comment);
+
+		ms_biff_put_var_next (bp, BIFF_NOTE);
+		MS_OLE_SET_GUINT16 (data + 0, row);
+		MS_OLE_SET_GUINT16 (data + 2, col);
+		MS_OLE_SET_GUINT16 (data + 4, len);
+		ms_biff_put_var_write (bp, data, 6);
+
+repeat:
+		if (len > MAX_BIFF_NOTE_CHUNK) {
+			char const tmp = comment [MAX_BIFF_NOTE_CHUNK];
+
+			comment [MAX_BIFF_NOTE_CHUNK] = '\0';
+			biff_put_text (bp, comment, ver, FALSE, AS_PER_VER);
+			comment [MAX_BIFF_NOTE_CHUNK] = tmp;
+
+			ms_biff_put_commit (bp);
+
+			comment += MAX_BIFF_NOTE_CHUNK;
+			len -= MAX_BIFF_NOTE_CHUNK;
+			
+	        	ms_biff_put_var_next (bp, BIFF_NOTE);
+		        MS_OLE_SET_GUINT16 (data + 0, 0xffff);
+	        	MS_OLE_SET_GUINT16 (data + 2, 0);
+		        MS_OLE_SET_GUINT16 (data + 4, MIN (MAX_BIFF_NOTE_CHUNK, len));
+		        ms_biff_put_var_write (bp, data, 6);
+
+			goto repeat;
+		} else {
+			biff_put_text (bp, comment, ver, FALSE, AS_PER_VER);
+			ms_biff_put_commit (bp);
+		}
+	}
+}
+
 /**
  * write_cell
  * @bp    biff buffer
@@ -2599,7 +2653,7 @@ margin_write (BiffPut *bp, guint16 op, PrintUnit *pu)
 	margin = unit_convert (pu->points, UNIT_POINTS, UNIT_INCH);
 
 	data = ms_biff_put_len_next (bp, op, 8);
-	BIFF_SETDOUBLE (data, margin);
+	gnumeric_set_le_double (data, margin);
 
 	ms_biff_put_commit (bp);
 }
@@ -2916,14 +2970,14 @@ write_sheet_bools (BiffPut *bp, ExcelSheet *sheet)
 }
 
 static void
-write_sheet_tail (BiffPut *bp, ExcelSheet *sheet)
+write_sheet_tail (IOContext *context, BiffPut *bp, ExcelSheet *sheet)
 {
 	guint8 *data;
 	MsBiffVersion ver = sheet->wb->ver;
 	guint16 options = 0x2b6; /* Arabic ? */
 
-	if (sheet->gnum_sheet == sheet->wb->gnum_wb->current_sheet)
-	    options |= 0x400;
+	if (sheet->gnum_sheet == wb_view_cur_sheet (sheet->wb->gnum_wb_view))
+		options |= 0x400;
 
 	write_window1 (bp, ver);
 
@@ -3188,7 +3242,7 @@ write_block (BiffPut *bp, ExcelSheet *sheet, guint32 begin, int nrows)
 /* See: 'Finding records in BIFF files': S59E28.HTM */
 /* and S59D99.HTM */
 static void
-write_sheet (BiffPut *bp, ExcelSheet *sheet)
+write_sheet (IOContext *context, BiffPut *bp, ExcelSheet *sheet)
 {
 	guint32 y, block_end, maxrows;
 	int rows_in_block = ROW_BLOCK_MAX_LEN;
@@ -3233,8 +3287,9 @@ write_sheet (BiffPut *bp, ExcelSheet *sheet)
 	for (y = 0; y < sheet->maxy; y = block_end + 1)
 		block_end = write_block (bp, sheet, y, rows_in_block);
 
+	write_biff7_comments (bp, sheet);
 	write_index (bp->pos, sheet, index_off);
-	write_sheet_tail (bp, sheet);
+	write_sheet_tail (context, bp, sheet);
 
 	biff_eof_write (bp);
 }
@@ -3286,8 +3341,8 @@ free_sheet (ExcelSheet *sheet)
 
 /**
  * pre_pass
- * @context: Command context.
- * @wb: the workbook to scan
+ * @context:  Command context.
+ * @wb:   The workbook to scan
  *
  * Scans all the workbook items. Adds all styles, fonts, formats and
  * colors to tables. Resolves any referencing problems before they
@@ -3295,7 +3350,7 @@ free_sheet (ExcelSheet *sheet)
  *
  **/
 static int
-pre_pass (CommandContext *context, ExcelWorkbook *wb)
+pre_pass (IOContext *context, ExcelWorkbook *wb)
 {
 	int ret = 0;
 
@@ -3338,7 +3393,7 @@ free_workbook (ExcelWorkbook *wb)
 }
 
 static void
-write_workbook (BiffPut *bp, ExcelWorkbook *wb, MsBiffVersion ver)
+write_workbook (IOContext *context, BiffPut *bp, ExcelWorkbook *wb, MsBiffVersion ver)
 {
 	ExcelSheet *s  = 0;
 	int        lp;
@@ -3370,7 +3425,7 @@ write_workbook (BiffPut *bp, ExcelWorkbook *wb, MsBiffVersion ver)
 
 	/* Sheets */
 	for (lp = 0; lp < wb->sheets->len; lp++)
-		write_sheet (bp, g_ptr_array_index (wb->sheets, lp));
+		write_sheet (context, bp, g_ptr_array_index (wb->sheets, lp));
 	/* End of Sheets */
 
 	/* Finalise Workbook stuff */
@@ -3394,7 +3449,7 @@ write_workbook (BiffPut *bp, ExcelWorkbook *wb, MsBiffVersion ver)
  * storing empty row records.
  */
 static int
-check_sheet (CommandContext *context, ExcelSheet *sheet)
+check_sheet (IOContext *context, ExcelSheet *sheet)
 {
 	guint32 maxrows;
 	int ret = 0;
@@ -3403,8 +3458,8 @@ check_sheet (CommandContext *context, ExcelSheet *sheet)
 		? MsBiffMaxRowsV8 : MsBiffMaxRowsV7;
 
 	if (sheet->maxy > maxrows) {
-		gnumeric_error_save
-			(context, _("Too many rows for this format"));
+		gnumeric_io_error_save (context,
+			_("Too many rows for this format"));
 		ret = -1;
 	}
 
@@ -3423,7 +3478,7 @@ check_sheet (CommandContext *context, ExcelSheet *sheet)
  * losing style information.
  */
 int
-ms_excel_check_write (CommandContext *context, void **state, Workbook *gwb,
+ms_excel_check_write (IOContext *context, void **state, WorkbookView *gwb_view,
 		      MsBiffVersion ver)
 {
 	int ret = 0;
@@ -3435,8 +3490,9 @@ ms_excel_check_write (CommandContext *context, void **state, Workbook *gwb,
 
 	*state = wb;
 
-	wb->ver      = ver;
-	wb->gnum_wb  = gwb;
+	wb->ver          = ver;
+	wb->gnum_wb      = wb_view_workbook (gwb_view);
+	wb->gnum_wb_view = gwb_view;
 	wb->sheets   = g_ptr_array_new ();
 	wb->names    = g_ptr_array_new ();
 	fonts_init (wb);
@@ -3444,7 +3500,7 @@ ms_excel_check_write (CommandContext *context, void **state, Workbook *gwb,
 	palette_init (wb);
 	xf_init (wb);
 
-	sheets = workbook_sheets (gwb);
+	sheets = workbook_sheets (wb->gnum_wb);
 	while (sheets) {
 		ExcelSheet *sheet = new_sheet (wb, sheets->data);
 		g_ptr_array_add (wb->sheets, sheet);
@@ -3466,7 +3522,7 @@ cleanup:
 }
 
 int
-ms_excel_write_workbook (CommandContext *context, MsOle *file, void *state,
+ms_excel_write_workbook (IOContext *context, MsOle *file, void *state,
 			 MsBiffVersion ver)
 {
 	MsOleErr     result;
@@ -3486,14 +3542,14 @@ ms_excel_write_workbook (CommandContext *context, MsOle *file, void *state,
 
 	if (result != MS_OLE_ERR_OK) {
 		free_workbook (wb);
-		gnumeric_error_save
-			(context, _("Can't open stream for writing\n"));
+		gnumeric_io_error_save (context,
+			_("Can't open stream for writing\n"));
 		return -1;
 	}
 
 	bp = ms_biff_put_new (str);
 
-	write_workbook (bp, wb, ver);
+	write_workbook (context, bp, wb, ver);
 	free_workbook (wb);
 
 	/* Kludge to make sure the file is a Big Block file */
