@@ -1,7 +1,7 @@
 /*
  * latex.c
  *
- * Copyright (C) 1999 Rasca, Berlin
+ * Copyright (C) 1999, 2000 Rasca, Berlin
  * EMail: thron@gmx.de
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,13 +22,14 @@
 #include <errno.h>
 #include <gnome.h>
 #include "config.h"
+#include "io-context.h"
+#include "workbook-view.h"
 #include "workbook.h"
 #include "sheet.h"
 #include "style.h"
 #include "latex.h"
 #include "font.h"
 #include "cell.h"
-#include "command-context.h"
 #include "rendered-value.h"
 
 /*
@@ -81,21 +82,21 @@ latex_fprintf_cell (FILE *fp, const Cell *cell)
  * write every sheet of the workbook to a latex table
  */
 int
-html_write_wb_latex (CommandContext *context, Workbook *wb,
+html_write_wb_latex (IOContext *context, WorkbookView *wb_view,
 		     const char *filename)
 {
 	FILE *fp;
 	GList *sheet_list;
-	Sheet *sheet;
 	Cell *cell;
 	int row, col;
+	Workbook *wb = wb_view_workbook (wb_view);
 
 	g_return_val_if_fail (wb != NULL, -1);
 	g_return_val_if_fail (filename != NULL, -1);
 
 	fp = fopen (filename, "w");
 	if (!fp) {
-		gnumeric_error_save (context, g_strerror (errno));
+		gnumeric_io_error_system (context, g_strerror (errno));
 		return -1;
 	}
 
@@ -105,7 +106,9 @@ html_write_wb_latex (CommandContext *context, Workbook *wb,
 	fprintf (fp, "\\begin{document}\n\n");
 	sheet_list = workbook_sheets (wb);
 	while (sheet_list) {
-		sheet = sheet_list->data;
+		Sheet *sheet = sheet_list->data;
+		Range r = sheet_get_extent (sheet);
+		
 		latex_fputs (sheet->name_unquoted, fp);
 		fprintf (fp, "\n\n");
 		fprintf (fp, "\\begin{tabular}{|");
@@ -114,8 +117,8 @@ html_write_wb_latex (CommandContext *context, Workbook *wb,
 		}
 		fprintf (fp, "}\\hline\n");
 
-		for (row = 0; row <= sheet->rows.max_used; row++) {
-			for (col = 0; col <= sheet->cols.max_used; col++) {
+		for (row = r.start.row; row <= r.end.row; row++) {
+			for (col = r.start.col; col <= r.end.col; col++) {
 				cell = sheet_cell_get (sheet, col, row);
 				if (!cell) {
 					if (col)
@@ -177,22 +180,22 @@ html_write_wb_latex (CommandContext *context, Workbook *wb,
  * write every sheet of the workbook to a latex2e table
  */
 int
-html_write_wb_latex2e (CommandContext *context, Workbook *wb,
+html_write_wb_latex2e (IOContext *context, WorkbookView *wb_view,
 		       const char *filename)
 {
 	FILE *fp;
 	GList *sheet_list;
-	Sheet *sheet;
 	Cell *cell;
 	int row, col;
 	unsigned char r,g,b;
+	Workbook *wb = wb_view_workbook (wb_view);
 
 	g_return_val_if_fail (wb != NULL, -1);
 	g_return_val_if_fail (filename != NULL, -1);
 
 	fp = fopen (filename, "w");
 	if (!fp) {
-		gnumeric_error_save (context, g_strerror (errno));
+		gnumeric_io_error_system (context, g_strerror (errno));
 		return -1;
 	}
 
@@ -204,7 +207,9 @@ html_write_wb_latex2e (CommandContext *context, Workbook *wb,
 	fprintf (fp, "\\begin{document}\n\n");
 	sheet_list = workbook_sheets (wb);
 	while (sheet_list) {
-		sheet = sheet_list->data;
+		Sheet *sheet = sheet_list->data;
+		Range range = sheet_get_extent (sheet);
+		
 		latex_fputs (sheet->name_unquoted, fp);
 		fprintf (fp, "\n\n");
 		fprintf (fp, "\\begin{tabular}{|");
@@ -213,8 +218,8 @@ html_write_wb_latex2e (CommandContext *context, Workbook *wb,
 		}
 		fprintf (fp, "}\\hline\n");
 
-		for (row = 0; row <= sheet->rows.max_used; row++) {
-			for (col = 0; col <= sheet->cols.max_used; col++) {
+		for (row = range.start.row; row <= range.end.row; row++) {
+			for (col = range.start.col; col <= range.end.col; col++) {
 				cell = sheet_cell_get (sheet, col, row);
 				if (!cell) {
 					if (col)
