@@ -10,10 +10,12 @@
 #include "application.h"
 #include "clipboard.h"
 #include "selection.h"
-#include "sheet.h"
-#include "workbook.h"
+#include "workbook-control.h"
 #include "workbook-view.h"
+#include "workbook.h"
+#include "sheet.h"
 
+#include "pixmaps/menu-print-preview.xpm"
 #include "pixmaps/print-preview.xpm"
 #include "pixmaps/sort-ascending.xpm"
 #include "pixmaps/sort-descending.xpm"
@@ -75,6 +77,7 @@ application_init (void)
 		char const * const name;
 		gchar **xpm_data;
 	} const entry_names [] = {
+		{ 16, 16, "Menu_Gnumeric_PrintPreview", menu_print_preview_xpm },
 		{ 24, 24, "Gnumeric_PrintPreview", print_preview_xpm },
 		{ 24, 24, "Gnumeric_SortAscending", sort_ascending_xpm },
 		{ 24, 24, "Gnumeric_SortDescending", sort_descending_xpm },
@@ -143,6 +146,23 @@ application_init (void)
 	gnome_config_pop_prefix ();
 }
 
+static GList *workbook_list = NULL;
+void
+application_workbook_list_add (Workbook *wb)
+{
+	workbook_list = g_list_prepend (workbook_list, wb);
+}
+void
+application_workbook_list_remove (Workbook *wb)
+{
+	workbook_list = g_list_remove (workbook_list, wb);
+}
+GList *
+application_workbook_list (void)
+{
+	return workbook_list;
+}
+
 /**
  * application_clipboard_clear:
  *
@@ -160,7 +180,8 @@ application_clipboard_clear (gboolean drop_selection)
 		Sheet *sheet = app.clipboard_sheet;
 
 		sheet_selection_unant (sheet);
-		workbook_view_set_paste_special_state (sheet->workbook, FALSE);
+		WORKBOOK_FOREACH_CONTROL (sheet->workbook, view, control,
+			wb_control_paste_special_enable (control, FALSE););
 		app.clipboard_sheet = NULL;
 
 		/* Release the selection */
@@ -179,15 +200,14 @@ application_clipboard_unant (void)
 }
 
 static gboolean
-application_set_selected_sheet (Sheet *sheet)
+application_set_selected_sheet (WorkbookControl *wbc, Sheet *sheet)
 {
 	g_return_val_if_fail (sheet != NULL, FALSE);
 
 	application_clipboard_clear (FALSE);
 
-	if (gtk_selection_owner_set (workbook_get_toplevel (sheet->workbook),
-				     GDK_SELECTION_PRIMARY,
-				     GDK_CURRENT_TIME)) {
+#warning how to do this on a per display basis ?
+	if (wb_control_claim_selection (wbc)) {
 		app.clipboard_sheet = sheet;
 		return TRUE;
 	}
@@ -200,6 +220,7 @@ application_set_selected_sheet (Sheet *sheet)
 /**
  * application_clipboard_copy:
  *
+ * @wbc   : the workbook control that requested the operation.
  * @sheet : The source sheet for the copy.
  * @area  : A single rectangular range to be copied.
  *
@@ -207,17 +228,19 @@ application_set_selected_sheet (Sheet *sheet)
  * into the clipboard.
  */
 void
-application_clipboard_copy (Sheet *sheet, Range const *area)
+application_clipboard_copy (WorkbookControl *wbc,
+			    Sheet *sheet, Range const *area)
 {
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (area != NULL);
 
-	if (application_set_selected_sheet (sheet) ) {
+	if (application_set_selected_sheet (wbc, sheet) ) {
 		app.clipboard_cut_range = *area;
 		app.clipboard_copied_contents = 
 			clipboard_copy_range (sheet, area);
 
-		workbook_view_set_paste_special_state (sheet->workbook, TRUE);
+		WORKBOOK_FOREACH_CONTROL (sheet->workbook, view, control,
+			wb_control_paste_special_enable (control, TRUE););
 
 		sheet_selection_ant (sheet);
 	}
@@ -226,6 +249,7 @@ application_clipboard_copy (Sheet *sheet, Range const *area)
 /**
  * application_clipboard_cut:
  *
+ * @wbc   : the workbook control that requested the operation.
  * @sheet : The source sheet for the copy.
  * @area  : A single rectangular range to be cut.
  *
@@ -234,16 +258,18 @@ application_clipboard_copy (Sheet *sheet, Range const *area)
  * cut operation.
  */
 void
-application_clipboard_cut (Sheet *sheet, Range const *area)
+application_clipboard_cut (WorkbookControl *wbc,
+			   Sheet *sheet, Range const *area)
 {
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (area != NULL);
 
-	if (application_set_selected_sheet (sheet) ) {
+	if (application_set_selected_sheet (wbc, sheet) ) {
 		app.clipboard_cut_range = *area;
 
 		/* No paste special for copies */
-		workbook_view_set_paste_special_state (sheet->workbook, FALSE);
+		WORKBOOK_FOREACH_CONTROL (sheet->workbook, view, control,
+			wb_control_paste_special_enable (control, FALSE););
 
 		sheet_selection_ant (sheet);
 	}
@@ -303,11 +329,25 @@ application_workbook_get_by_name (char const * const name)
 	struct wb_name_closure close;
 	close.wb = NULL;
 	close.name = name;
-	workbook_foreach (&cb_workbook_name, &close);
+	application_workbook_foreach (&cb_workbook_name, &close);
 
 	return close.wb;
 }
 
+gboolean
+application_workbook_foreach (WorkbookCallback cback, gpointer data)
+{
+	GList *l;
+
+	for (l = workbook_list; l; l = l->next){
+		Workbook *wb = l->data;
+
+		if (!(*cback)(wb, data))
+			return FALSE;
+	}
+	return TRUE;
+}
+
 struct wb_index_closure
 {
 	Workbook *wb;
@@ -326,7 +366,7 @@ application_workbook_get_by_index (int i)
 	struct wb_index_closure close;
 	close.wb = NULL;
 	close.index = i;
-	workbook_foreach (&cb_workbook_index, &close);
+	application_workbook_foreach (&cb_workbook_index, &close);
 
 	return close.wb;
 }
