@@ -36,7 +36,13 @@ draw_overflow (GdkDrawable *drawable, GdkGC *gc, GdkFont *font,
 	       int const * const line_offset, int num_lines)
 {
 	int const len = gdk_string_width (font, "#");
-	int count = (len != 0) ? (width / len) : 0;
+	int count = 0;
+
+	if (len != 0)  {
+		count = width / len;
+		if (count == 0)
+			count = 1;
+	}
 
 	/* Center */
 	for (x1 += (width - count*len) / 2; --count >= 0 ; x1 += len )
@@ -161,10 +167,8 @@ cell_draw (Cell const *cell, MStyle *mstyle, CellSpanInfo const * const spaninfo
 	 * a long cells contents extending past the edge of the edit box.
 	 * Don't print zeros if they should be ignored.
 	 */
-	if (sheet && (cell == sheet->workbook->editing_cell ||
-		      (!sheet->display_formulas &&
-		       !sheet->display_zero &&
-		       cell_is_zero (cell))))
+	if (sheet &&
+	    (!sheet->display_formulas && !sheet->display_zero && cell_is_zero (cell)))
 		return;
 
 	g_return_if_fail (cell->rendered_value);
@@ -222,7 +226,7 @@ cell_draw (Cell const *cell, MStyle *mstyle, CellSpanInfo const * const spaninfo
 		break;
 	}
 	
-	halign = value_get_default_halign (cell->value, mstyle);
+	halign = cell_default_halign (cell, mstyle);
 
 	is_single_line = (halign != HALIGN_JUSTIFY &&
 			  valign != VALIGN_JUSTIFY &&
@@ -292,7 +296,8 @@ cell_draw (Cell const *cell, MStyle *mstyle, CellSpanInfo const * const spaninfo
 	cell_width_pixel = cell_rendered_width (cell);
 
 	/* if a number overflows, do special drawing */
-	if (width < cell_width_pixel && cell_is_number (cell)) {
+	if (width < cell_width_pixel && cell_is_number (cell) &&
+	    sheet && !sheet->display_formulas) {
 		draw_overflow (drawable, gc, font,
 			       x1 + ci->margin_a + 1,
 			       text_base, width, line_offset, num_lines);
