@@ -2,6 +2,7 @@
  * cell.c: Cell management of the Gnumeric spreadsheet.
  *
  * Author:
+ *    Jdy Goldberg 2000 (jgoldberg@home.com)
  *    Miguel de Icaza 1998, 1999 (miguel@kernel.org)
  */
 #include "config.h"
@@ -11,6 +12,7 @@
 #include "cell-comment.h"
 #include "expr.h"
 #include "eval.h"
+#include "value.h"
 #include "style.h"
 #include "sheet.h"
 #include "value.h"
@@ -42,8 +44,8 @@ cell_dirty (Cell *cell)
  *      Empty a cell's
  *      	- value.
  *      	- rendered_value.
- *      	- entered_text/expression.
- *      	- optional format.
+ *      	- expression.
+ *      	- parse format.
  *     
  *      Clears the flags to
  *      	- not queued for recalc.
@@ -65,9 +67,6 @@ cell_cleanout (Cell *cell)
 			dependent_unlink (CELL_TO_DEP (cell), &cell->pos);
 		expr_tree_unref (cell->base.expression);
 		cell->base.expression = NULL;
-	} else if (cell->entered_text) {
-		string_unref (cell->entered_text);
-		cell->entered_text = NULL;
 	}
 
 	if (cell->value) {
@@ -113,8 +112,6 @@ cell_copy (Cell const *cell)
 	/* now copy properly the rest */
 	if (cell_has_expr (new_cell))
 		expr_tree_ref (new_cell->base.expression);
-	else
-		string_ref (new_cell->entered_text);
 
 	new_cell->rendered_value = NULL;
 
@@ -195,7 +192,10 @@ cell_eval_content (Cell *cell)
 		v = value_new_error (&pos, "Internal error");
 
 	cell_assign_value (cell, v, NULL);
-	rendered_value_calc_size (cell);
+
+	/* TODO : Can we Use spancalc without too much performance overhead
+	 * Seems like a better approach may be to do this at display time */
+	cell_render_value (cell, TRUE);
 	sheet_redraw_cell (cell);
 }
 
@@ -281,12 +281,12 @@ cell_relocate (Cell *cell, ExprRewriteInfo *rwinfo)
 /****************************************************************************/
 
 /*
- * cell_set_text : Stores the supplied text as the entered_text, then parses it
- *      for storage as a value or expression.  It marks the sheet as dirty.
+ * cell_set_text : Parses the supplied text for storage as a value or
+ * 		expression.  It marks the sheet as dirty.
  *
  * If the text is an expression it IS queued for recalc.
  *        the format prefered by the expression is stored for later use.
- * If the text is a value it is rendered but spans are not calculated.
+ * If the text is a value it is NOT rendered and spans are NOT calculated.
  *        the format that matched the text is stored for later use.
  *
  * WARNING : This is an internal routine that does not queue redraws,
@@ -315,9 +315,8 @@ cell_set_text (Cell *cell, char const *text)
 
 		cell->base.flags &= ~CELL_HAS_EXPRESSION;
 		cell->value = val;
-		cell->entered_text = string_get (text);
 		cell->format = format;
-		cell_render_value (cell);
+		cell_render_value (cell, TRUE);
 	} else {		/* String was an expression */
 		cell_set_expr (cell, expr, format);
 		if (format) style_format_unref (format);
@@ -326,40 +325,6 @@ cell_set_text (Cell *cell, char const *text)
 	cell_dirty (cell);
 }
 
-/*
- * cell_set_text_and_value : Stores the supplied text as the entered_text, then
- *      stores (WITHOUT COPYING) the supplied value.
- *      It marks the sheet as dirty. The text is NOT parsed.
- *
- * The cell is rendered but spans are not calculated.
- * If an optional format is supplied it is stored for later use.
- *
- * WARNING : This is an internal routine that does not queue redraws,
- *           does not auto-resize, and does not calculate spans.
- *
- * NOTE : This DOES check for array partitioning.
- */
-void
-cell_set_text_and_value (Cell *cell, String *text,
-			 Value *v, StyleFormat *opt_fmt)
-{
-	g_return_if_fail (cell);
-	g_return_if_fail (text);
-	g_return_if_fail (v);
-	g_return_if_fail (!cell_is_partial_array (cell));
-
-	if (opt_fmt != NULL)
-		style_format_ref (opt_fmt);
-
-	cell_dirty (cell);
-	cell_cleanout (cell);
-
-	cell->format = opt_fmt;
-	cell->entered_text = string_ref (text);
-	cell->value = v;
-	cell_render_value (cell);
-}
-
 /*
  * cell_assign_value : Stores (WITHOUT COPYING) the supplied value.
  *    no changes are made to the expression or entered text.  This
@@ -391,7 +356,7 @@ cell_assign_value (Cell *cell, Value *v, StyleFormat *opt_fmt)
 	if (cell->value != NULL)
 		value_release (cell->value);
 	cell->value = v;
-	cell_render_value (cell);
+	cell_render_value (cell, TRUE);
 }
 
 /*
@@ -426,22 +391,10 @@ cell_set_value (Cell *cell, Value *v, StyleFormat *opt_fmt)
 	cell_dirty (cell);
 	cell_cleanout (cell);
 
-	cell->format = opt_fmt;
+	/* TODO : It would be nice to standardize on NULL == General */
+	cell->format = (opt_fmt == NULL || style_format_is_general (opt_fmt))
+		? NULL : opt_fmt;
 	cell->value = v;
-	cell_render_value (cell);
-
-	/* Be careful that a value passes as a string stays a string */
-	if (v->type == VALUE_STRING) {
-		/* TODO : add new string routine to avoid the extra copy */
-		char *tmp = g_strconcat ("\'", v->v_str.val->str, NULL);
-		cell->entered_text = string_get (tmp);
-		g_free (tmp);
-	} else if (opt_fmt) {
-		/* If available use the supplied format */
-		cell->entered_text = string_get (format_value (opt_fmt, v, NULL, NULL));
-	} else
-		/* Fall back on using the format applied to the cell */
-		cell->entered_text = string_ref (cell->rendered_value->rendered_text);
 }
 
 /*
@@ -452,16 +405,15 @@ cell_set_value (Cell *cell, Value *v, StyleFormat *opt_fmt)
  *
  * If an optional format is supplied it is stored for later use.
  *
- * The cell is rendered but spans are not calculated,  the cell is NOT marked for
- * recalc.
- *
  * WARNING : This is an internal routine that does not queue redraws,
- *           does not auto-resize, and does not calculate spans.
+ *           does not auto-resize, does not calculate spans, and does
+ *           not render the value.
  *
  * NOTE : This DOES check for array partitioning.
  */
 void
-cell_set_expr_and_value (Cell *cell, ExprTree *expr, Value *v)
+cell_set_expr_and_value (Cell *cell, ExprTree *expr, Value *v,
+			 StyleFormat *opt_fmt)
 {
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (expr != NULL);
@@ -469,19 +421,17 @@ cell_set_expr_and_value (Cell *cell, ExprTree *expr, Value *v)
 
 	/* Repeat after me.  Ref before unref. */
 	expr_tree_ref (expr);
+	if (opt_fmt != NULL)
+		style_format_ref (opt_fmt);
 
 	cell_dirty (cell);
 	cell_cleanout (cell);
 
+	cell->format = opt_fmt;
 	cell->base.expression = expr;
 	cell->base.flags |= CELL_HAS_EXPRESSION;
 	dependent_link (CELL_TO_DEP (cell), &cell->pos);
-#if 0
-	/* TODO : Should we add this for consistancy ? */
-	cell->format = fmt;
-#endif
 	cell->value = v;
-	cell_render_value (cell);
 }
 
 /**
@@ -500,17 +450,13 @@ cell_set_expr_and_value (Cell *cell, ExprTree *expr, Value *v)
 static void
 cell_set_expr_internal (Cell *cell, ExprTree *expr, StyleFormat *opt_fmt)
 {
+	expr_tree_ref (expr);
 	if (opt_fmt != NULL)
 		style_format_ref (opt_fmt);
 
-	expr_tree_ref (expr);
-
 	cell_dirty (cell);
 	cell_cleanout (cell);
 
-	if (cell->base.expression)
-		expr_tree_unref (cell->base.expression);
-
 	cell->format = opt_fmt;
 	cell->base.expression = expr;
 	cell->base.flags |= CELL_HAS_EXPRESSION;
@@ -687,11 +633,29 @@ cell_is_partial_array (Cell const *cell)
 	return ref != NULL && (ref->cols > 1 || ref->rows > 1);
 }
 
+StyleHAlignFlags
+cell_default_halign (Cell const *c, MStyle const *mstyle)
+{
+	StyleHAlignFlags align = mstyle_get_align_h (mstyle);
+
+	if (align == HALIGN_GENERAL) {
+		g_return_val_if_fail (c != NULL, HALIGN_RIGHT);
+
+		if (cell_is_number (c) &&
+		    c->base.sheet && !c->base.sheet->display_formulas)
+			return HALIGN_RIGHT;
+		return HALIGN_LEFT;
+	}
+
+	return align;
+}
+
 /***************************************************************************/
 
-/*
- * cell_render_value
+/**
+ * cell_render_value :
  * @cell: The cell whose value needs to be rendered
+ * @dynamic_width : Allow format to depend on column width.
  *
  * TODO :
  * There is no reason currently for this to allocate the rendered value as
@@ -700,15 +664,22 @@ cell_is_partial_array (Cell const *cell)
  * are shared.
  */
 void
-cell_render_value (Cell *cell)
+cell_render_value (Cell *cell, gboolean dynamic_width)
 {
 	RenderedValue *rv;
+	MStyle *mstyle;
+		
 	g_return_if_fail (cell != NULL);
 
-	rv = rendered_value_new (cell, NULL);
+	mstyle = cell_get_mstyle (cell);
+
+	rv = rendered_value_new (cell, mstyle, dynamic_width);
 	if (cell->rendered_value)
 		rendered_value_destroy (cell->rendered_value);
 	cell->rendered_value = rv;
+
+	rendered_value_calc_size_ext (cell, mstyle);
+	mstyle_unref (mstyle);
 }
 
 
@@ -805,15 +776,16 @@ cell_make_value (Cell *cell)
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (cell_has_expr(cell));
 
+	/* Clipboard cells, e.g., are not attached to a sheet.  */
+	if (cell_expr_is_linked (cell))
+		dependent_unlink (CELL_TO_DEP (cell), &cell->pos);
+
 	expr_tree_unref (cell->base.expression);
 	cell->base.expression = NULL;
 	cell->base.flags &= ~CELL_HAS_EXPRESSION;
 
 	if (cell->rendered_value == NULL)
-		cell_render_value (cell);
+		cell_render_value (cell, TRUE);
 
-	g_return_if_fail (cell->rendered_value != NULL);
-
-	cell->entered_text = string_ref (cell->rendered_value->rendered_text);
 	cell_dirty (cell);
 }
