@@ -30,9 +30,4 @@ gnumeric_arg_parse (int argc, char *argv [])
 
 	if (bonobo_init (orb, NULL, NULL) == FALSE)
 		g_error ("Failure starting up Bonobo");
-
-	if (!WorkbookFactory_init ())
-		g_warning (_("Could not initialize the Gnumeric Workbook factory"));
-
-	EmbeddableGridFactory_init ();
 }
