@@ -42,8 +42,10 @@
 #include "format-template.h"
 #include "file-autoft.h"
 
-#include "command-context.h"
+#include "workbook-control.h"
 #include "workbook.h"
+#include "commands.h"
+#include "selection.h"
 
 #define GLADE_FILE "autoformat.glade"
 
@@ -68,7 +70,8 @@ static char* demotable[PREVIEW_ROWS][PREVIEW_COLS] = {
 };
 
 typedef struct {
-	Workbook       *wb;                               /* Workbook we are working on */
+	Workbook        *wb;                              /* Workbook we are working on */
+	WorkbookControlGUI *wbcg;                             
 	PreviewGridController *controller[NUM_PREVIEWS];  /* Controller for each canvas */
 	GSList         *templates;                        /* List of FormatTemplate's */
 	FormatTemplate *selected_template;                /* The currently selected template */
@@ -282,11 +285,11 @@ templates_load (AutoFormatInfo *info)
 		if (strcmp (info->current_category, category) == 0) {
 			FormatTemplate *ft;
 
-			ft = format_template_new_from_file (workbook_command_context_gui (info->wb),
-							    filename);
+			ft = format_template_new_from_file (
+				WORKBOOK_CONTROL (info->wbcg), filename);
 								    
 			format_template_set_size (ft, 0, 0, PREVIEW_COLS - 1, PREVIEW_ROWS - 1);
-				
+
 			if (ft == NULL) {
 				
 				g_warning ("Error while reading %s", filename);
@@ -707,6 +710,37 @@ cb_canvas_button_release (GnomeCanvas *canvas, GdkEventButton *event, AutoFormat
 	return TRUE;
 }
 
+/**
+ * cb_apply_item_toggled:
+ * @item:
+ * @info: 
+ * 
+ * This callback is invoked when one of the (6) apply items is toggled.
+ * It will change the filter for each FormatTemplate. This way certain elements
+ * can be filtered out (like the background, border, etc..)
+ **/
+static void
+cb_apply_item_toggled (GtkCheckMenuItem *item, AutoFormatInfo *info)
+{
+	GSList *iterator = info->templates;
+	int i;
+
+	while (iterator) {
+		format_template_set_filter ((FormatTemplate *) iterator->data,
+					    info->number->active,
+					    info->border->active,
+					    info->font->active,
+					    info->patterns->active,
+					    info->alignment->active);
+
+		iterator = g_slist_next (iterator);
+	}
+
+	for (i = 0; i < NUM_PREVIEWS; i++)
+		if (info->controller[i])
+			preview_grid_controller_force_redraw (info->controller[i]);
+}
+
 /**
  * cb_category_popwin_hide:
  * @widget:
@@ -734,6 +768,11 @@ cb_category_popwin_hide (GtkWidget *widget, AutoFormatInfo *info)
 		
 	previews_load (info, 0);
 
+	/*
+	 * Apply filter
+	 */
+	cb_apply_item_toggled (NULL, info);
+	
 	/*
 	 * This is for the initial selection
 	 */
@@ -767,37 +806,6 @@ cb_category_popwin_hide (GtkWidget *widget, AutoFormatInfo *info)
 	gtk_widget_set_sensitive (GTK_WIDGET (info->new), FALSE);
 }
 
-/**
- * cb_apply_item_toggled:
- * @item:
- * @info: 
- * 
- * This callback is invoked when one of the (6) apply items is toggled.
- * It will change the filter for each FormatTemplate. This way certain elements
- * can be filtered out (like the background, border, etc..)
- **/
-static void
-cb_apply_item_toggled (GtkCheckMenuItem *item, AutoFormatInfo *info)
-{
-	GSList *iterator = info->templates;
-	int i;
-
-	while (iterator) {
-		format_template_set_filter ((FormatTemplate *) iterator->data,
-					    info->number->active,
-					    info->border->active,
-					    info->font->active,
-					    info->patterns->active,
-					    info->alignment->active);
-
-		iterator = g_slist_next (iterator);
-	}
-
-	for (i = 0; i < NUM_PREVIEWS; i++)
-		if (info->controller[i])
-			preview_grid_controller_force_redraw (info->controller[i]);
-}
-
 static void
 cb_gridlines_item_toggled (GtkCheckMenuItem *item, AutoFormatInfo *info)
 {
@@ -843,7 +851,7 @@ setup_apply_item (GladeXML *gui, AutoFormatInfo *info, char *name)
  * sheet of the workbook if the user desires.
  **/
 void
-dialog_autoformat (Workbook *wb)
+dialog_autoformat (WorkbookControlGUI *wbcg)
 {
 	GladeXML *gui;
 	AutoFormatInfo *info;
@@ -870,7 +878,8 @@ dialog_autoformat (Workbook *wb)
 	 */
 	info = g_new0 (AutoFormatInfo, 1);
 
-	info->wb         = wb;
+	info->wb         = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	info->wbcg        = wbcg;
 	info->templates  = NULL;
 	info->categories = NULL;
 	for (i = 0; i < NUM_PREVIEWS; i++) {
@@ -1018,14 +1027,20 @@ dialog_autoformat (Workbook *wb)
 	 */
 	info->canceled = FALSE;
 
-	gnumeric_dialog_run (wb, info->dialog);	
+	gnumeric_dialog_run (wbcg, info->dialog);	
 
 	/*
-	 * If the user didn't cancel, apply the autoformat
-	 * template to the current selection
+	 * If the user didn't cancel, record undo information and apply the autoformat
+	 * template to the current selection.
+	 * Observe that we pass a copy of the selected template to the undo function.
 	 */
-	if (!info->canceled && info->selected_template)
-		format_template_apply_to_sheet_selection (info->selected_template, wb->current_sheet);
+	if (!info->canceled && info->selected_template) {
+		WorkbookControl *wbc = WORKBOOK_CONTROL (wbcg);
+		Sheet *sheet = wb_control_cur_sheet (wbc);
+
+		cmd_autoformat (wbc, sheet,
+			format_template_clone (info->selected_template));
+	}
 
 	/*
 	 * Free templates, previews, etc..
