@@ -87,12 +87,12 @@ typedef struct
 typedef struct _FormatState
 {
 	GladeXML	*gui;
+	WorkbookControlGUI	*wbcg;
 	GnomePropertyBox*dialog;
 
 	Sheet		*sheet;
 	MStyle		*style, *result;
 	Value		*value;
-	char const	*sample_rendered_text;
 
 	int	 	 selection_mask;
 	gboolean	 enable_edit;
@@ -102,7 +102,7 @@ typedef struct _FormatState
 		GtkBox		*box;
 		GtkWidget	*widget[F_MAX_WIDGET];
 
-		gchar const	*spec;
+		gchar const	*spec;	/* in internal (not localized) form */
 		gint		 current_type;
 		int		 num_decimals;
 		int		 negative_format;
@@ -154,7 +154,8 @@ fmt_dialog_changed (FormatState *state)
 
 /* Default to the 'Format' page but remember which page we were on between
  * invocations */
-static int fmt_dialog_page = 0;
+static FormatDialogPosition_t fmt_dialog_page = FD_NUMBER;
+
 
 /*
  * Callback routine to help remember which format tab was selected
@@ -491,9 +492,8 @@ draw_format_preview (FormatState *state)
 	if (sf == NULL)
 		return;
 
-	if (state->sample_rendered_text) {
-		preview = format_value (sf, state->value, &preview_color,
-					state->sample_rendered_text);
+	if (state->value) {
+		preview = format_value (sf, state->value, &preview_color, -1);
 
 		if (strlen (preview) > FORMAT_PREVIEW_MAX)
 			strcpy (&preview [FORMAT_PREVIEW_MAX - 5], " ...");
@@ -744,7 +744,7 @@ cb_format_entry (GtkEditable *w, FormatState *state)
 	if (state->enable_edit) {
 		g_free ((char *)state->format.spec);
 		state->format.spec = fmt;
-		mstyle_set_format_text (state->result, state->format.spec);
+		mstyle_set_format_text (state->result, fmt);
 		fmt_dialog_changed (state);
 		draw_format_preview (state);
 	} else
@@ -843,7 +843,7 @@ fmt_dialog_init_format_page (FormatState *state)
 	char const * format;
 	if (!mstyle_is_element_conflict (state->style, MSTYLE_FORMAT)) {
 		StyleFormat const *fmt = mstyle_get_format (state->style);
-		format = style_format_as_XL (fmt, TRUE);
+		format = style_format_as_XL (fmt, FALSE);
 	} else
 		format = g_strdup (cell_formats [0][0]);
 
@@ -1923,7 +1923,7 @@ cb_fmt_dialog_dialog_apply (GtkObject *w, int page, FormatState *state)
 	for (i = STYLE_BORDER_TOP; i < STYLE_BORDER_EDGE_MAX; i++)
 		borders [i] = border_get_mstyle (state, i);
 
-	cmd_format (workbook_command_context_gui (state->sheet->workbook),
+	cmd_format (WORKBOOK_CONTROL (state->wbcg),
 		    state->sheet, state->result, borders);
 
 	mstyle_unref (state->result);
@@ -1938,10 +1938,6 @@ static gboolean
 cb_fmt_dialog_dialog_destroy (GtkObject *w, FormatState *state)
 {
 	g_free ((char *)state->format.spec);
-	if (state->sample_rendered_text != NULL) {
-		g_free ((char *)state->sample_rendered_text);
-		state->sample_rendered_text = NULL;
-	}
 	mstyle_unref (state->style);
 	mstyle_unref (state->result);
 	gtk_object_unref (GTK_OBJECT (state->gui));
@@ -1988,7 +1984,8 @@ static void set_initial_focus (FormatState *state)
 }
 
 static void
-fmt_dialog_impl (FormatState *state, MStyleBorder **borders)
+fmt_dialog_impl (FormatState *state, MStyleBorder **borders,
+		 FormatDialogPosition_t pageno)
 {
 	static GnomeHelpMenuEntry help_ref = { "gnumeric", "formatting.html" };
 	static struct {
@@ -2087,7 +2084,12 @@ fmt_dialog_impl (FormatState *state, MStyleBorder **borders)
 	state->back.pattern_item	= NULL;
 	state->back.pattern.cur_index	= 0;
 
-	/* Select the same page the last invocation used */
+	/* Use same page as last invocation used unless pageno >= 0 and
+	 * sensible */
+	if (pageno >= 0 && pageno <= FD_LAST)
+		fmt_dialog_page = pageno;
+	
+	/* Select the wanted page the last invocation used */
 	gtk_notebook_set_page (
 		GTK_NOTEBOOK (GNOME_PROPERTY_BOX (dialog)->notebook),
 		fmt_dialog_page);
@@ -2235,10 +2237,8 @@ fmt_dialog_impl (FormatState *state, MStyleBorder **borders)
 	/* Make it modal */
 	gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
 
-
 	/* Bring up the dialog */
-	gnumeric_dialog_show (state->sheet->workbook->toplevel,
-			      GNOME_DIALOG (dialog), FALSE, TRUE);
+	gnumeric_dialog_show (state->wbcg, GNOME_DIALOG (dialog), FALSE, TRUE);
 }
 
 static gboolean
@@ -2255,7 +2255,7 @@ fmt_dialog_selection_type (Sheet *sheet,
 }
 
 void
-dialog_cell_format (Workbook *wb, Sheet *sheet)
+dialog_cell_format (WorkbookControlGUI *wbcg, Sheet *sheet, FormatDialogPosition_t pageno)
 {
 	GladeXML     *gui;
 	MStyle       *mstyle;
@@ -2264,12 +2264,11 @@ dialog_cell_format (Workbook *wb, Sheet *sheet)
 	MStyleBorder *borders[STYLE_BORDER_EDGE_MAX];
 	FormatState  *state;
 
-	g_return_if_fail (wb != NULL);
+	g_return_if_fail (wbcg != NULL);
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet));
 
-	gui = gnumeric_glade_xml_new (workbook_command_context_gui (wb),
-				      "cell-format.glade");
+	gui = gnumeric_glade_xml_new (wbcg, "cell-format.glade");
         if (gui == NULL)
                 return;
 
@@ -2282,12 +2281,10 @@ dialog_cell_format (Workbook *wb, Sheet *sheet)
 
 	/* Initialize */
 	state = g_new (FormatState, 1);
+	state->wbcg		= wbcg;
 	state->gui		= gui;
 	state->sheet		= sheet;
 	state->value		= sample_val;
-	state->sample_rendered_text = edit_cell
-	    ? cell_get_rendered_text (edit_cell)
-	    : NULL;
 	state->style		= mstyle;
 	state->result		= mstyle_new ();
 	state->selection_mask	= 0;
@@ -2297,7 +2294,7 @@ dialog_cell_format (Workbook *wb, Sheet *sheet)
 					&state->selection_mask);
 	state->selection_mask	= 1 << state->selection_mask;
 
-	fmt_dialog_impl (state, borders);
+	fmt_dialog_impl (state, borders, pageno);
 }
 
 /*
