@@ -12,7 +12,7 @@
 #include "gnumeric-util.h"
 #include "selection.h"
 #include "dialogs.h"
-#include "workbook-view.h"
+#include "workbook-control.h"
 #include "workbook.h"
 #include "sheet.h"
 #include "commands.h"
@@ -21,9 +21,9 @@
 #define GLADE_FILE "delete-cells.glade"
 
 static void
-dialog_delete_cells_impl (Workbook *wb, Sheet *sheet, GladeXML  *gui)
+dialog_delete_cells_impl (WorkbookControlGUI *wbcg, Sheet *sheet, GladeXML  *gui)
 {
-
+	WorkbookControl *wbc = WORKBOOK_CONTROL (wbcg);
 	GtkWidget *dialog, *radio_0;
 	SheetSelection *ss;
 	int  cols, rows;
@@ -38,7 +38,7 @@ dialog_delete_cells_impl (Workbook *wb, Sheet *sheet, GladeXML  *gui)
 	g_return_if_fail (radio_0 != NULL);
 
 	/* Bring up the dialog */
-	res = gnumeric_dialog_run (wb, GNOME_DIALOG (dialog));
+	res = gnumeric_dialog_run (wbcg, GNOME_DIALOG (dialog));
 	if (res == GNOME_OK) {
 		i = gtk_radio_group_get_selected
 			(GTK_RADIO_BUTTON(radio_0)->group);
@@ -48,20 +48,20 @@ dialog_delete_cells_impl (Workbook *wb, Sheet *sheet, GladeXML  *gui)
 		rows = ss->user.end.row - ss->user.start.row + 1;
 
 		if (i == 0)
-			cmd_shift_rows (workbook_command_context_gui (wb), sheet,
+			cmd_shift_rows (wbc, sheet,
 					ss->user.start.col + cols, 
 					ss->user.start.row, 
 					ss->user.end.row, -cols);
 		else if (i == 1)
-			cmd_shift_cols (workbook_command_context_gui (wb), sheet,
+			cmd_shift_cols (wbc, sheet,
 					ss->user.start.col,
 					ss->user.end.col, 
 					ss->user.start.row + rows, -rows);
 		else if (i == 2)
-			cmd_delete_rows (workbook_command_context_gui (wb), sheet,
+			cmd_delete_rows (wbc, sheet,
 					   ss->user.start.row, rows);
 		else if (i == 3)
-			cmd_delete_cols (workbook_command_context_gui (wb), sheet,
+			cmd_delete_cols (wbc, sheet,
 					   ss->user.start.col, cols);
 	}
 
@@ -71,17 +71,18 @@ dialog_delete_cells_impl (Workbook *wb, Sheet *sheet, GladeXML  *gui)
 }
 
 void
-dialog_delete_cells (Workbook *wb, Sheet *sheet)
+dialog_delete_cells (WorkbookControlGUI *wbcg, Sheet *sheet)
 {
+	WorkbookControl *wbc = WORKBOOK_CONTROL (wbcg);
 	GladeXML  *gui;
 	SheetSelection *ss;
 	int  cols, rows;
 
-	g_return_if_fail (wb != NULL);
+	g_return_if_fail (wbcg != NULL);
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet));
 
-	if (!selection_is_simple (workbook_command_context_gui (wb), sheet, _("delete cells")))
+	if (!selection_is_simple (wbc, sheet, _("delete cells")))
 		return;
 
 	ss = sheet->selections->data;
@@ -90,23 +91,22 @@ dialog_delete_cells (Workbook *wb, Sheet *sheet)
 
 	/* short circuit the dialog if an entire row/column is selected */
 	if (ss->user.start.row == 0 && ss->user.end.row  >= SHEET_MAX_ROWS-1) {
-		cmd_delete_cols (workbook_command_context_gui (wb), sheet,
+		cmd_delete_cols (wbc, sheet,
 				 ss->user.start.col, cols);
 		return;
 	}
 	if (ss->user.start.col == 0 && ss->user.end.col  >= SHEET_MAX_COLS-1) {
-		cmd_delete_rows (workbook_command_context_gui (wb), sheet,
+		cmd_delete_rows (wbc, sheet,
 				 ss->user.start.row, rows);
 		return;
 	}
 
-	gui = gnumeric_glade_xml_new (workbook_command_context_gui (wb),
-				GLADE_FILE);
+	gui = gnumeric_glade_xml_new (wbcg, GLADE_FILE);
         if (gui == NULL)
                 return;
 
 	/* Wrapper to ensure the libglade object gets removed on error */
-	dialog_delete_cells_impl (wb, sheet, gui);
+	dialog_delete_cells_impl (wbcg, sheet, gui);
 	
 	gtk_object_unref (GTK_OBJECT (gui));
 }
