@@ -166,24 +166,6 @@ embeddable_grid_init_anon (EmbeddableGrid *eg)
 
 	eg->workbook = workbook_new_with_sheets (1);
 	sheets = workbook_sheets (eg->workbook);
-
-	g_return_if_fail (sheets != NULL &&
-			  sheets->data != NULL);
-	eg->sheet = sheets->data;
-	g_list_free (sheets);
-
-	/*
-	 * Workaround code.  Sheets are born with a sheet_view,
-	 * but we do not need it at all.  So manually get rid of it.
-	 *
-	 * When sheet_new semantics change (see TODO), the code below
-	 * will warn about this condition to remove this piece of code
-	 */
-	if (eg->sheet->sheet_views)
-		sheet_destroy_sheet_view (eg->sheet,
-					  SHEET_VIEW (eg->sheet->sheet_views->data));
-	else
-		g_error ("Remove workaround code here");
 }
 
 EmbeddableGrid *
@@ -278,12 +260,13 @@ embeddable_grid_get_type (void)
 	return type;
 }
 
-void
+gboolean
 EmbeddableGridFactory_init (void)
 {
 	bonobo_embeddable_grid_factory = bonobo_embeddable_factory_new (
 		"OAFIID:GNOME:Gnumeric:GridFactory:1.0:a1c2ad1c-33bd-4c42-b5d1-9a454f862873",
 		embeddable_grid_factory, NULL);
+	return TRUE;
 }
 
 /*
@@ -316,7 +299,7 @@ grid_view_activate (GridView *grid_view, gboolean state)
 	if (state) {
 		g_return_if_fail (grid_view != NULL);
 
-		gtk_widget_grab_focus (GTK_WIDGET (grid_view->sheet_view));
+		gtk_widget_grab_focus (GTK_WIDGET (grid_view->sheet_view->canvas));
 	}
 
 	bonobo_view_activate_notify (BONOBO_VIEW (grid_view), state);
