@@ -26,6 +26,7 @@
 #include <string.h>
 #include <time.h>
 #include <math.h>
+#include <float.h>
 #include <langinfo.h>
 #include <limits.h>
 #include <ctype.h>
@@ -149,10 +150,10 @@ format_get_col_sep (void)
 static GHashTable *style_format_hash = NULL;
 
 typedef struct {
-        char     *format;
-	gboolean  want_am_pm;
-        char      restriction_type;
-        int       restriction_value;
+        char const *format;
+	gboolean    want_am_pm;
+        char        restriction_type;
+        int         restriction_value;
 } StyleFormatEntry;
 
 struct _StyleFormat {
@@ -456,13 +457,11 @@ format_compile (StyleFormat *format)
 	for (i = 0; i < length; i++){
 
 		switch (format->format[i]){
-
 		case ';':
-			if (which < 4){
-				standard_entries [which].format = g_malloc0 (string->len + 1);
-				strncpy (standard_entries[which].format, string->str, string->len);
-				standard_entries [which].format[string->len] = 0;
+			if (which < 4) {
 				standard_entries [which].restriction_type = '*';
+				standard_entries [which].format = 
+					g_strndup (string->str, string->len + 1);
 				which++;
 			}
 			string = g_string_truncate (string, 0);
@@ -473,11 +472,10 @@ format_compile (StyleFormat *format)
 		}
 	}
 
-	if (which < 4){
-		standard_entries[which].format = g_malloc0 (string->len + 1);
-		strncpy (standard_entries[which].format, string->str, string->len);
-		standard_entries[which].format[string->len] = 0;
+	if (which < 4) {
 		standard_entries[which].restriction_type = '*';
+		standard_entries[which].format =
+			g_strndup (string->str, string->len + 1);
 		which++;
 	}
 
@@ -511,11 +509,17 @@ format_compile (StyleFormat *format)
 	g_string_free (string, TRUE);
 }
 
+/**
+ * format_entry_dtor :
+ *
+ * WARNING : do not call this for temporary formats generated for
+ * 'General'.
+ */
 static void
-format_entry_dtor (gpointer data, gpointer	user_data)
+format_entry_dtor (gpointer data, gpointer user_data)
 {
 	StyleFormatEntry *entry = data;
-	g_free (entry->format);
+	g_free ((char *)entry->format);
 	g_free (entry);
 }
 
@@ -743,7 +747,7 @@ do_render_number (gdouble number, format_info_t *info)
 	 * decimal point
 	 */
 	if (number > 0.0 && number < 1.0 && info->right_allowed == 0 && info->right_optional > 0){
-		decimal_point [0] = lc->decimal_point [0];
+		decimal_point [0] = format_get_decimal ();
 		decimal_point [1] = 0;
 	} else
 		decimal_point [0] = 0;
@@ -1121,7 +1125,8 @@ format_number (gdouble number, const StyleFormatEntry *style_format_entry)
 			}
 			/* FIXME: this is a gross hack */
 			{
-				char *buffer = g_alloca (40 + info.right_req + 2);
+				int prec = info.right_optional + info.right_req;
+				char *buffer = g_alloca (40 + prec + 2);
 
 				sprintf (buffer, (info.scientific == 'e')
 					 ? "%s%.*e"
@@ -1130,7 +1135,7 @@ format_number (gdouble number, const StyleFormatEntry *style_format_entry)
 					 ? "-" :
 					 info.scientific_shows_plus
 					 ? "+" : "",
-					 info.right_req, number);
+					 prec, number);
 
 				g_string_append (result, buffer);
 				goto finish;
@@ -1425,76 +1430,161 @@ check_valid (const StyleFormatEntry *entry, const Value *value)
 	}
 }
 
+/**
+ * fmt_general_float:
+ *
+ * @val : the integer value being formated.
+ * @col_width : the approximate width in characters.
+ */
+static char *
+fmt_general_float (float_t val, float col_width)
+{
+	double tmp;
+	int log_val, prec;
+
+	if (col_width < 0.)
+		return g_strdup_printf ("%.*g", DBL_DIG, val);
+
+	if (val < 0.) {
+		/* leave space for minus sign */
+		/* FIXME : idealy we would use the width of a minus sign */
+		col_width -= 1.;
+		tmp = log10 (-val);
+	} else
+		tmp = (val > 0.) ? log10 (val) : 0;
+
+	/* leave space for the decimal */
+	/* FIXME : idealy we would use the width of a decimal point */
+	prec = (int) floor (col_width - 1.);
+	if (prec < 0)
+		prec = 0;
+
+	if (tmp > 0.) {
+		log_val = ceil (tmp);
+
+		/* Decrease precision to leave space for the E+00 */
+		if (log_val > prec)
+			for (prec -= 4; log_val >= 100 ; log_val /= 10)
+				prec--;
+	} else {
+		log_val = floor (tmp);
+
+		/* Display 0 for cols that are too narrow for scientific
+		 * notation with abs (value) < 1 */
+		if (col_width < 5. && -log_val >= prec)
+			return g_strdup ("0");
+
+		/* Include leading zeros eg 0.0x has 2 leading zero */
+		if (log_val >= -4)
+			prec += log_val;
+
+		/* Decrease precision to leave space for the E+00 */
+		else for (prec -= 4; log_val <= -100 ; log_val /= 10)
+			prec--;
+	}
+
+	if (prec < 1)
+		prec = 1;
+	else if (prec > DBL_DIG)
+		prec = DBL_DIG;
+
+	/* FIXME : glib bug.  it does not handle G, use g (fixed in 1.2.9) */
+	return g_strdup_printf ("%.*g", prec, val);
+}
+
+/**
+ * fmt_general_int :
+ *
+ * @val : the integer value being formated.
+ * @col_width : the approximate width in characters.
+ */
+static char *
+fmt_general_int (int val, int col_width)
+{
+	if (col_width > 0) {
+		int log_val;
+
+		if (val < 0) {
+			/* leave space for minus sign */
+			col_width--;
+			log_val = ceil (log10 (-val));
+		} else
+			log_val = (val > 0) ? ceil (log10 (val)) : 0;
+
+		/* Switch to scientific notation if things are too wide */
+		if (log_val > col_width)
+			/* FIXME : glib bug.  it does not handle G, use g */
+			/* Decrease available width by 5 to account for .+E00 */
+			return g_strdup_printf ("%.*g", col_width - 5, (double)val);
+	}
+	return g_strdup_printf ("%d", val);
+}
+
 /*
  * Returns NULL when the value should be formated as text
  */
 gchar *
 format_value (StyleFormat *format, const Value *value, StyleColor **color,
-	      char const * entered_text)
+	      float col_width)
 {
 	char *v = NULL;
 	StyleFormatEntry entry;
 	gboolean is_general = FALSE;
 	GList *list;
 
-	g_return_val_if_fail (value != NULL, "<ERROR>");
-
 	if (color)
 		*color = NULL;
 
-	/* get format */
-	for (list = format->entries; list; list = g_list_next (list))
-		if (check_valid (list->data, value))
-			break;
+	g_return_val_if_fail (value != NULL, "<ERROR>");
 
-	if (list)
-		entry = *(StyleFormatEntry *)(list->data);
-	else
-		entry.format = format->format;
+	if (format) {
+		/* get format */
+		for (list = format->entries; list; list = g_list_next (list))
+			if (check_valid (list->data, value))
+				break;
 
-	/* Try to parse a color specification */
-	if (entry.format [0] == '['){
-		char *end = strchr (entry.format, ']');
+		if (list)
+			entry = *(StyleFormatEntry *)(list->data);
+		else
+			entry.format = format->format;
 
-		if (end) {
-			char first_after_bracket = entry.format [1];
+		/* Try to parse a color specification */
+		if (entry.format [0] == '['){
+			char *end = strchr (entry.format, ']');
 
-			/*
-			 * Special [h*], [m*], [*s] is using for
-			 * and [$*] are for currencies.
-			 * measuring times, not for specifying colors.
-			 */
-			if (!(first_after_bracket == 'h' ||
-			      first_after_bracket == 's' ||
-			      first_after_bracket == 'm' ||
-			      first_after_bracket == '$')){
-				if (color)
-					*color = lookup_color (&entry.format [1], end);
-				entry.format = end+1;
+			if (end) {
+				char first_after_bracket = entry.format [1];
+
+				/*
+				 * Special [h*], [m*], [*s] is using for
+				 * and [$*] are for currencies.
+				 * measuring times, not for specifying colors.
+				 */
+				if (!(first_after_bracket == 'h' ||
+				      first_after_bracket == 's' ||
+				      first_after_bracket == 'm' ||
+				      first_after_bracket == '$')){
+					if (color)
+						*color = lookup_color (&entry.format [1], end);
+					entry.format = end+1;
+				}
 			}
 		}
-	}
 
-	/* Empty formats should be ignored */
-	if (entry.format [0] == '\0')
-		return g_strdup ("");
+		/* Empty formats should be ignored */
+		if (entry.format [0] == '\0')
+			return g_strdup ("");
 
-	/* Formatting a value as a text returns the entered text */
-	if (strcmp (entry.format, "@") == 0) {
-		if (entered_text != NULL)
-			return g_strdup (entered_text);
-		if (value->type == VALUE_STRING)
-			return g_strdup (value->v_str.val->str);
+		/* Formatting a value as a text returns the entered text */
+		if (strcmp (entry.format, "@") == 0) {
+			if (value->type == VALUE_STRING)
+				return g_strdup (value->v_str.val->str);
 
-		/* FIXME : What does it mean to format a value as text
-		 * without specifying the entered text ??
-		 * use General as a failsafe */
+			is_general = TRUE;
+		} else if (strcmp (entry.format, "General") == 0)
+			is_general = TRUE;
+	} else
 		is_general = TRUE;
-	}
-
-	/* No need to translate we always store in C locale */
-	if (strcmp (entry.format, "General") == 0)
-	    is_general = TRUE;
 
 	/*
 	 * Use top left corner of an array result.
@@ -1505,26 +1595,24 @@ format_value (StyleFormat *format, const Value *value, StyleColor **color,
 
 	switch (value->type){
 	case VALUE_FLOAT:
-		if (finite (value->v_float.val)) {
-			if (is_general) {
-				/* FIXME FIXME FIXME : This is pathetic
-				 * rendering for General should be done
-				 * with knowledge of the cell width to
-				 * compute the optimal format
-				 */
-				if (floor (value->v_float.val) == value->v_float.val)
-					entry.format = "0";
-				else
-					entry.format = "0.0########";
-			}
-			v = format_number (value->v_float.val, &entry);
-		} else
+		if (!finite (value->v_float.val))
 			return g_strdup (gnumeric_err_VALUE);
+
+		if (is_general) {
+			float_t val = value->v_float.val;
+			if ((float_t)INT_MAX >= val && val >= (float_t)INT_MIN) {
+				double int_val = floor (value->v_float.val);
+				if (int_val == value->v_float.val)
+					return fmt_general_int (int_val, col_width);
+			}
+			return fmt_general_float (val, col_width);
+		}
+		v = format_number (value->v_float.val, &entry);
 		break;
 
 	case VALUE_INTEGER:
 		if (is_general)
-			entry.format = "0";
+			return fmt_general_int (value->v_int.val, col_width);
 		v = format_number (value->v_int.val, &entry);
 		break;
 
@@ -1654,12 +1742,22 @@ style_format_delocalize (const char *descriptor_string)
 		return g_strdup ("General");
 }
 
+/**
+ * style_format_new_XL :
+ *
+ * Looks up and potentially creates a StyleFormat from the supplied string in
+ * XL format.
+ */
 StyleFormat *
-style_format_new_XL (const char *descriptor_string, gboolean delocalize)
+style_format_new_XL (char const *descriptor_string, gboolean delocalize)
 {
 	StyleFormat *format;
 
-	if (delocalize)
+	/* Safety net */
+	if (descriptor_string == NULL) {
+		g_warning ("Invalid format descriptor string, using General");
+		descriptor_string = "General";
+	} else if (delocalize)
 		descriptor_string = style_format_delocalize (descriptor_string);
 
 	format = (StyleFormat *) g_hash_table_lookup (style_format_hash, descriptor_string);
@@ -1754,6 +1852,13 @@ style_format_str_as_XL (char const *ptr, gboolean localized)
 	}
 }
 
+/**
+ * style_format_as_XL :
+ * @sf :
+ * @localized : should the string be in cannonical or locale specific form.
+ *
+ * Return a string which the caller is responsible for freeing.
+ */
 char *
 style_format_as_XL (StyleFormat const *fmt, gboolean localized)
 {
@@ -1766,6 +1871,12 @@ style_format_as_XL (StyleFormat const *fmt, gboolean localized)
 	return style_format_str_as_XL (fmt->format, localized);
 }
 
+/**
+ * style_format_ref :
+ * @sf :
+ *
+ * Add a reference to a StyleFormat
+ */
 void
 style_format_ref (StyleFormat *sf)
 {
@@ -1774,6 +1885,12 @@ style_format_ref (StyleFormat *sf)
 	sf->ref_count++;
 }
 
+/**
+ * style_format_unref :
+ * @sf :
+ *
+ * Remove a reference to a StyleFormat, freeing when it goes to zero.
+ */
 void
 style_format_unref (StyleFormat *sf)
 {
@@ -1790,9 +1907,14 @@ style_format_unref (StyleFormat *sf)
 	g_free (sf);
 }
 
+/**
+ * style_format_is_general :
+ * @sf : the format to check
+ *
+ * A small utility to check whether a format is 'General'
+ */
 gboolean
-style_format_is_general(StyleFormat const *sf)
+style_format_is_general (StyleFormat const *sf)
 {
 	return 0 == strcmp (sf->format, "General");
 }
-
