@@ -8,6 +8,7 @@
 #include <math.h>
 #include <glib.h>
 #include "parse-util.h"
+#include "func-util.h"
 #include "func.h"
 #include "cell.h"
 #include "rendered-value.h"
@@ -138,7 +139,7 @@ find_column_of_field (const EvalPos *ep, Value *database, Value *field)
 		if (cell == NULL)
 		        continue;
 
-		txt = cell_get_entered_text (cell);
+		txt = cell_get_rendered_text (cell);
 		match = (g_strcasecmp (field_name, txt) == 0);
 		g_free (txt);
 		if (match) {
@@ -250,7 +251,7 @@ parse_criteria_range(Sheet *sheet, int b_col, int b_row, int e_col, int e_row,
 			}
 
 			/* Other conditions (in string format) */
-			cell_str = cell_get_entered_text(cell);
+			cell_str = cell_get_rendered_text (cell);
 			parse_criteria(cell_str, &cond->fun, &cond->x);
 			if (field_ind != NULL)
 			        cond->column = field_ind[j-b_col];
@@ -425,8 +426,8 @@ find_rows_that_match (Sheet *sheet, int first_col, int first_row,
 						sheet_cell_get(sheet, i, trow);
 					      cell =
 						sheet_cell_get(sheet, i, row);
-					      t1 = cell_get_entered_text (cell);
-					      t2 = cell_get_entered_text (test_cell);
+					      t1 = cell_get_rendered_text (cell);
+					      t2 = cell_get_rendered_text (test_cell);
 					      if (strcmp (t1, t2) != 0)
 						      goto row_ok;
 				       }
@@ -1329,6 +1330,7 @@ gnumeric_getpivotdata (FunctionEvalInfo *ei, Value **argv)
 
 /***************************************************************************/
 
+void database_functions_init(void);
 void
 database_functions_init(void)
 {
