@@ -9,6 +9,7 @@
  */
 #include <config.h>
 #include <math.h>
+#include <limits.h>
 #include "gnumeric.h"
 #include "parse-util.h"
 #include "cell.h"
@@ -191,6 +192,369 @@ days_monthly_basis (Value *issue_date, Value *maturity_date, int basis)
 	}
 }
 
+/*
+ * Returns the number of days in the coupon period of the settlement date.
+ * Currently, returns negative numbers if the branch is not implemented.
+ */
+static float_t
+coupdays (GDate *settlement, GDate *maturity, int freq, int basis)
+{
+        GDateYear  sy, my;
+	GDateMonth sm, mm;
+	GDateDay   sd, md;
+
+        switch (basis) {
+        case 0:
+        case 2:
+        case 4:
+                if (freq == 1)
+                        return 360.0;
+                else if (freq == 2)
+                        return 180.0;
+                else
+                        return 90.0;
+                break;
+        case 3:
+                return 365.0 / freq;
+        case 1:
+		sy = g_date_year  (settlement);
+		my = g_date_year  (maturity);
+		sm = g_date_month (settlement);
+		mm = g_date_month (maturity);
+		sd = g_date_day   (settlement);
+		md = g_date_day   (maturity);
+
+	        if (freq == 1) {
+		        if (g_date_is_leap_year (sy)) {
+			        if (sm == 1 || sm == 2) {
+				        if (sm < mm || (sm == mm && sd < md))
+					        return 365.0;
+					else
+					        return 366.0;
+				}
+				if (mm == 1 || mm == 2)
+				        return 366.0;
+				if (sm < mm || (sm == mm && sd < md))
+				        return 366.0;
+				else
+				        return 365.0;
+			} else if (g_date_is_leap_year (sy-1)) {
+			        if (sm == 1 || sm == 2)
+				        if (mm == 1 || mm == 2)
+					        if (sm < mm || 
+						    (sm == mm && sd < md))
+						        return 366.0;
+						else
+						        return 365.0;
+					else
+					        return 365.0;
+				else
+				        return 365.0;
+			} else if (g_date_is_leap_year (sy+1)) {
+			        if (sm == 1 || sm == 2)
+				        return 365.0;
+				if (mm == 1 || mm == 2)
+				        return 365.0;
+				if (sm < mm || (sm == mm && sd < md))
+				        return 365.0;
+				else
+				        return 366.0;
+			} else
+			        return 365.0;
+		}
+        default:
+                return -1.0;
+        }
+}
+
+/*
+ * Returns the number of days from the beginning of the coupon period to 
+ * the settlement date.  Currently, returns negative numbers if the branch
+ * is not implemented.
+ */
+static int
+coupdaybs(GDate *settlement, GDate *maturity, int freq, int basis)
+{
+        int        d, months, days;
+	GDateYear  sy, my;
+	GDateMonth sm, mm;
+	GDateDay   sd, md;
+
+	sy = g_date_year  (settlement);
+	my = g_date_year  (maturity);
+	sm = g_date_month (settlement);
+	mm = g_date_month (maturity);
+	sd = g_date_day   (settlement);
+	md = g_date_day   (maturity);
+
+	months = mm - sm;
+
+        switch (basis) {
+	case 0: /* US 30/360 */
+	        days = md - sd;
+
+	        if (! g_date_is_leap_year (sy) &&
+		    g_date_is_leap_year (my) &&
+		    mm == 2 && md == 29)
+		        --days;
+		else if (g_date_is_leap_year (sy) &&
+			 !g_date_is_leap_year (my) &&
+			 mm == 2 && md == 28) {
+		        if (sd == 29 && sm == 2)
+			        return 0;
+			else
+			        days += 2;
+		}
+
+		d = 360 - months*30 - days;
+
+	        if (freq == 1) {
+			if (d >= 360) {
+			        if (days == 0 && months == 0)
+				        d = 0;
+			        else if (d % 360 == 0)
+				        d = 360;
+				else
+				        d %= 360;
+			}
+		} else if (freq == 2) {
+			if ((d % 180) == 0 && days)
+			        d = 180;
+			else
+			        d %= 180;
+		} else {
+			if ((d % 90) == 0 && days)
+			        d = 90;
+			else
+			        d %= 90;
+		}
+
+		return d;
+	case 1:
+	case 2:
+	case 3:
+	        return -1;
+	case 4: /* European 30/360 */
+	        if (!g_date_is_leap_year (sy) &&
+		    g_date_is_leap_year (my) &&
+		    mm == 2 && md == 29)
+		        return 0;
+
+	        if (sd == 31)
+		        sd = 30;
+		if (md == 31)
+		        md = 30;
+
+		days = md - sd;
+		d = 360 - months*30 - days;
+
+	        if (freq == 1) {
+		        if ((d % 360) == 0 && days)
+			        d = 360;
+			else
+			        d %= 360;
+		} else if (freq == 2) {
+		        if ((d % 180) == 0 && days)
+			        d = 180;
+			else
+			        d %= 180;
+		} else
+		        if ((d % 90) == 0 && days)
+			        d = 90;
+			else
+			        d %= 90;
+		return d;
+	default:
+	        return -1;
+	}
+}
+
+static GDateDay
+days_in_month(GDateYear year, GDateMonth month)
+{
+        switch (month) {
+	case 1:
+	case 3:
+	case 5:
+	case 7:
+	case 8:
+	case 10:
+	case 12:
+	        return 31;
+	case 4:
+	case 6:
+	case 9:
+	case 11:
+	        return 30;
+	case 2:
+	        if (g_date_is_leap_year(year))
+		        return 29;
+		else
+		        return 28;
+	default:
+	        return 0;
+	}
+}
+
+static int
+coupnum(GDate *settlement, GDate *maturity, int freq, int basis)
+{
+        int        years, months, days;
+	GDateYear  sy, my;
+	GDateMonth sm, mm;
+	GDateDay   sd, md;
+
+	sy = g_date_year  (settlement);
+	sm = g_date_month (settlement);
+	sd = g_date_day   (settlement);
+	my = g_date_year  (maturity);
+	mm = g_date_month (maturity);
+	md = g_date_day   (maturity);
+
+	years = my - sy;
+	months = mm - sm;
+	if (md == days_in_month (my, mm) && sd == days_in_month (sy, sm))
+	        days = 0;
+	else
+	        days = md - sd + (g_date_is_leap_year (sy) && sd==28 && sm==2);
+
+	if (freq == 1)
+	        return years + (months > 0) + (months == 0 && days > 0);
+	else if (freq == 2)
+	        return years*2
+		        + (months >  6) + (months ==  6 && days > 0)
+		        + (months >  0) + (months ==  0 && days > 0)
+		        - (months < -6) - (months == -6 && days < 0);
+	else
+	        return years*4
+		        + (months >  9) + (months ==  9 && days > 0)
+		        + (months >  6) + (months ==  6 && days > 0)
+		        + (months >  3) + (months ==  3 && days > 0)
+		        + (months >  0) + (months ==  0 && days > 0)
+		        - (months < -3) - (months == -3 && days < 0)
+		        - (months < -6) - (months == -6 && days < 0)
+		        - (months < -9) - (months == -9 && days < 0);
+}
+
+static GDate *
+coupncd(GDate *settlement, GDate *maturity, int freq, int basis)
+{
+        int        months, days;
+	GDateYear  sy, my, year;
+	GDateMonth sm, mm, month;
+	GDateDay   sd, md, day;
+
+	sy = g_date_year (settlement);
+	sm = g_date_month (settlement);
+	sd = g_date_day (settlement);
+	my = g_date_year (maturity);
+	mm = g_date_month (maturity);
+	md = g_date_day (maturity);
+
+	if (freq == 1) {
+	        day = md;
+		month = mm;
+		if (sy ==  my || sm < mm || (sm == mm && sd < md))
+		        year = sy;
+		else
+		        year = sy + 1;
+	} else if (freq == 2) {
+	        days = md - sd;
+		months = mm - sm;
+
+		if ((months > 0 || (months == 0 && days > 0)) &&
+		    (months < 6 || (months == 6 && days <= 0))) {
+		        month = mm;
+			year = sy;
+		} else if (months > 6 || (months == 6 && days > 0)) {
+		        month = mm - 6;
+			year = sy;
+		} else if (months < -6 || (months == -6 && days <= 0)) {
+		        month = mm;
+			year = sy + 1;
+		} else {
+		        month = mm + 6;
+			year = sy;
+			if (month > 12) {
+			        month -= 12;
+				year++;
+			}
+		}
+		if (md == days_in_month (my, mm))
+		        day = days_in_month (year, month);
+		else
+		        day = md;
+	}  else {
+	        days = md - sd;
+		months = mm - sm;
+		
+		if ((months > 0 || (months == 0 && days > 0)) &&
+		    (months < 3 || (months == 3 && days <= 0))) {
+		        month = mm;
+			year = sy;
+		} else if ((months > 3 || (months == 3 && days > 0))
+			   && (months < 6 || (months == 6 && days <= 0))) {
+		        month = mm - 3;
+			year = sy;
+		} else if ((months > 6 || (months == 6 && days > 0))
+			   && (months < 9 || (months == 9 && days <= 0))) {
+		        month = mm - 6;
+			year = sy;
+		} else if ((months > 9 || (months == 9 && days > 0))) {
+		        month = mm - 9;
+			year = sy;
+		} else if (months < -9 || (months == -9 && days <= 0)) {
+		        month = mm;
+			year = sy + 1;
+		} else if (months < -6 || (months == -6 && days <= 0)) {
+		        month = mm + 9;
+			year = sy;
+			if (month > 12) {
+			        month -= 12;
+				year++;
+			}
+		} else if (months < -3 || (months == -3 && days <= 0)) {
+		        month = mm + 6;
+			year = sy;
+			if (month > 12) {
+			        month -= 12;
+				year++;
+			}
+		} else {
+		        month = mm + 3;
+			year = sy;
+			if (month > 12) {
+			        month -= 12;
+				year++;
+			}
+		}
+
+		if (md == days_in_month(my, mm))
+		        day = days_in_month(year, month);
+		else
+		        day = md;
+	}
+
+	if (! g_date_is_leap_year (year) && day == 29 && month == 2)
+	        day = 28;
+
+	if ((year > my || (year == my && month > mm)
+	     || (year == my && month == mm && day > md))) {
+	        year = my;
+		month = mm;
+		day = md;
+	}
+
+	if (year < sy || (year == sy && month < sm)
+	    || (year == sy && month == sm && day < sd)) {
+	        year = sy;
+		month = mm;
+		day = md;
+	}
+
+	return g_date_new_dmy (day, month, year);
+}
+
 /***************************************************************************/
 
 static char *help_accrint = {
@@ -828,7 +1192,7 @@ gnumeric_sln (FunctionEvalInfo *ei, Value **argv)
 	float_t cost,salvage_value,life;
 
 	cost = value_get_as_float (argv[0]);
-	salvage_value = value_get_as_int (argv[1]);
+	salvage_value = value_get_as_float (argv[1]);
 	life = value_get_as_float (argv[2]);
 
 	/* Life of an asset cannot be negative */
@@ -881,7 +1245,7 @@ gnumeric_syd (FunctionEvalInfo *ei, Value **argv)
 	float_t cost, salvage_value, life, period;
 
 	cost   = value_get_as_float (argv[0]);
-	salvage_value = value_get_as_int (argv[1]);
+	salvage_value = value_get_as_float (argv[1]);
 	life   = value_get_as_float (argv[2]);
 	period = value_get_as_float (argv[3]);
 
@@ -1004,7 +1368,8 @@ gnumeric_mirr (FunctionEvalInfo *ei, Value **argv)
 	rrate = value_get_as_float (argv[2]);
 
 	values = collect_floats_value (argv[0], ei->pos,
-				       COLLECT_IGNORE_STRINGS,
+				       COLLECT_IGNORE_STRINGS |
+				       COLLECT_IGNORE_BLANKS,
 				       &n, &result);
 	if (result)
 		goto out;
@@ -1185,12 +1550,11 @@ typedef struct {
 static GoalSeekStatus
 gnumeric_rate_f (float_t rate, float_t *y, void *user_data)
 {
-	if (rate > -1.0) {
+	if (rate > -1.0 && rate != 0) {
 		gnumeric_rate_t *data = user_data;
 
 		*y = data->pv * calculate_pvif (rate, data->nper) +
-			data->pmt * (1 + rate * data->type) *
-		  calculate_fvifa (rate, data->nper) +
+			data->pmt * (1 + rate * data->type) * calculate_fvifa (rate, data->nper) +
 			data->fv;
 		return GOAL_SEEK_OK;
 	} else
@@ -1226,7 +1590,7 @@ gnumeric_rate (FunctionEvalInfo *ei, Value **argv)
 	udata.pv   = value_get_as_float (argv[2]);
 	udata.fv   = argv[3] ? value_get_as_float (argv[3]) : 0.0;
 	udata.type = argv[4] ? value_get_as_int (argv[4]) : 0;
-	rate0 = argv[5] ?  value_get_as_int (argv[5]) : 0.1;
+	rate0 = argv[5] ?  value_get_as_float (argv[5]) : 0.1;
 
 	if (udata.nper <= 0)
 		return value_new_error (ei->pos, gnumeric_err_NUM);
@@ -1234,22 +1598,38 @@ gnumeric_rate (FunctionEvalInfo *ei, Value **argv)
 	if (udata.type != 0 && udata.type != 1)
 		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
-	if (udata.pmt == 0) {
-		if (udata.pv == 0 || udata.pv * udata.fv > 0)
-			return value_new_error (ei->pos, gnumeric_err_NUM);
-		else {
-			/* Exact case.  */
-			return value_new_float (pow (-udata.fv / udata.pv,
-						     -1.0 / udata.nper) - 1);
-		}
-	}
-
 #if 0
 	printf ("Guess = %.15g\n", rate0);
 #endif
 	goal_seek_initialise (&data);
+
+	/* Respect the sign of the guess.  */
+	if (rate0 >= 0) {
+		data.xmin = 0;
+		data.xmax = MIN (data.xmax,
+				 pow (DBL_MAX / 1e10, 1.0 / udata.nper) - 1);
+	} else {
+		data.xmin = MAX (data.xmin,
+				 -pow (DBL_MAX / 1e10, 1.0 / udata.nper) + 1);
+		data.xmax = 0;
+	}
+
+	/* Newton search from guess.  */
 	status = goal_seek_newton (&gnumeric_rate_f, &gnumeric_rate_df,
 				   &data, &udata, rate0);
+
+	if (status != GOAL_SEEK_OK) {
+		int factor;
+		/* Lay a net of test points around the guess.  */
+		for (factor = 2; !(data.havexneg && data.havexpos) && factor < 100; factor *= 2) {
+			goal_seek_point (&gnumeric_rate_f, &data, &udata, rate0 * factor);
+			goal_seek_point (&gnumeric_rate_f, &data, &udata, rate0 / factor);
+		}
+
+		/* Pray we got both sides of the root.  */
+		status = goal_seek_bisection (&gnumeric_rate_f, &data, &udata);
+	}
+
 	if (status == GOAL_SEEK_OK) {
 #if 0
 		printf ("Root = %.15g\n\n", data.root);
@@ -1341,21 +1721,44 @@ gnumeric_irr (FunctionEvalInfo *ei, Value **argv)
 	Value           *result = NULL;
 	gnumeric_irr_t  p;
 	float_t         rate0;
-	int             n;
 
-	goal_seek_initialise (&data);
 	rate0 = argv[1] ? value_get_as_float (argv[1]) : 0.1;
 
 	p.values = collect_floats_value (argv[0], ei->pos,
-					 COLLECT_IGNORE_STRINGS,
-					 &n, &result);
+					 COLLECT_IGNORE_STRINGS |
+					 COLLECT_IGNORE_BLANKS,
+					 &p.n, &result);
 	if (result != NULL) {
 		g_free (p.values);
 	        return result;
 	}
 
-	p.n = n;
+	goal_seek_initialise (&data);
+
+	/* Respect the sign of the guess.  */
+	if (rate0 >= 0) {
+		data.xmin = 0;
+		data.xmax = MIN (data.xmax,
+				 pow (DBL_MAX / 1e10, 1.0 / (p.n + 1)) - 1);
+	} else {
+		data.xmin = MAX (data.xmin,
+				 -pow (DBL_MAX / 1e10, 1.0 / (p.n + 1)) + 1);
+		data.xmax = 0;
+	}
+
 	status = goal_seek_newton (&irr_npv, &irr_npv_df, &data, &p, rate0);
+	if (status != GOAL_SEEK_OK) {
+		int factor;
+		/* Lay a net of test points around the guess.  */
+		for (factor = 2; !(data.havexneg && data.havexpos) && factor < 100; factor *= 2) {
+			goal_seek_point (&irr_npv, &data, &p, rate0 * factor);
+			goal_seek_point (&irr_npv, &data, &p, rate0 / factor);
+		}
+
+		/* Pray we got both sides of the root.  */
+		status = goal_seek_bisection (&irr_npv, &data, &p);
+	}
+
 	g_free (p.values);
 
 	if (status == GOAL_SEEK_OK)
@@ -1458,7 +1861,7 @@ gnumeric_npv (FunctionEvalInfo *ei, GList *nodes)
 	p.num   = 0;
 
 	v = function_iterate_argument_values (ei->pos, callback_function_npv,
-					      &p, nodes, TRUE);
+					      &p, nodes, TRUE, TRUE);
 
 	return (v != NULL) ? v : value_new_float (p.sum);
 }
@@ -2157,7 +2560,8 @@ gnumeric_oddlyield (FunctionEvalInfo *ei, Value **argv)
 static char *help_amordegrc = {
 	N_("@FUNCTION=AMORDEGRC\n"
 	   "@SYNTAX=AMORDEGRC(cost,purchase_date,first_period,salvage,period,rate,basis)\n"
-	   "@DESCRIPTION=Returns the depreciation for each accounting period."
+	   "@DESCRIPTION="
+	   "Returns the depreciation for each accounting period."
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "\n"
@@ -2175,7 +2579,8 @@ gnumeric_amordegrc (FunctionEvalInfo *ei, Value **argv)
 static char *help_amorlinc = {
 	N_("@FUNCTION=AMORLINC\n"
 	   "@SYNTAX=AMORLINC(cost,purchase_date,first_period,salvage,period,rate,basis)\n"
-	   "@DESCRIPTION=Returns the depreciation for each accounting period."
+	   "@DESCRIPTION="
+	   "Returns the depreciation for each accounting period."
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "\n"
@@ -2193,8 +2598,9 @@ gnumeric_amorlinc (FunctionEvalInfo *ei, Value **argv)
 static char *help_coupdaybs = {
 	N_("@FUNCTION=COUPDAYBS\n"
 	   "@SYNTAX=COUPDAYBS(settlement,maturity,frequency[,basis])\n"
-	   "@DESCRIPTION=Returns the number of days from the beginning "
-	   "of the coupon period to the settlement date."
+	   "@DESCRIPTION="
+	   "Returns the number of days from the beginning of the coupon "
+	   "period to the settlement date."
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "\n"
@@ -2204,7 +2610,37 @@ static char *help_coupdaybs = {
 static Value *
 gnumeric_coupdaybs (FunctionEvalInfo *ei, Value **argv)
 {
-	return value_new_error (ei->pos, "#UNIMPLEMENTED!");
+        GDate   *settlement;
+        GDate   *maturity;
+        int     freq, basis;
+        int     days;
+
+	if (argv[0]->type == VALUE_ERROR)
+		return value_duplicate (argv[0]);
+	if (argv[1]->type == VALUE_ERROR)
+		return value_duplicate (argv[1]);
+
+        settlement = datetime_value_to_g(argv[0]);
+        maturity = datetime_value_to_g(argv[1]);
+        freq = value_get_as_int (argv[2]);
+	if (argv[3] != NULL)
+	        basis = value_get_as_int (argv[3]);
+	else
+	        basis = 0;
+
+        if (settlement == NULL || maturity == NULL)
+                return value_new_error (ei->pos, gnumeric_err_VALUE);
+
+        if (basis < 0 || basis > 4 || (freq != 1 && freq != 2 && freq != 4)
+	    || g_date_compare(settlement, maturity) > 0)
+                return value_new_error (ei->pos, gnumeric_err_NUM);
+
+        days = coupdaybs(settlement, maturity, freq, basis);
+
+        if (days < 0)
+	        return value_new_error (ei->pos, "#UNIMPLEMENTED!");
+
+        return value_new_int (days);
 }
 
 /***************************************************************************/
@@ -2212,8 +2648,9 @@ gnumeric_coupdaybs (FunctionEvalInfo *ei, Value **argv)
 static char *help_coupdays = {
 	N_("@FUNCTION=COUPDAYS\n"
 	   "@SYNTAX=COUPDAYS(settlement,maturity,frequency[,basis])\n"
-	   "@DESCRIPTION=Returns the number of days in the coupon period "
-	   "of the settlement date."
+	   "@DESCRIPTION="
+	   "Returns the number of days in the coupon period of the "
+	   "settlement date."
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "\n"
@@ -2223,7 +2660,37 @@ static char *help_coupdays = {
 static Value *
 gnumeric_coupdays (FunctionEvalInfo *ei, Value **argv)
 {
-	return value_new_error (ei->pos, "#UNIMPLEMENTED!");
+        GDate   *settlement;
+        GDate   *maturity;
+        int     freq, basis;
+        float_t days;
+
+	if (argv[0]->type == VALUE_ERROR)
+		return value_duplicate (argv[0]);
+	if (argv[1]->type == VALUE_ERROR)
+		return value_duplicate (argv[1]);
+
+        settlement = datetime_value_to_g(argv[0]);
+        maturity = datetime_value_to_g(argv[1]);
+        freq = value_get_as_int (argv[2]);
+	if (argv[3] != NULL)
+	        basis = value_get_as_int (argv[3]);
+	else
+	        basis = 0;
+
+        if (settlement == NULL || maturity == NULL)
+                return value_new_error (ei->pos, gnumeric_err_VALUE);
+
+        if (basis < 0 || basis > 4 || (freq != 1 && freq != 2 && freq != 4)
+	    || g_date_compare(settlement, maturity) > 0)
+                return value_new_error (ei->pos, gnumeric_err_NUM);
+
+        days = coupdays(settlement, maturity, freq, basis);
+
+        if (days < 0)
+	        return value_new_error (ei->pos, "#UNIMPLEMENTED!");
+
+        return value_new_float (days);
 }
 
 /***************************************************************************/
@@ -2231,8 +2698,9 @@ gnumeric_coupdays (FunctionEvalInfo *ei, Value **argv)
 static char *help_coupdaysnc = {
 	N_("@FUNCTION=COUPDAYSNC\n"
 	   "@SYNTAX=COUPDAYSNC(settlement,maturity,frequency[,basis])\n"
-	   "@DESCRIPTION=Returns the number of days from the settlement date "
-	   "to the next coupon date."
+	   "@DESCRIPTION="
+	   "Returns the number of days from the settlement date to the "
+	   "next coupon date."
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "\n"
@@ -2250,7 +2718,8 @@ gnumeric_coupdaysnc (FunctionEvalInfo *ei, Value **argv)
 static char *help_coupncd = {
 	N_("@FUNCTION=COUPNCD\n"
 	   "@SYNTAX=COUPNCD(settlement,maturity,frequency[,basis])\n"
-	   "@DESCRIPTION=Returns the coupon date following settlement."
+	   "@DESCRIPTION="
+	   "Returns the coupon date following settlement."
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "\n"
@@ -2260,7 +2729,37 @@ static char *help_coupncd = {
 static Value *
 gnumeric_coupncd (FunctionEvalInfo *ei, Value **argv)
 {
-	return value_new_error (ei->pos, "#UNIMPLEMENTED!");
+        GDate   *settlement;
+        GDate   *maturity;
+        GDate   *date;
+        int     freq, basis;
+	Value   *v;
+
+	if (argv[0]->type == VALUE_ERROR)
+		return value_duplicate (argv[0]);
+	if (argv[1]->type == VALUE_ERROR)
+		return value_duplicate (argv[1]);
+
+        settlement = datetime_value_to_g(argv[0]);
+        maturity = datetime_value_to_g(argv[1]);
+        freq = value_get_as_int (argv[2]);
+	if (argv[3] != NULL)
+	        basis = value_get_as_int (argv[3]);
+	else
+	        basis = 0;
+
+        if (settlement == NULL || maturity == NULL)
+                return value_new_error (ei->pos, gnumeric_err_VALUE);
+
+        if (basis < 0 || basis > 4 || (freq != 1 && freq != 2 && freq != 4)
+	    || g_date_compare(settlement, maturity) > 0)
+                return value_new_error (ei->pos, gnumeric_err_NUM);
+
+        date = coupncd (settlement, maturity, freq, basis);
+	v = value_new_int (datetime_g_to_serial (date));
+	g_free (date);
+
+	return v;
 }
 
 /***************************************************************************/
@@ -2268,7 +2767,8 @@ gnumeric_coupncd (FunctionEvalInfo *ei, Value **argv)
 static char *help_couppcd = {
 	N_("@FUNCTION=COUPPCD\n"
 	   "@SYNTAX=COUPPCD(settlement,maturity,frequency[,basis])\n"
-	   "@DESCRIPTION=Returns the coupon date preceeding settlement."
+	   "@DESCRIPTION="
+	   "Returns the coupon date preceeding settlement."
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "\n"
@@ -2297,7 +2797,34 @@ static char *help_coupnum = {
 static Value *
 gnumeric_coupnum (FunctionEvalInfo *ei, Value **argv)
 {
-	return value_new_error (ei->pos, "#UNIMPLEMENTED!");
+        GDate   *settlement;
+        GDate   *maturity;
+        int     freq, basis;
+        float_t n;
+
+	if (argv[0]->type == VALUE_ERROR)
+		return value_duplicate (argv[0]);
+	if (argv[1]->type == VALUE_ERROR)
+		return value_duplicate (argv[1]);
+
+        settlement = datetime_value_to_g(argv[0]);
+        maturity = datetime_value_to_g(argv[1]);
+        freq = value_get_as_int (argv[2]);
+	if (argv[3] != NULL)
+	        basis = value_get_as_int (argv[3]);
+	else
+	        basis = 0;
+
+        if (settlement == NULL || maturity == NULL)
+                return value_new_error (ei->pos, gnumeric_err_VALUE);
+
+        if (basis < 0 || basis > 4 || (freq != 1 && freq != 2 && freq != 4)
+	    || g_date_compare(settlement, maturity) > 0)
+                return value_new_error (ei->pos, gnumeric_err_NUM);
+
+        n = coupnum (settlement, maturity, freq, basis);
+
+        return value_new_float (n);
 }
 
 /***************************************************************************/
@@ -2305,8 +2832,9 @@ gnumeric_coupnum (FunctionEvalInfo *ei, Value **argv)
 static char *help_cumipmt = {
 	N_("@FUNCTION=CUMIPMT\n"
 	   "@SYNTAX=CUMIPMT(rate,nper,pv,start_period,end_period,type)\n"
-	   "@DESCRIPTION=Returns the cumulative interest paid on a loan "
-	   "between @start_period and @end_period."
+	   "@DESCRIPTION="
+	   "Returns the cumulative interest paid on a loan between "
+	   "@start_period and @end_period."
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "\n"
@@ -2324,8 +2852,9 @@ gnumeric_cumipmt (FunctionEvalInfo *ei, Value **argv)
 static char *help_cumprinc = {
 	N_("@FUNCTION=CUMPRINC\n"
 	   "@SYNTAX=CUMPRINC(rate,nper,pv,start_period,end_period,type)\n"
-	   "@DESCRIPTION=Returns the cumulative principal paid on a loan "
-	   "between @start_period and @end_period."
+	   "@DESCRIPTION="
+	   "Returns the cumulative principal paid on a loan between "
+	   "@start_period and @end_period."
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "\n"
@@ -2343,8 +2872,8 @@ gnumeric_cumprinc (FunctionEvalInfo *ei, Value **argv)
 static char *help_mduration = {
 	N_("@FUNCTION=MDURATION\n"
 	   "@SYNTAX=MDURATION(settlement,maturity,coupon,yield,frequency[,basis])\n"
-	   "@DESCRIPTION=Returns the Macauley duration for a security with "
-	   "par value 100."
+	   "@DESCRIPTION="
+	   "Returns the Macauley duration for a security with par value 100."
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "\n"
@@ -2379,6 +2908,7 @@ gnumeric_vdb (FunctionEvalInfo *ei, Value **argv)
 
 /***************************************************************************/
 
+void finance_functions_init (void);
 void
 finance_functions_init (void)
 {
