@@ -6,12 +6,12 @@
  */
 #include <config.h>
 
-#include <gnome.h>
-#include <gdk/gdkkeysyms.h>
-#include <gdk/gdkx.h>
-#include <string.h>
-#include "gnumeric.h"
 #include "gnumeric-sheet.h"
+#include "item-bar.h"
+#include "item-cursor.h"
+#include "item-edit.h"
+#include "item-grid.h"
+#include "sheet-view.h"
 #include "gnumeric-util.h"
 #include "color.h"
 #include "selection.h"
@@ -20,6 +20,7 @@
 #include "application.h"
 #include "workbook-view.h"
 #include "workbook-edit.h"
+#include "workbook-control-gui-priv.h"
 #include "workbook.h"
 #include "commands.h"
 
@@ -92,6 +93,7 @@ move_cursor (GnumericSheet *gsheet, int col, int row, gboolean clear_selection)
 		sheet_selection_reset_only (sheet);
 
 	/* Set the cursor BEFORE making it visible to decrease flicker */
+	workbook_finish_editing (gsheet->sheet_view->wbcg, TRUE);
 	sheet_cursor_set (sheet, col, row, col, row, col, row);
 	sheet_make_cell_visible (sheet, col, row);
 
@@ -124,17 +126,19 @@ gnumeric_sheet_set_cursor_bounds (GnumericSheet *gsheet,
 		end_col, end_row);
 }
 
-/*
+/**
  * move_cursor_horizontal:
- *  @Sheet:  The sheet name
- *  @count:  number of units to move the cursor horizontally
- *  @jump_to_boundaries: skip from the start to the end of ranges
+ *
+ * @gsheet : The sheet
+ * @count  : Number of units to move the cursor horizontally
+ * @jump_to_boundaries: skip from the start to the end of ranges
  *                       of filled or unfilled cells.
  *
  * Moves the cursor count columns
  */
 static void
-move_cursor_horizontal (GnumericSheet *gsheet, int count, gboolean jump_to_boundaries)
+move_cursor_horizontal (GnumericSheet *gsheet, int count,
+			gboolean jump_to_boundaries)
 {
 	Sheet *sheet = gsheet->sheet_view->sheet;
 	int const new_col =
@@ -153,17 +157,19 @@ move_horizontal_selection (GnumericSheet *gsheet,
 				count, jump_to_boundaries, TRUE);
 }
 
-/*
+/**
  * move_cursor_vertical:
- *  @Sheet:  The sheet name
- *  @count:  number of units to move the cursor vertically
- *  @jump_to_boundaries: skip from the start to the end of ranges
+ *
+ * @gsheet : The sheet
+ * @count  : Number of units to move the cursor vertically
+ * @jump_to_boundaries: skip from the start to the end of ranges
  *                       of filled or unfilled cells.
  *
  * Moves the cursor count rows
  */
 static void
-move_cursor_vertical (GnumericSheet *gsheet, int count, gboolean jump_to_boundaries)
+move_cursor_vertical (GnumericSheet *gsheet, int count,
+		      gboolean jump_to_boundaries)
 {
 	Sheet *sheet = gsheet->sheet_view->sheet;
 	int const new_row =
@@ -193,32 +199,36 @@ move_vertical_selection (GnumericSheet *gsheet,
 gboolean
 gnumeric_sheet_can_select_expr_range (GnumericSheet *gsheet)
 {
-	Workbook const *wb;
+	WorkbookControlGUI const *wbcg;
 
 	g_return_val_if_fail (gsheet != NULL, FALSE);
 	g_return_val_if_fail (GNUMERIC_IS_SHEET (gsheet), FALSE);
 
-	wb = gsheet->sheet_view->sheet->workbook;
-	if (workbook_edit_entry_redirect_p (wb))
+	wbcg = gsheet->sheet_view->wbcg;
+	if (workbook_edit_entry_redirect_p (wbcg))
 		return TRUE;
 
-	if (!wb->editing)
+	if (!wbcg->editing)
 		return FALSE;
 
 	if (gsheet->selecting_cell)
 		return TRUE;
 
-	return workbook_editing_expr (wb);
+	return workbook_editing_expr (wbcg);
 }
 
 static void
 selection_remove_selection_string (GnumericSheet *gsheet)
 {
-	Workbook const *wb = gsheet->sheet_view->sheet->workbook;
+	WorkbookControlGUI const *wbcg = gsheet->sheet_view->wbcg;
 
-	gtk_editable_delete_text (GTK_EDITABLE (workbook_get_entry_logical (wb)),
-				  gsheet->sel_cursor_pos,
-				  gsheet->sel_cursor_pos+gsheet->sel_text_len);
+	if (gsheet->sel_text_len <= 0)
+		return;
+
+	gtk_editable_delete_text (
+		GTK_EDITABLE (workbook_get_entry_logical (wbcg)),
+		gsheet->sel_cursor_pos,
+		gsheet->sel_cursor_pos + gsheet->sel_text_len);
 }
 
 static void
@@ -226,25 +236,25 @@ selection_insert_selection_string (GnumericSheet *gsheet)
 {
 	ItemCursor *sel = gsheet->sel_cursor;
 	Sheet const *sheet = gsheet->sheet_view->sheet;
-	Workbook const *wb = sheet->workbook;
-	GtkEditable *editable = GTK_EDITABLE (workbook_get_entry_logical (wb));
-	gboolean const inter_sheet = (sheet != wb->editing_sheet);
+	WorkbookControlGUI const *wbcg = gsheet->sheet_view->wbcg;
+	GtkEditable *editable = GTK_EDITABLE (workbook_get_entry_logical (wbcg));
+	gboolean const inter_sheet = (sheet != wbcg->editing_sheet);
 	char *buffer;
 	int pos;
 
 	/* Get the new selection string */
 	buffer = g_strdup_printf ("%s%s%s%d",
-				  wb->select_abs_col ? "$" : "",
+				  wbcg->select_abs_col ? "$" : "",
 				  col_name (sel->pos.start.col),
-				  wb->select_abs_row ? "$" : "",
+				  wbcg->select_abs_row ? "$" : "",
 				  sel->pos.start.row+1);
 
 	if (!range_is_singleton (&sel->pos)) {
 		char *tmp = g_strdup_printf ("%s:%s%s%s%d",
 					      buffer,
-					      wb->select_abs_col ? "$": "",
+					      wbcg->select_abs_col ? "$": "",
 					      col_name (sel->pos.end.col),
-					      wb->select_abs_row ? "$": "",
+					      wbcg->select_abs_row ? "$": "",
 					      sel->pos.end.row+1);
 		g_free (buffer);
 		buffer = tmp;
@@ -259,9 +269,12 @@ selection_insert_selection_string (GnumericSheet *gsheet)
 
 	gsheet->sel_text_len = strlen (buffer);
 	pos = gsheet->sel_cursor_pos;
+
+	gtk_editable_set_position (editable, pos);
 	gtk_editable_insert_text (editable, buffer,
 				  gsheet->sel_text_len,
 				  &pos);
+
 	g_free (buffer);
 
 	/* Set the cursor at the end.  It looks nicer */
@@ -276,14 +289,14 @@ start_cell_selection_at (GnumericSheet *gsheet, int col, int row)
 	GnomeCanvas *canvas = GNOME_CANVAS (gsheet);
 	GnomeCanvasGroup *group = GNOME_CANVAS_GROUP (canvas->root);
 	Sheet *sheet = gsheet->sheet_view->sheet;
-	Workbook *wb = sheet->workbook;
+	WorkbookControlGUI *wbcg = gsheet->sheet_view->wbcg;
 
 	g_return_if_fail (gsheet->selecting_cell == FALSE);
 
 	/* Hide the primary cursor while the range selection cursor is visible 
 	 * and we are selecting on a different sheet than the expr being edited
 	 */
-	if (sheet != sheet->workbook->editing_sheet)
+	if (sheet != wb_control_cur_sheet (WORKBOOK_CONTROL (wbcg)))
 		item_cursor_set_visibility (gsheet->item_cursor, FALSE);
 
 	gsheet->selecting_cell = TRUE;
@@ -300,7 +313,7 @@ start_cell_selection_at (GnumericSheet *gsheet, int col, int row)
 	if (gsheet->item_editor)
 		item_edit_disable_highlight (ITEM_EDIT (gsheet->item_editor));
 
-	gsheet->sel_cursor_pos = GTK_EDITABLE (workbook_get_entry_logical (wb))->current_pos;
+	gsheet->sel_cursor_pos = GTK_EDITABLE (workbook_get_entry_logical (wbcg))->current_pos;
 	gsheet->sel_text_len = 0;
 }
 
@@ -501,7 +514,7 @@ selection_expand_horizontal (GnumericSheet *gsheet, int n, gboolean jump_to_boun
 
 	g_return_if_fail (n == -1 || n == 1);
 
-	if (!gsheet->selecting_cell){
+	if (!gsheet->selecting_cell) {
 		selection_cursor_move_horizontal (gsheet, n, jump_to_boundaries);
 		return;
 	}
@@ -552,7 +565,7 @@ selection_expand_vertical (GnumericSheet *gsheet, int n, gboolean jump_to_bounda
 
 	g_return_if_fail (n == -1 || n == 1);
 
-	if (!gsheet->selecting_cell){
+	if (!gsheet->selecting_cell) {
 		selection_cursor_move_vertical (gsheet, n, jump_to_boundaries);
 		return;
 	}
@@ -602,14 +615,14 @@ static gint
 gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 {
 	Sheet *sheet = gsheet->sheet_view->sheet;
-	Workbook *wb = sheet->workbook;
+	WorkbookControlGUI *wbcg = gsheet->sheet_view->wbcg;
 	void (*movefn_horizontal) (GnumericSheet *, int, gboolean);
 	void (*movefn_vertical)   (GnumericSheet *, int, gboolean);
 	gboolean const select_expr_range = gnumeric_sheet_can_select_expr_range (gsheet);
 	gboolean const jump_to_bounds = event->state & GDK_CONTROL_MASK;
 
 	if (event->state & GDK_SHIFT_MASK) {
-		if (select_expr_range){
+		if (select_expr_range) {
 			movefn_horizontal = selection_expand_horizontal;
 			movefn_vertical = selection_expand_vertical;
 		} else {
@@ -617,7 +630,7 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 			movefn_vertical   = move_vertical_selection;
 		}
 	} else {
-		if (select_expr_range){
+		if (select_expr_range) {
 			movefn_horizontal = selection_cursor_move_horizontal;
 			movefn_vertical   = selection_cursor_move_vertical;
 		} else {
@@ -629,8 +642,8 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 	/* Ignore a few keys (to avoid the selection cursor to be killed
 	 * in some cases
 	 */
-	if (select_expr_range){
-		switch (event->keyval){
+	if (select_expr_range) {
+		switch (event->keyval) {
 		case GDK_Shift_L:   case GDK_Shift_R:
 		case GDK_Alt_L:     case GDK_Alt_R:
 		case GDK_Control_L: case GDK_Control_R:
@@ -642,11 +655,11 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 	 * Magic : Some of these are accelerators,
 	 * we need to catch them before entering because they appear to be printable
 	 */
-	if (!wb->editing && event->keyval == GDK_space &&
+	if (!wbcg->editing && event->keyval == GDK_space &&
 	    (event->state & (GDK_SHIFT_MASK|GDK_CONTROL_MASK)))
 		return FALSE;
 
-	switch (event->keyval){
+	switch (event->keyval) {
 	case GDK_KP_Left:
 	case GDK_Left:
 		(*movefn_horizontal)(gsheet, -1, jump_to_bounds);
@@ -670,7 +683,7 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 	case GDK_KP_Page_Up:
 	case GDK_Page_Up:
 		if ((event->state & GDK_CONTROL_MASK) != 0)
-			gtk_notebook_prev_page (GTK_NOTEBOOK (wb->notebook));
+			gtk_notebook_prev_page (GTK_NOTEBOOK (wbcg->notebook));
 		else if ((event->state & GDK_MOD1_MASK) == 0)
 			(*movefn_vertical)(gsheet, -(gsheet->row.last_visible-gsheet->row.first), FALSE);
 		else
@@ -680,7 +693,7 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 	case GDK_KP_Page_Down:
 	case GDK_Page_Down:
 		if ((event->state & GDK_CONTROL_MASK) != 0)
-			gtk_notebook_next_page (GTK_NOTEBOOK (wb->notebook));
+			gtk_notebook_next_page (GTK_NOTEBOOK (wbcg->notebook));
 		else if ((event->state & GDK_MOD1_MASK) == 0)
 			(*movefn_vertical)(gsheet, gsheet->row.last_visible-gsheet->row.first, FALSE);
 		else
@@ -689,18 +702,15 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 
 	case GDK_KP_Home:
 	case GDK_Home:
-		if ((event->state & GDK_CONTROL_MASK) != 0){
-			/* Set the cursor BEFORE making it visible to decrease flicker */
-			sheet_cursor_set (sheet, 0, 0, 0, 0, 0, 0);
-			sheet_make_cell_visible (sheet, 0, 0);
-			break;
-		} else
+		if ((event->state & GDK_CONTROL_MASK) != 0)
+			move_cursor (gsheet, 0, 0, TRUE);
+		else
 			(*movefn_horizontal)(gsheet, -sheet->cursor.edit_pos.col, FALSE);
 		break;
 
 	case GDK_KP_Delete:
 	case GDK_Delete:
-		cmd_clear_selection (workbook_command_context_gui (wb), sheet, CLEAR_VALUES);
+		cmd_clear_selection (WORKBOOK_CONTROL (wbcg), sheet, CLEAR_VALUES);
 		break;
 
 	/*
@@ -709,13 +719,13 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 	 */
 	case GDK_KP_Enter:
 	case GDK_Return:
-		if (wb->editing &&
+		if (wbcg->editing &&
 		    (event->state == GDK_CONTROL_MASK ||
 		     event->state == (GDK_CONTROL_MASK|GDK_SHIFT_MASK) ||
 		     event->state == GDK_MOD1_MASK))
 			/* Forward the keystroke to the input line */
 			return gtk_widget_event (
-				GTK_WIDGET (workbook_get_entry_logical (wb)),
+				GTK_WIDGET (workbook_get_entry_logical (wbcg)),
 				(GdkEvent *) event);
 		/* fall down */
 
@@ -729,33 +739,33 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 					     event->keyval == GDK_Return) ? FALSE : TRUE;
 
 		/* Be careful to restore the editing sheet if we are editing */
-		if (wb->editing)
-			sheet = wb->editing_sheet;
-		workbook_finish_editing (wb, TRUE);
+		if (wbcg->editing)
+			sheet = wbcg->editing_sheet;
+		workbook_finish_editing (wbcg, TRUE);
 		sheet_selection_walk_step (sheet, direction, horizontal);
 		break;
 	}
 
 	case GDK_Escape:
-		workbook_finish_editing (wb, FALSE);
+		workbook_finish_editing (wbcg, FALSE);
 		application_clipboard_unant ();
 		break;
 
 	case GDK_F4:
-		if (wb->editing && gsheet->sel_cursor) {
+		if (wbcg->editing && gsheet->sel_cursor) {
 			selection_remove_selection_string (gsheet);
-			wb->select_abs_row = (wb->select_abs_row == wb->select_abs_col);
-			wb->select_abs_col = !wb->select_abs_col;
+			wbcg->select_abs_row = (wbcg->select_abs_row == wbcg->select_abs_col);
+			wbcg->select_abs_col = !wbcg->select_abs_col;
 			selection_insert_selection_string (gsheet);
 		}
 		break;
 
 	case GDK_F2:
-		workbook_start_editing_at_cursor (wb, FALSE, FALSE);
+		workbook_start_editing_at_cursor (wbcg, FALSE, FALSE);
 		/* fall down */
 
 	default:
-		if (!wb->editing){
+		if (!wbcg->editing) {
 			if ((event->state & (GDK_MOD1_MASK|GDK_CONTROL_MASK)) != 0)
 				return 0;
 
@@ -763,15 +773,16 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 			if (event->length == 0)
 				return 0;
 
-			workbook_start_editing_at_cursor (wb, TRUE, TRUE);
+			workbook_start_editing_at_cursor (wbcg, TRUE, TRUE);
 		}
 		gnumeric_sheet_stop_cell_selection (gsheet, FALSE);
 
 		/* Forward the keystroke to the input line */
-		return gtk_widget_event (GTK_WIDGET (workbook_get_entry_logical (wb)),
+		return gtk_widget_event (GTK_WIDGET (workbook_get_entry_logical (wbcg)),
 					 (GdkEvent *) event);
 	}
-	sheet_update (sheet);
+	if (!wbcg->editing)
+		sheet_update (sheet);
 
 	return TRUE;
 }
@@ -781,7 +792,7 @@ gnumeric_sheet_key_mode_object (GnumericSheet *gsheet, GdkEventKey *event)
 {
 	Sheet *sheet = gsheet->sheet_view->sheet;
 
-	switch (event->keyval){
+	switch (event->keyval) {
 	case GDK_Escape:
 		sheet_mode_edit	(sheet);
 		application_clipboard_unant ();
@@ -826,8 +837,9 @@ gnumeric_sheet_key_release (GtkWidget *widget, GdkEventKey *event)
 	 */
 	if (sheet->current_object == NULL &&
 	    (event->keyval == GDK_Shift_L || event->keyval == GDK_Shift_R))
-		workbook_set_region_status (sheet->workbook,
-					    cell_pos_name (&sheet->cursor.edit_pos));
+		wb_control_selection_descr_set (
+			WORKBOOK_CONTROL (gsheet->sheet_view->wbcg),
+			cell_pos_name (&sheet->cursor.edit_pos));
 
 	return FALSE;
 }
@@ -862,11 +874,10 @@ gnumeric_sheet_drag_data_get (GtkWidget *widget,
 	Sheet *sheet = GNUMERIC_SHEET (widget)->sheet_view->sheet;
 	Workbook *wb = sheet->workbook;
 	char *s;
-	CommandContext *command_context =
-	    workbook_command_context_gui (sheet->workbook);
+	WorkbookControl *wbc = WORKBOOK_CONTROL (gsheet->sheet_view->wbcg);
 
 	if (wb->filename == NULL)
-		workbook_save (command_context, wb);
+		workbook_save (wbc, wb);
 	if (wb->filename == NULL)
 		return;
 
@@ -899,15 +910,12 @@ gnumeric_sheet_filenames_dropped (GtkWidget        *widget,
 				  GnumericSheet    *gsheet)
 {
 	GList *names, *tmp_list;
-	CommandContext *command_context;
-	Sheet *sheet = gsheet->sheet_view->sheet;
+	WorkbookControl *wbc = WORKBOOK_CONTROL (gsheet->sheet_view->wbcg);
 
-	command_context = workbook_command_context_gui (sheet->workbook);
 	names = gnome_uri_list_extract_filenames ((char *)selection_data->data);
 
 	for (tmp_list = names; tmp_list != NULL ; tmp_list = tmp_list->next) {
-		Workbook *new_wb =
-			workbook_try_read (command_context, tmp_list->data);
+		WorkbookView *new_wb = workbook_try_read (wbc, tmp_list->data);
 
 		if (new_wb == NULL) {
 #ifdef ENABLE_BONOBO
@@ -917,8 +925,7 @@ gnumeric_sheet_filenames_dropped (GtkWidget        *widget,
 			if (so != NULL)
 				sheet_mode_create_object (so);
 #endif
-		} else
-			workbook_show (new_wb);
+		}
 	}
 }
 
@@ -951,7 +958,7 @@ gnumeric_sheet_new (SheetView *sheet_view, ItemBar *colbar, ItemBar *rowbar)
 
 	/* FIXME: figure out some real size for the canvas scrolling region */
 	gnome_canvas_set_scroll_region (GNOME_CANVAS (gsheet), 0, 0,
-					1000000, 1200000);
+					GNUMERIC_SHEET_FACTOR_X, GNUMERIC_SHEET_FACTOR_Y);
 
 	/* handy shortcuts */
 	gsheet_canvas = GNOME_CANVAS (gsheet);
@@ -1103,12 +1110,12 @@ gnumeric_sheet_compute_visible_ranges (GnumericSheet *gsheet,
 		if (ci->visible) {
 			int const bound = pixels + ci->size_pixels;
 
-			if (bound == width){
+			if (bound == width) {
 				gsheet->col.last_visible = col;
 				gsheet->col.last_full = col;
 				break;
 			}
-			if (bound > width){
+			if (bound > width) {
 				gsheet->col.last_visible = col;
 				if (col == gsheet->col.first)
 					gsheet->col.last_full = gsheet->col.first;
@@ -1135,12 +1142,12 @@ gnumeric_sheet_compute_visible_ranges (GnumericSheet *gsheet,
 		if (ri->visible) {
 			int const bound = pixels + ri->size_pixels;
 
-			if (bound == height){
+			if (bound == height) {
 				gsheet->row.last_visible = row;
 				gsheet->row.last_full = row;
 				break;
 			}
-			if (bound > height){
+			if (bound > height) {
 				gsheet->row.last_visible = row;
 				if (row == gsheet->row.first)
 					gsheet->row.last_full = gsheet->row.first;
@@ -1293,13 +1300,13 @@ gnumeric_sheet_make_cell_visible (GnumericSheet *gsheet, int col, int row,
 	canvas = GNOME_CANVAS (gsheet);
 
 	/* Find the new gsheet->col.first */
-	if (col < gsheet->col.first){
+	if (col < gsheet->col.first) {
 		new_first_col = col;
-	} else if (col > gsheet->col.last_full){
+	} else if (col > gsheet->col.last_full) {
 		int width = GTK_WIDGET (canvas)->allocation.width;
 		int first_col;
 
-		for (first_col = col; first_col > 0; --first_col){
+		for (first_col = col; first_col > 0; --first_col) {
 			ColRowInfo const * const ci = sheet_col_get_info (sheet, first_col);
 			if (ci->visible) {
 				width -= ci->size_pixels;
@@ -1312,13 +1319,13 @@ gnumeric_sheet_make_cell_visible (GnumericSheet *gsheet, int col, int row,
 		new_first_col = gsheet->col.first;
 
 	/* Find the new gsheet->row.first */
-	if (row < gsheet->row.first){
+	if (row < gsheet->row.first) {
 		new_first_row = row;
-	} else if (row > gsheet->row.last_full){
+	} else if (row > gsheet->row.last_full) {
 		int height = GTK_WIDGET (canvas)->allocation.height;
 		int first_row;
 
-		for (first_row = row; first_row > 0; --first_row){
+		for (first_row = row; first_row > 0; --first_row) {
 			ColRowInfo const * const ri = sheet_row_get_info (sheet, first_row);
 			if (ri->visible) {
 				height -= ri->size_pixels;
@@ -1411,7 +1418,7 @@ gnumeric_sheet_get_type (void)
 {
 	static GtkType gnumeric_sheet_type = 0;
 
-	if (!gnumeric_sheet_type){
+	if (!gnumeric_sheet_type) {
 		GtkTypeInfo gnumeric_sheet_info = {
 			"GnumericSheet",
 			sizeof (GnumericSheet),
