@@ -1,18 +1,17 @@
 #ifndef GNUMERIC_GNUMERIC_SHEET_H
 #define GNUMERIC_GNUMERIC_SHEET_H
 
-#include "item-grid.h"
-#include "item-cursor.h"
-#include "item-bar.h"
-#include "item-edit.h"
-#include "sheet.h"
+#include "gui-gnumeric.h"
 
 #define GNUMERIC_TYPE_SHEET     (gnumeric_sheet_get_type ())
 #define GNUMERIC_SHEET(obj)     (GTK_CHECK_CAST((obj), GNUMERIC_TYPE_SHEET, GnumericSheet))
 #define GNUMERIC_SHEET_CLASS(k) (GTK_CHECK_CLASS_CAST ((k), GNUMERIC_TYPE_SHEET))
 #define GNUMERIC_IS_SHEET(o)    (GTK_CHECK_TYPE((o), GNUMERIC_TYPE_SHEET))
 
-typedef struct {
+#define GNUMERIC_SHEET_FACTOR_X 1000000
+#define GNUMERIC_SHEET_FACTOR_Y 2000000
+
+struct _GnumericSheet {
 	GnomeCanvas   canvas;
 
 	SheetView     *sheet_view;
@@ -44,7 +43,7 @@ typedef struct {
 	 */
 	ItemBar       *colbar;
 	ItemBar       *rowbar;
-} GnumericSheet;
+};
 
 GtkType    gnumeric_sheet_get_type               (void);
 
