@@ -6,7 +6,7 @@
  * to be typed every time we create a GtkObject
  */
  
-#define GNUMERIC_MAKE_TYPE_WITH_PARENT(l,str,t,pt,ci,i,parent) \
+#define GNUMERIC_MAKE_TYPE_WITH_CLASS(l,str,t,pt,ci,i,parent) \
 GtkType l##_get_type(void)\
 {\
 	static GtkType type = 0;\
@@ -27,6 +27,6 @@ GtkType l##_get_type(void)\
 }
 
 #define GNUMERIC_MAKE_TYPE(l,str,t,ci,i,parent) \
-	GNUMERIC_MAKE_TYPE_WITH_PARENT(l,str,t,t##Class,ci,i,parent)
+	GNUMERIC_MAKE_TYPE_WITH_CLASS(l,str,t,t##Class,ci,i,parent)
 
 #endif /* GNUMERIC_GNUMERIC_TYPE_UTIL_H */
