@@ -6,7 +6,13 @@
 #define SHEET_MAX_ROWS		(64 * 1024)	/* 0 - 65535 inclusive */
 #define SHEET_MAX_COLS		256		/* 0 - 255 inclusive */
 
+typedef struct _CommandContext	CommandContext;
+typedef struct _IOContext	IOContext; /*for future use */
+
 typedef struct _Workbook	Workbook;
+typedef struct _WorkbookView	WorkbookView;
+typedef struct _WorkbookControl	WorkbookControl;
+
 typedef struct _Sheet		Sheet;
 typedef struct _SheetObject	SheetObject;
 typedef struct _Dependent	Dependent;
@@ -65,8 +71,6 @@ typedef struct _ParsePos	   ParsePos;
 typedef struct _FunctionEvalInfo   FunctionEvalInfo;
 typedef struct _FunctionDefinition FunctionDefinition;
 
-typedef struct _CommandContext	   CommandContext;
-
 typedef struct _PrintInformation PrintInformation;
 typedef struct _String	 	 String;
 
@@ -96,10 +100,10 @@ typedef Value * (*ForeachCellCB)(Sheet *sheet, int col, int row,
 
 typedef enum _SpanCalcFlags {
 	SPANCALC_SIMPLE 	= 0x0,	/* Just calc spans */
-	SPANCALC_RESIZE		= 0x1,	/* Calculate the size of the rendered result */
-	SPANCALC_RENDER		= 0x2,	/* render and size any unrendered cells */
-	SPANCALC_RE_RENDER	= 0x4,	/* render of all cells */
-	SPANCALC_NO_DRAW	= 0x8,	/* Do not */
+	SPANCALC_RESIZE		= 0x1,	/* Calculate sizes of all cells */
+	SPANCALC_RE_RENDER	= 0x2,	/* Render and Size all cells */
+	SPANCALC_RENDER		= 0x4,	/* Render and Size any unrendered cells */
+	SPANCALC_NO_DRAW	= 0x8,	/* Do not queue a redraw */
 } SpanCalcFlags;
 
 #endif /* GNUMERIC_H */
