@@ -7,11 +7,12 @@
  */
 #include <config.h>
 
-#include <gnome.h>
-#include "gnumeric.h"
-#include "gnumeric-sheet.h"
 #include "item-bar.h"
 #include "item-debug.h"
+#include "item-grid.h"
+#include "gnumeric-sheet.h"
+#include "sheet-view.h"
+#include "sheet.h"
 #include "parse-util.h"
 #include "gnumeric-util.h"
 #include "selection.h"
@@ -219,7 +220,7 @@ item_bar_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int w
 {
 	ItemBar const * const item_bar = ITEM_BAR (item);
 	Sheet   const * const sheet = item_bar->sheet_view->sheet;
-	GnumericSheet const * const gsheet = GNUMERIC_SHEET (item_bar->sheet_view->sheet_view);
+	GnumericSheet const * const gsheet = GNUMERIC_SHEET (item_bar->sheet_view->canvas);
 	GtkWidget *canvas = GTK_WIDGET (GNOME_CANVAS_ITEM (item)->canvas);
 	int pixels;
 	GdkRectangle rect;
@@ -397,7 +398,7 @@ item_bar_start_resize (ItemBar *bar)
 							 GTK_ORIENTATION_VERTICAL);
 #endif
 	double const zoom = sheet->last_zoom_factor_used; /* * res / 72.; */
-	GnumericSheet const * const gsheet = GNUMERIC_SHEET (bar->sheet_view->sheet_view);
+	GnumericSheet const * const gsheet = GNUMERIC_SHEET (bar->sheet_view->canvas);
 	GnomeCanvas const * const canvas = GNOME_CANVAS (gsheet);
 	GnomeCanvasGroup * const group = GNOME_CANVAS_GROUP (canvas->root);
 	GnomeCanvasPoints * const points =
@@ -532,7 +533,7 @@ item_bar_event (GnomeCanvasItem *item, GdkEvent *e)
 	GnomeCanvas * const canvas = item->canvas;
 	ItemBar * const item_bar = ITEM_BAR (item);
 	Sheet   * const sheet = item_bar->sheet_view->sheet;
-	GnumericSheet * const gsheet = GNUMERIC_SHEET (item_bar->sheet_view->sheet_view);
+	GnumericSheet * const gsheet = GNUMERIC_SHEET (item_bar->sheet_view->canvas);
 	gboolean const is_vertical = (item_bar->orientation == GTK_ORIENTATION_VERTICAL);
 #if 0
 	/*
@@ -672,10 +673,8 @@ item_bar_event (GnomeCanvasItem *item, GdkEvent *e)
 						 item_bar_signals [SELECTION_CHANGED],
 						 element, e->button.state | GDK_BUTTON1_MASK);
 
-			if (is_vertical)
-				item_grid_popup_menu (sheet, e, 0, element, FALSE, TRUE);
-			else
-				item_grid_popup_menu (sheet, e, element, 0, TRUE, FALSE);
+			item_grid_popup_menu (item_bar->sheet_view, &e->button,
+					      !is_vertical, is_vertical);
 		} else if (cri) {
 			/*
 			 * Record the important bits.
