@@ -1,13 +1,13 @@
 #ifndef GNUMERIC_ITEM_BAR_H
 #define GNUMERIC_ITEM_BAR_H
 
-#include "sheet.h"
+#include "gui-gnumeric.h"
 
 #define ITEM_BAR(obj)          (GTK_CHECK_CAST((obj), item_bar_get_type (), ItemBar))
 #define ITEM_BAR_CLASS(k)      (GTK_CHECK_CLASS_CAST ((k), item_bar_get_type (), ItemBarType))
 #define IS_ITEM_BAR(o)         (GTK_CHECK_TYPE((o), item_bar_get_type ()))
 
-typedef struct {
+struct _ItemBar {
 	GnomeCanvasItem  canvas_item;
 	SheetView       *sheet_view;
 	int              first_element;
@@ -32,7 +32,7 @@ typedef struct {
 
 	/* Where the selection started */
 	int             start_selection;
-} ItemBar;
+};
 
 #define ITEM_BAR_IS_SELECTING(ib) ((ib)->start_selection != -1)
 
