@@ -6,10 +6,9 @@
  */
 #include <config.h>
 
-#include <gnome.h>
-#include "gnumeric.h"
 #include "item-cursor.h"
 #include "gnumeric-sheet.h"
+#include "sheet-view.h"
 #include "color.h"
 #include "clipboard.h"
 #include "selection.h"
@@ -52,17 +51,17 @@ item_cursor_animation_callback (ItemCursor *item_cursor)
 static void
 item_cursor_stop_animation (ItemCursor *item_cursor)
 {
-	if (item_cursor->tag == -1)
+	if (item_cursor->animation_timer == -1)
 		return;
 
-	gtk_timeout_remove (item_cursor->tag);
-	item_cursor->tag = -1;
+	gtk_timeout_remove (item_cursor->animation_timer);
+	item_cursor->animation_timer = -1;
 }
 
 static void
 item_cursor_start_animation (ItemCursor *item_cursor)
 {
-	item_cursor->tag = gtk_timeout_add (
+	item_cursor->animation_timer = gtk_timeout_add (
 		150, (GtkFunction)(item_cursor_animation_callback),
 		item_cursor);
 }
@@ -74,6 +73,11 @@ item_cursor_destroy (GtkObject *object)
 
 	item_cursor = ITEM_CURSOR (object);
 
+	if (item_cursor->tip) {
+		gtk_widget_destroy (gtk_widget_get_toplevel (item_cursor->tip));
+		item_cursor->tip = NULL;
+	}
+
 	item_cursor_stop_animation (item_cursor);
 	if (GTK_OBJECT_CLASS (item_cursor_parent_class)->destroy)
 		(*GTK_OBJECT_CLASS (item_cursor_parent_class)->destroy)(object);
@@ -213,7 +217,7 @@ item_cursor_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, in
 	int premove = 0;
 	GdkColor *fore = NULL, *back = NULL;
 	GdkRectangle clip_rect;
-	
+
 	int const xd = item_cursor->cached_x;
 	int const yd = item_cursor->cached_y;
 	int const w = item_cursor->cached_w;
@@ -230,7 +234,7 @@ item_cursor_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, in
 	dy0 = yd - y;
 	dx1 = dx0 + w;
 	dy1 = dy0 + h;
-	
+
 	draw_external = 0;
 	draw_internal = 0;
 	draw_handle   = 0;
@@ -255,7 +259,7 @@ item_cursor_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, in
 		draw_external = 1;
 		draw_stippled = 0;
 		break;
-		
+
 	case ITEM_CURSOR_SELECTION:
 		draw_internal = 1;
 		draw_external = 1;
@@ -291,7 +295,7 @@ item_cursor_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, in
 		fore = &item_cursor->color;
 		back = &item_cursor->color;
 	}
-	
+
 	item_cursor->auto_fill_handle_at_top = (draw_handle >= 2);
 
 	clip_rect.x = 0;
@@ -529,12 +533,11 @@ item_cursor_setup_auto_fill (ItemCursor *item_cursor, ItemCursor const *parent,
 }
 
 static void
-item_cursor_set_cursor (GnomeCanvas *canvas, GnomeCanvasItem *item, int x, int y)
+item_cursor_set_cursor (GnomeCanvas *canvas, ItemCursor *ic, int x, int y)
 {
-	ItemCursor *item_cursor = ITEM_CURSOR (item);
 	int cursor;
 
-	if (IS_LITTLE_SQUARE (item_cursor, x, y))
+	if (IS_LITTLE_SQUARE (ic, x, y))
 		cursor = E_CURSOR_THIN_CROSS;
 	else
 		cursor = E_CURSOR_ARROW;
@@ -556,23 +559,21 @@ item_cursor_selection_event (GnomeCanvasItem *item, GdkEvent *event)
 		gnome_canvas_w2c (
 			canvas, event->crossing.x, event->crossing.y, &x, &y);
 
-		item_cursor_set_cursor (canvas, item, x, y);
+		item_cursor_set_cursor (canvas, ic, x, y);
 		return TRUE;
 
-	case GDK_MOTION_NOTIFY:
-		gnome_canvas_w2c (
-			canvas, event->motion.x, event->motion.y, &x, &y);
-
-		item_cursor_set_cursor (canvas, item, x, y);
-		return TRUE;
-
-	case GDK_BUTTON_PRESS: {
-		GnomeCanvasGroup *group;
+	case GDK_MOTION_NOTIFY: {
 		int style;
 
 		gnome_canvas_w2c (
-			canvas, event->button.x, event->button.y, &x, &y);
-		group = GNOME_CANVAS_GROUP (canvas->root);
+			canvas, event->motion.x, event->motion.y, &x, &y);
+
+		if (!ic->prepared_to_drag) {
+			item_cursor_set_cursor (canvas, ic, x, y);
+			return TRUE;
+		}
+		ic->prepared_to_drag = FALSE;
+		gnome_canvas_item_ungrab (item, event->button.time);
 
 		/*
 		 * determine which part of the cursor was clicked:
@@ -584,7 +585,7 @@ item_cursor_selection_event (GnomeCanvasItem *item, GdkEvent *event)
 			style = ITEM_CURSOR_DRAG;
 
 		new_item = gnome_canvas_item_new (
-			group,
+			GNOME_CANVAS_GROUP (canvas->root),
 			item_cursor_get_type (),
 			"ItemCursor::SheetView", ic->sheet_view,
 			"ItemCursor::Grid",  ic->item_grid,
@@ -644,27 +645,130 @@ item_cursor_selection_event (GnomeCanvasItem *item, GdkEvent *event)
 			NULL,
 			event->button.time);
 
+		/*
+		 * We flush after the grab to ensure that the new item-cursor
+		 * gets created.  If it is not ready in time double click
+		 * events will be disrupted and it will appear as if we are
+		 * doing an button_press with a missing release.
+		 */
+		gdk_flush ();
 		return TRUE;
 	}
+
+	case GDK_2BUTTON_PRESS: {
+		Sheet *sheet = ic->sheet_view->sheet;
+		int final_col = ic->pos.end.col;
+		int final_row = ic->pos.end.col;
+
+		g_return_val_if_fail (ic->prepared_to_drag, TRUE);
+
+		ic->prepared_to_drag = FALSE;
+
+		/*
+		 * We flush after the ungrab, to have the ungrab take
+		 * effect immediately (the copy operation might take
+		 * long, and we do not want the mouse to be grabbed
+		 * all this time).
+		 */
+		gnome_canvas_item_ungrab (item, event->button.time);
+		gdk_flush ();
+
+		workbook_finish_editing (ic->sheet_view->wbcg, TRUE);
+
+		if (sheet_is_region_empty (sheet, &ic->pos))
+			return TRUE;
+
+		if (event->button.state & GDK_MOD1_MASK) {
+			int template_row = ic->pos.start.row - 1;
+			if (template_row < 0 ||
+			    sheet_is_cell_empty (sheet, ic->pos.start.col,
+						 template_row)) {
+
+				template_row = ic->pos.end.row + 1;
+				if (template_row >= SHEET_MAX_ROWS ||
+				    sheet_is_cell_empty (sheet, ic->pos.end.col,
+							 template_row))
+					return TRUE;
+			}
+
+			final_col = sheet_find_boundary_horizontal (sheet,
+				ic->pos.end.col, template_row, 1, TRUE);
+			if (final_row <= ic->pos.end.row)
+				return TRUE;
+		} else {
+			int template_col = ic->pos.start.col - 1;
+			if (template_col < 0 ||
+			    sheet_is_cell_empty (sheet, template_col,
+						 ic->pos.start.row)) {
+
+				template_col = ic->pos.end.col + 1;
+				if (template_col >= SHEET_MAX_COLS ||
+				    sheet_is_cell_empty (sheet, template_col,
+							 ic->pos.start.row))
+					return TRUE;
+			}
+
+			final_row = sheet_find_boundary_vertical (sheet,
+				template_col, ic->pos.end.row, 1, TRUE);
+			if (final_row <= ic->pos.end.row)
+				return TRUE;
+		}
+
+		/* fill the row/column */
+		cmd_autofill (WORKBOOK_CONTROL (ic->sheet_view->wbcg), sheet,
+			      ic->pos.start.col,    ic->pos.start.row,
+			      ic->pos.end.col - ic->pos.start.col + 1,
+			      ic->pos.end.row - ic->pos.start.row + 1,
+			      final_col, final_row);
+
+		return TRUE;
+	}
+
+	case GDK_BUTTON_PRESS:
+		g_return_val_if_fail (!ic->prepared_to_drag, TRUE);
+		ic->prepared_to_drag = TRUE;
+
+		/* prepare to create fill or drag cursors, but dont until we
+		 * move.  If we did create them here there would be problems
+		 * with race conditions when the new cursors pop into existence
+		 * during a double-click
+		 */
+		gnome_canvas_item_grab (
+			item,
+			GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK,
+			NULL,
+			event->button.time);
+
+		/* Be extra paranoid.  Ensure that the grab is registered */
+		gdk_flush ();
+		return TRUE;
+
+	case GDK_BUTTON_RELEASE:
+		/* Double clicks may have already released the drag prep */
+		if (ic->prepared_to_drag) {
+			ic->prepared_to_drag = FALSE;
+			gnome_canvas_item_ungrab (item, event->button.time);
+			gdk_flush ();
+		}
+		return TRUE;
+
 	default:
 		return FALSE;
 	}
 }
 
 static gboolean
-item_cursor_target_region_ok (ItemCursor *item_cursor)
+item_cursor_target_region_ok (ItemCursor *ic)
 {
-	GtkWidget *message;
-	GnomeCanvas *canvas = GNOME_CANVAS_ITEM (item_cursor)->canvas;
-	GtkWidget *window;
 	int v;
+	GtkWidget	*message;
+	GnomeCanvasItem *gci = GNOME_CANVAS_ITEM (ic);
+	SheetView	*sheet_view = ic->sheet_view;
 
-	v = sheet_is_region_empty_or_selected (
-		item_cursor->sheet_view->sheet,
-		item_cursor->pos.start.col, item_cursor->pos.start.row,
-		item_cursor->pos.end.col, item_cursor->pos.end.row);
+	g_return_val_if_fail (gci != NULL, FALSE);
+	g_return_val_if_fail (gci->canvas != NULL, FALSE);
 
-	if (v)
+	if (sheet_is_region_empty_or_selected (sheet_view->sheet, &ic->pos))
 		return TRUE;
 
 	message = gnome_message_box_new (
@@ -675,8 +779,7 @@ item_cursor_target_region_ok (ItemCursor *item_cursor)
 		GNOME_STOCK_BUTTON_YES,
 		GNOME_STOCK_BUTTON_NO,
 		NULL);
-	window = gtk_widget_get_toplevel (GTK_WIDGET (canvas));
-	v = gnumeric_dialog_run (item_cursor->sheet_view->sheet->workbook, GNOME_DIALOG (message));
+	v = gnumeric_dialog_run (sheet_view->wbcg, GNOME_DIALOG (message));
 
 	if (v == 0)
 		return TRUE;
@@ -684,9 +787,9 @@ item_cursor_target_region_ok (ItemCursor *item_cursor)
 }
 
 typedef enum {
-	ACTION_NONE = -1,
+	ACTION_NONE = 0,
+	ACTION_MOVE_CELLS = 1,
 	ACTION_COPY_CELLS,
-	ACTION_MOVE_CELLS,
 	ACTION_COPY_FORMATS,
 	ACTION_COPY_VALUES,
 	ACTION_SHIFT_DOWN_AND_COPY,
@@ -698,95 +801,135 @@ typedef enum {
 static void
 item_cursor_do_action (ItemCursor *item_cursor, ActionType action, guint32 time)
 {
-	Sheet *sheet = item_cursor->sheet_view->sheet;
-	Workbook *wb = sheet->workbook;
+	Sheet *sheet;
 	PasteTarget pt;
-	Range const *target = selection_first_range (sheet, FALSE);
 
-	if (target != NULL && range_equal (target, &item_cursor->pos))
+	if (action == ACTION_NONE || !item_cursor_target_region_ok (item_cursor)) {
+		gtk_object_destroy (GTK_OBJECT (item_cursor));
 		return;
+	}
 
-	switch (action){
-	case ACTION_NONE:
-		return;
+	g_return_if_fail (item_cursor != NULL);
 
+	sheet = item_cursor->sheet_view->sheet;
+
+	switch (action) {
 	case ACTION_COPY_CELLS:
-		if (!item_cursor_target_region_ok (item_cursor))
-			return;
-		if (!sheet_selection_copy (workbook_command_context_gui (wb), sheet))
-			return;
-		cmd_paste (workbook_command_context_gui (wb),
+		if (!sheet_selection_copy (WORKBOOK_CONTROL (item_cursor->sheet_view->wbcg), sheet))
+			break;
+		cmd_paste (WORKBOOK_CONTROL (item_cursor->sheet_view->wbcg),
 			   paste_target_init (&pt, sheet, &item_cursor->pos, PASTE_ALL_TYPES),
 			   time);
-		return;
+		break;
 
 	case ACTION_MOVE_CELLS:
-		if (!item_cursor_target_region_ok (item_cursor))
-			return;
-		if (!sheet_selection_cut (workbook_command_context_gui (wb), sheet))
-			return;
-		cmd_paste (workbook_command_context_gui (wb),
+		if (!sheet_selection_cut (WORKBOOK_CONTROL (item_cursor->sheet_view->wbcg), sheet))
+			break;
+		cmd_paste (WORKBOOK_CONTROL (item_cursor->sheet_view->wbcg),
 			   paste_target_init (&pt, sheet, &item_cursor->pos, PASTE_ALL_TYPES),
 			   time);
-		return;
+		break;
 
 	case ACTION_COPY_FORMATS:
-		if (!item_cursor_target_region_ok (item_cursor))
-			return;
-		if (!sheet_selection_copy (workbook_command_context_gui (wb), sheet))
-			return;
-		cmd_paste (workbook_command_context_gui (wb),
+		if (!sheet_selection_copy (WORKBOOK_CONTROL (item_cursor->sheet_view->wbcg), sheet))
+			break;
+		cmd_paste (WORKBOOK_CONTROL (item_cursor->sheet_view->wbcg),
 			   paste_target_init (&pt, sheet, &item_cursor->pos, PASTE_FORMATS),
 			   time);
-		return;
+		break;
 
 	case ACTION_COPY_VALUES:
-		if (!item_cursor_target_region_ok (item_cursor))
-			return;
-		if (!sheet_selection_copy (workbook_command_context_gui (wb), sheet))
-			return;
-		cmd_paste (workbook_command_context_gui (wb),
-			   paste_target_init (&pt, sheet, &item_cursor->pos, PASTE_VALUES),
+		if (!sheet_selection_copy (WORKBOOK_CONTROL (item_cursor->sheet_view->wbcg), sheet))
+			break;
+		cmd_paste (WORKBOOK_CONTROL (item_cursor->sheet_view->wbcg),
+			   paste_target_init (&pt, sheet, &item_cursor->pos, PASTE_AS_VALUES),
 			   time);
-		return;
+		break;
 
 	case ACTION_SHIFT_DOWN_AND_COPY:
 	case ACTION_SHIFT_RIGHT_AND_COPY:
 	case ACTION_SHIFT_DOWN_AND_MOVE:
 	case ACTION_SHIFT_RIGHT_AND_MOVE:
 		g_warning ("Operation not yet implemented\n");
+		break;
+
+	default :
+		g_warning ("Invalid Operation %d\n", action);
 	}
+
+	gtk_object_destroy (GTK_OBJECT (item_cursor));
 }
 
-static char *drop_context_actions [] = {
-	N_("Copy"),
-	N_("Move"),
-	N_("Copy formats"),
-	N_("Copy values"),
-	N_("Shift cells down and copy"),
-	N_("Shift cells right and copy"),
-	N_("Shift cells down and move"),
-	N_("Shift cells right and move"),
-	NULL,
-};
+static gboolean
+context_menu_hander (GnumericPopupMenuElement const *element,
+		     gpointer ic)
+{
+    g_return_val_if_fail (element != NULL, TRUE);
+
+    item_cursor_do_action (ic, element->index, GDK_CURRENT_TIME);
+    return TRUE;
+}
+
+static void
+item_cursor_popup_menu (ItemCursor *ic, GdkEventButton *event)
+{
+	static GnumericPopupMenuElement const popup_elements[] = {
+		{ N_("_Move"),		NULL,
+		    0, 0, ACTION_MOVE_CELLS },
+
+		{ N_("_Copy"),		GNOME_STOCK_PIXMAP_COPY,
+		    0, 0, ACTION_COPY_CELLS },
+
+		{ N_("Copy _Formats"),		NULL,
+		    0, 0, ACTION_COPY_FORMATS },
+		{ N_("Copy _Values"),		NULL,
+		    0, 0, ACTION_COPY_VALUES },
+
+		{ "", NULL, 0, 0, 0 },
+
+		{ N_("Shift _Down and Copy"),		NULL,
+		    0, 0, ACTION_SHIFT_DOWN_AND_COPY },
+		{ N_("Shift _Right and Copy"),		NULL,
+		    0, 0, ACTION_SHIFT_RIGHT_AND_COPY },
+		{ N_("Shift Dow_n and Move"),		NULL,
+		    0, 0, ACTION_SHIFT_DOWN_AND_MOVE },
+		{ N_("Shift Righ_t and Move"),		NULL,
+		    0, 0, ACTION_SHIFT_RIGHT_AND_MOVE },
+
+		{ "", NULL, 0, 0, 0 },
+
+		{ N_("C_ancel"),		NULL,
+		    0, 0, ACTION_NONE },
+
+		{ NULL, NULL, 0, 0, 0 }
+	};
+
+	gnumeric_create_popup_menu (popup_elements,
+				    &context_menu_hander, ic,
+				    0, 0, event);
+}
 
 /*
  * Invoked when the item has been dropped
  */
 static void
-item_cursor_do_drop (ItemCursor *item_cursor, GdkEventButton *event)
+item_cursor_do_drop (ItemCursor *ic, GdkEventButton *event)
 {
-	ActionType action;
+	/* Only do the operation if something moved */
+	Sheet const *sheet = ic->sheet_view->sheet;
+	Range const *target = selection_first_range (sheet, FALSE);
+
+	if (target == NULL || range_equal (target, &ic->pos)) {
+		gtk_object_destroy (GTK_OBJECT (ic));
+		return;
+	}
 
-	/* Find out what to do */
 	if (event->button == 3)
-		action = (ActionType) run_popup_menu ((GdkEvent *)event, 1, drop_context_actions);
-	else if (event->state & GDK_CONTROL_MASK)
-		action = ACTION_COPY_CELLS;
+		item_cursor_popup_menu (ic, event);
 	else
-		action = ACTION_MOVE_CELLS;
-
-	item_cursor_do_action (item_cursor, action, event->time);
+		item_cursor_do_action (ic, (event->state & GDK_CONTROL_MASK)
+				       ? ACTION_COPY_CELLS : ACTION_MOVE_CELLS,
+				       event->time);
 }
 
 static void
@@ -955,13 +1098,6 @@ item_cursor_drag_event (GnomeCanvasItem *item, GdkEvent *event)
 
 		gnome_canvas_item_ungrab (item, event->button.time);
 		item_cursor_do_drop (item_cursor, (GdkEventButton *) event);
-
-		if (item_cursor->tip) {
-			gtk_widget_destroy (gtk_widget_get_toplevel (item_cursor->tip));
-			item_cursor->tip = NULL;
-		}
-
-		gtk_object_destroy (GTK_OBJECT (item));
 		return TRUE;
 
 	case GDK_BUTTON_PRESS:
@@ -1029,8 +1165,8 @@ item_cursor_autofill_event (GnomeCanvasItem *item, GdkEvent *event)
 		gnome_canvas_item_ungrab (item, event->button.time);
 		gdk_flush ();
 
-		workbook_finish_editing (sheet->workbook, TRUE);
-		cmd_autofill (workbook_command_context_gui (sheet->workbook), sheet,
+		workbook_finish_editing (item_cursor->sheet_view->wbcg, TRUE);
+		cmd_autofill (WORKBOOK_CONTROL (item_cursor->sheet_view->wbcg), sheet,
 			      item_cursor->base_col,    item_cursor->base_row,
 			      item_cursor->base_cols+1, item_cursor->base_rows+1,
 			      item_cursor->pos.end.col, item_cursor->pos.end.row);
@@ -1043,46 +1179,6 @@ item_cursor_autofill_event (GnomeCanvasItem *item, GdkEvent *event)
 	case GDK_MOTION_NOTIFY:
 		item_cursor_handle_motion (item_cursor, event, &cb_autofill_scroll);
 		return TRUE;
-		
-	case GDK_2BUTTON_PRESS: {
-		Sheet *sheet = item_cursor->sheet_view->sheet;
-		int final_row = item_cursor->base_row + item_cursor->base_rows;
-		int final_col = item_cursor->base_col + item_cursor->base_cols;
-		
-		/*
-		 * We flush after the ungrab, to have the ungrab take
-		 * effect inmediately (the copy operation might take
-		 * long, and we do not want the mouse to be grabbed
-		 * all this time).
-		 */
-		gnome_canvas_item_ungrab (item, event->button.time);
-		gdk_flush ();
-
-		workbook_finish_editing (sheet->workbook, TRUE);
-		
-		/* fill current column to boundary of column to left
-		 * OR current row to boundary of row above 
-		 */
-		if (event->button.state & GDK_MOD1_MASK)
-			final_col = sheet_find_boundary_horizontal (sheet, 
-				item_cursor->base_col,
-				MAX(0, item_cursor->base_row-1), 
-				1, TRUE);
-		else
-			final_row = sheet_find_boundary_vertical (sheet, 
-				MAX(0, item_cursor->base_col-1), 
-				item_cursor->base_row,
-				1, TRUE);
-
-		/* fill the row/column */
-		cmd_autofill (workbook_command_context_gui (sheet->workbook), sheet,
-			      item_cursor->base_col,    item_cursor->base_row,
-			      item_cursor->base_cols+1, item_cursor->base_rows+1,
-			      final_col, final_row);
-
-		gtk_object_destroy (GTK_OBJECT (item));
-		return TRUE;
-	}
 
 	default:
 		return FALSE;
@@ -1094,7 +1190,18 @@ item_cursor_event (GnomeCanvasItem *item, GdkEvent *event)
 {
 	ItemCursor *item_cursor = ITEM_CURSOR (item);
 
+#if 0
+	switch (event->type)
+	{
+	case GDK_BUTTON_RELEASE: printf ("release %d\n", item_cursor->style); break;
+	case GDK_BUTTON_PRESS: printf ("press %d\n", item_cursor->style); break;
+	case GDK_2BUTTON_PRESS: printf ("2press %d\n", item_cursor->style); break;
+	default :
+	    break;
+	};
+#endif
 	switch (item_cursor->style){
+	case ITEM_CURSOR_ANTED:
 	case ITEM_CURSOR_SELECTION:
 		return item_cursor_selection_event (item, event);
 
@@ -1129,12 +1236,17 @@ item_cursor_init (ItemCursor *item_cursor)
 
 	item_cursor->col_delta = 0;
 	item_cursor->row_delta = 0;
+
 	item_cursor->tip       = NULL;
 
 	item_cursor->style = ITEM_CURSOR_SELECTION;
-	item_cursor->tag = -1;
+	item_cursor->gc = NULL;
+	item_cursor->state = 0;
+	item_cursor->animation_timer = -1;
+
+	item_cursor->visible = TRUE;
 	item_cursor->auto_fill_handle_at_top = FALSE;
-	item_cursor->visible = 1;
+	item_cursor->prepared_to_drag = FALSE;
 }
 
 static void
@@ -1142,7 +1254,7 @@ item_cursor_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 {
 	GnomeCanvasItem *item;
 	ItemCursor *item_cursor;
-	
+
 	item = GNOME_CANVAS_ITEM (o);
 	item_cursor = ITEM_CURSOR (o);
 
@@ -1160,7 +1272,7 @@ item_cursor_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 	case ARG_COLOR: {
 		GdkColor color;
 		char *color_name;
-		
+
 		color_name = GTK_VALUE_STRING (*arg);
 		if (gnome_canvas_get_color (item->canvas, color_name, &color)){
 			item_cursor->color = color;
