@@ -3,12 +3,16 @@
  *
  * Author:
  *   Michael Meeks <mmeeks@gnu.org>
+ *
+ * Contributors:
+ *   Almer S. Tigelaar <almer1@dds.nl>
  */
 #include <config.h>
 #include "mstyle.h"
 #include "str.h"
 #include "border.h"
 #include "pattern.h"
+#include "format.h"
 #include "main.h"
 
 #define STYLE_DEBUG (style_debugging > 2)
@@ -238,8 +242,13 @@ mstyle_element_dump (const MStyleElement *e)
 			g_string_sprintf (ans, "%s blank", mstyle_names [e->type]);
 		break;
 	case MSTYLE_FORMAT:
-		g_string_sprintf (ans, "format '%s'", e->u.format->format);
+	{
+		char *fmt = style_format_as_XL (e->u.format, TRUE);
+		g_string_sprintf (ans, "format '%s'", fmt);
+		g_free (fmt);
+
 		break;
+	}
 	default:
 		g_string_sprintf (ans, "%s", mstyle_names [e->type]);
 		break;
@@ -536,7 +545,7 @@ mstyle_new_default (void)
 	
 	mstyle = mstyle_new ();
 
-	mstyle_set_format      (mstyle, "General");
+	mstyle_set_format_text (mstyle, "General");
 	mstyle_set_align_v     (mstyle, VALIGN_BOTTOM);
 	mstyle_set_align_h     (mstyle, HALIGN_GENERAL);
 	mstyle_set_orientation (mstyle, ORIENT_HORIZ);
@@ -781,7 +790,7 @@ mstyle_verify (const MStyle *style)
 	return TRUE;
 }
 
-gboolean inline
+gboolean
 mstyle_is_element_set (const MStyle *st, MStyleElementType t)
 {
 	g_return_val_if_fail (st != NULL, FALSE);
@@ -810,6 +819,30 @@ mstyle_unset_element (MStyle *st, MStyleElementType t)
 	st->elements [t].type = MSTYLE_ELEMENT_UNSET;
 }
 
+/**
+ * mstyle_replace_element:
+ * @src: Source mstyle
+ * @dst: Destination mstyle
+ * @t: Element to replace
+ * 
+ * This function replaces element 't' in mstyle 'dst' with element 't'
+ * in mstyle 'src'. (If element 't' was already set in mstyle 'dst' then
+ * the element will first be unset)
+ **/
+void
+mstyle_replace_element (MStyle *src, MStyle *dst, MStyleElementType t)
+{
+	g_return_if_fail (src != NULL);
+	g_return_if_fail (dst != NULL);
+
+	mstyle_element_ref (&src->elements [t]);
+	
+	if (mstyle_is_element_set (dst, t))
+		mstyle_unset_element (dst, t);
+
+	dst->elements [t] = src->elements [t];
+}
+
 void
 mstyle_set_color (MStyle *st, MStyleElementType t,
 		  StyleColor *col)
@@ -864,7 +897,7 @@ mstyle_set_border (MStyle *st, MStyleElementType t,
 
 }
 
-const MStyleBorder *
+MStyleBorder *
 mstyle_get_border (const MStyle *st, MStyleElementType t)
 {
 	g_return_val_if_fail (mstyle_is_element_set (st, t), NULL);
@@ -1039,14 +1072,32 @@ mstyle_get_font_size (const MStyle *style)
 }
 
 void
-mstyle_set_format (MStyle *style, const char *format)
+mstyle_set_format (MStyle *style, StyleFormat *format)
 {
 	g_return_if_fail (style != NULL);
 	g_return_if_fail (format != NULL);
 
-	mstyle_element_unref (style->elements [MSTYLE_FORMAT]);
-	style->elements [MSTYLE_FORMAT].type = MSTYLE_FORMAT;
-	style->elements [MSTYLE_FORMAT].u.format = style_format_new (format);
+	style_format_ref (format);
+	mstyle_element_unref (style->elements[MSTYLE_FORMAT]);
+	style->elements[MSTYLE_FORMAT].type = MSTYLE_FORMAT;
+	style->elements[MSTYLE_FORMAT].u.format = format;
+}
+
+void
+mstyle_set_format_text (MStyle *style, const char *format)
+{
+	StyleFormat *sf;
+
+	g_return_if_fail (style != NULL);
+	g_return_if_fail (format != NULL);
+
+	/* FIXME FIXME FIXME : This is a potential problem
+	 * I am not sure people are feeding us only translated formats.
+	 * This entire function should be deleted.
+	 */
+	sf = style_format_new_XL (format, FALSE);
+	mstyle_set_format (style, sf);
+	style_format_unref (sf);
 }
 
 StyleFormat *
@@ -1124,3 +1175,20 @@ mstyle_get_fit_in_cell (const MStyle *style)
 
 	return style->elements [MSTYLE_FIT_IN_CELL].u.fit_in_cell;
 }
+
+gboolean
+mstyle_visible_in_blank (const MStyle *st)
+{
+	MStyleElementType i;
+
+	if (mstyle_is_element_set (st, MSTYLE_PATTERN) &&
+	    mstyle_get_pattern (st) > 0)
+		return TRUE;
+
+	for (i = MSTYLE_BORDER_TOP ; i <= MSTYLE_BORDER_REV_DIAGONAL ; ++i)
+		if (mstyle_is_element_set (st, i) &&
+		    style_border_visible_in_blank (mstyle_get_border (st, i)))
+			return TRUE;
+
+	return FALSE;
+}
