@@ -5,7 +5,6 @@
  *    Miguel de Icaza 1999 (miguel@kernel.org)
  *
  * g_unichar_to_utf8: Copyright Red Hat, Inc
- *
  */
 #include <config.h>
 #include <gnome.h>
@@ -109,7 +108,7 @@ g_unichar_to_utf8 (gint c, gchar *outbuf)
  *
  * NOTE: This function got introduced when gnome-print switched to UTF-8,
  * and will disappear again once Gnumeric makes the switch. Deprecated at
- * birth! 
+ * birth!
  */
 int
 print_show_iso8859_1 (GnomePrintContext *pc, char const *text)
@@ -179,7 +178,13 @@ print_overflow (GnomePrintContext *context, GnomeFont *font,
 		double const * const line_offset, int num_lines)
 {
 	double const len = gnome_font_get_width_string_n (font, "#", 1);
-	int count = (len != 0) ? (width / len) : 0;
+	int count = 0;
+
+	if (len != 0)  {
+		count = width / len;
+		if (count == 0)
+			count = 1;
+	}
 
 	/* Center */
 	for (x1 += (width - count*len) / 2; --count >= 0 ; x1 += len )
@@ -352,12 +357,12 @@ print_cell (Cell const *cell, MStyle *mstyle, CellSpanInfo const * const spaninf
 		 */
 		text_base = y1 - font_ascent - ri->margin_a;
 		break;
-		
+
 	case VALIGN_CENTER:
 		text_base = y1 - font_ascent - ri->margin_a -
 		    (height - font_height) / 2;
 		break;
-		
+
 	case VALIGN_BOTTOM:
 		/*
 		 * y1 + row->size_pts == bottom grid line.
@@ -367,8 +372,8 @@ print_cell (Cell const *cell, MStyle *mstyle, CellSpanInfo const * const spaninf
 		text_base = y1 - ri->size_pts + font_descent + ri->margin_b;
 		break;
 	}
-	
-	halign = value_get_default_halign (cell->value, mstyle);
+
+	halign = cell_default_halign (cell, mstyle);
 
 	is_single_line = (halign != HALIGN_JUSTIFY &&
 			  valign != VALIGN_JUSTIFY &&
@@ -441,7 +446,8 @@ print_cell (Cell const *cell, MStyle *mstyle, CellSpanInfo const * const spaninf
 	cell_width_pts = gnome_font_get_width_string (print_font, text);
 
 	/* if a number overflows, do special drawing */
-	if (width < cell_width_pts && cell_is_number (cell)) {
+	if (width < cell_width_pts && cell_is_number (cell) &&
+	    sheet && !sheet->display_formulas) {
 		print_overflow (context, print_font,
 				x1 + ci->margin_a + 1,
 				text_base, width, line_offset, num_lines);
@@ -467,11 +473,11 @@ print_cell (Cell const *cell, MStyle *mstyle, CellSpanInfo const * const spaninf
 			break;
 
 		case HALIGN_CENTER:
-			x1 += 1 + ci->margin_a + (width - cell_width_pts) / 2; 
+			x1 += 1 + ci->margin_a + (width - cell_width_pts) / 2;
 			break;
 
 		case HALIGN_CENTER_ACROSS_SELECTION:
-			x1 = clip_x + (clip_width - cell_width_pts) / 2; 
+			x1 = clip_x + (clip_width - cell_width_pts) / 2;
 			break;
 
 		default:
@@ -695,7 +701,7 @@ print_cell_background (GnomePrintContext *context, Sheet *sheet,
  *
  * Return value: returns TRUE if at least one cell was printed
  */
-gboolean 
+gboolean
 print_cell_range (GnomePrintContext *context,
 		  Sheet *sheet,
 		  int start_col, int start_row,
@@ -706,7 +712,7 @@ print_cell_range (GnomePrintContext *context,
 	int row, col;
 	double x, y;
 	gboolean printed = FALSE;
-	
+
 	g_return_val_if_fail (context != NULL, FALSE);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (context), FALSE);
 	g_return_val_if_fail (sheet != NULL, FALSE);
@@ -740,22 +746,24 @@ print_cell_range (GnomePrintContext *context,
 			 *    there is a span descriptor.
 			 */
 			if (ri->pos == -1 || NULL == (span = row_span_get (ri, col))){
-				Cell *cell = sheet_cell_get (sheet, col, row);
-				MStyle *mstyle = NULL;
-
-				if (output)
-					mstyle = print_cell_background (
+				Cell   *cell   = sheet_cell_get (sheet, col, row);
+				MStyle *mstyle = (output)
+					? print_cell_background (
 						context, sheet, ci, ri,
-						col, row, x, y, FALSE);
+						col, row, x, y, FALSE)
+					: sheet_style_compute (sheet, col, row);
 
-				if (!cell_is_blank (cell)){
+				if (cell_is_blank (cell)) {
+					if (!output)
+						printed |= mstyle_visible_in_blank (mstyle);
+				} else {
+					printed = TRUE;
 					if (output)
 						print_cell (cell, mstyle, NULL,
 							    context, x, y);
-					printed = TRUE;
 				}
-				if (mstyle)
-					mstyle_unref (mstyle);
+
+				mstyle_unref (mstyle);
 
 				/* Increment the column
 				 * DO NOT move this outside the if, spanning
@@ -785,7 +793,7 @@ print_cell_range (GnomePrintContext *context,
 								context, sheet, ci, ri,
 								col, row, x, y,
 								col != start_span_col && is_visible);
-						
+
 						if (col == real_col) {
 							real_style = mstyle;
 							real_x = x;
