@@ -33,72 +33,78 @@
 #include "sheet-view.h"
 #include "application.h"
 
-/*
- * rendered_value_new
- * @cell: The cell whose value needs to be rendered
- *
- * Returns a RenderedValue displaying the value of the cell formated according
- * to the format style. If formulas are being displayed the text of the a
- * formula instead of its value.
- */
-
-RenderedValue *
-rendered_value_new (Cell *cell, GList *styles)
-{
-	MStyle *mstyle = (styles != NULL)
-		? sheet_style_compute_from_list (styles, cell->pos.col, cell->pos.row)
-		: cell_get_mstyle (cell);
-		
-	return rendered_value_new_ext (cell, mstyle);
-}
-
 /**
- * rendered_value_new_ext:
+ * rendered_value_new:
  * @cell:   The cell
  * @mstyle: The mstyle associated with the cell
+ * @dynamic_width : Allow format to depend on column width.
  * 
  * Formats the value of the cell according to the format style given in @mstyle
- * 
+ *
  * Return value: a new RenderedValue
  **/
 RenderedValue *
-rendered_value_new_ext (Cell *cell, MStyle *mstyle)
+rendered_value_new (Cell *cell, MStyle *mstyle, gboolean dynamic_width)
 {
 	RenderedValue	*res;
+	Sheet		*sheet;
 	StyleColor	*color;
+	float		 col_width = -1.;
 	char *str;
 
 	g_return_val_if_fail (cell != NULL, NULL);
 	g_return_val_if_fail (cell->value != NULL, NULL);
 
-	if (cell_has_expr (cell) &&
-	    cell->base.sheet != NULL && cell->base.sheet->display_formulas) {
+	sheet = cell->base.sheet;
+
+	if (cell_has_expr (cell) && sheet != NULL && sheet->display_formulas) {
 		ParsePos pp;
 		char *tmpstr = expr_tree_as_string (cell->base.expression,
-			parse_pos_init_cell (&pp, cell));
+						    parse_pos_init_cell (&pp, cell));
 		str = g_strconcat ("=", tmpstr, NULL);
 		g_free (tmpstr);
 		color = NULL;
-	} else {
+		dynamic_width = FALSE;
+	} else if (mstyle_is_element_set (mstyle, MSTYLE_FORMAT)) {
+		/* entered text CAN be null if called by set_value */
+		StyleFormat *format = mstyle_get_format (mstyle);
 
+		/* For format general approximate the cell width in characters */
+		if (style_format_is_general (format)) {
+			if (dynamic_width &&
+			    (cell->format == NULL ||
+			     style_format_is_general (cell->format))) {
+				StyleFont *style_font =
+					sheet_view_get_style_font (sheet, mstyle);
+				GdkFont *gdk_font =
+					style_font_gdk_font (style_font);
 
-		if (mstyle_is_element_set (mstyle, MSTYLE_FORMAT)) {
-			/* entered text CAN be null if called by
-			 * set_value
-			 */
-			char const *entered =
-				(!cell_has_expr (cell) && cell->entered_text != NULL)
-				? cell->entered_text->str : NULL;
-			StyleFormat *format = mstyle_get_format (mstyle);
+				/* FIXME : how does one get the width of the
+				 * widest character used to display numbers.
+				 * Use 4 as the max width for now.  Count the
+				 * inter character spacing by measuring a
+				 * string with 10 digits
+				 *
+				 * This does not belong here at all.  There
+				 * should be a global scaled StyleFont cache
+				 * indexed by stylefont and scale (res*zoom)
+				 * which pre-calculates this stuff.
+				 */
+				double const font_width = gdk_string_measure (gdk_font, "4444444444") / 10.;
+				if (font_width > 0)
+					col_width = (COL_INTERNAL_WIDTH (cell->col_info)) / font_width;
 
-			if  (style_format_is_general(format) && cell->format)
+				style_font_unref (style_font);
+			} else {
 				format = cell->format;
-			str = format_value (format, cell->value, &color, entered);
-		} else {
-			g_warning ("No format: serious error");
-			str = g_strdup ("Error");
-		}
-		mstyle_unref (mstyle);
+				dynamic_width = FALSE;
+			}
+		} else
+			dynamic_width = FALSE;
+		str = format_value (format, cell->value, &color, col_width);
+	} else {
+		g_warning ("No format: serious error");
+		str = g_strdup ("Error");
 	}
 
 	g_return_val_if_fail (str != NULL, NULL);
@@ -107,6 +113,7 @@ rendered_value_new_ext (Cell *cell, MStyle *mstyle)
 	res->rendered_text = string_get (str);
 	res->render_color = color;
 	res->width_pixel = res->height_pixel = 0;
+	res->dynamic_width = dynamic_width;
 	g_free (str);
 
 	return res;
@@ -140,6 +147,7 @@ rendered_value_calc_size (Cell const *cell)
 	MStyle *mstyle = cell_get_mstyle (cell);
 
 	rendered_value_calc_size_ext (cell, mstyle);
+	mstyle_unref (mstyle);
 }
 
 /*
@@ -158,8 +166,9 @@ rendered_value_calc_size (Cell const *cell)
 void
 rendered_value_calc_size_ext (Cell const *cell, MStyle *mstyle)
 {
+	Sheet *sheet = cell->base.sheet;
 	RenderedValue *rv = cell->rendered_value;
-	StyleFont *style_font = sheet_view_get_style_font (cell->base.sheet, mstyle);
+	StyleFont *style_font = sheet_view_get_style_font (sheet, mstyle);
 	GdkFont *gdk_font = style_font_gdk_font (style_font);
 	int font_height = style_font_get_height (style_font);
 	int const cell_w = COL_INTERNAL_WIDTH (cell->col_info);
@@ -172,7 +181,9 @@ rendered_value_calc_size_ext (Cell const *cell, MStyle *mstyle)
 	text       = rv->rendered_text->str;
 	text_width = gdk_string_measure (gdk_font, text);
 	
-	if (text_width < cell_w || cell_is_number (cell)) {
+	if (text_width < cell_w ||
+	    (cell_is_number (cell) &&
+	     sheet != NULL && !sheet->display_formulas)) {
 		rv->width_pixel  = text_width;
 		rv->height_pixel = font_height;
 	} else if (mstyle_get_align_h (mstyle) == HALIGN_JUSTIFY ||
@@ -239,7 +250,6 @@ rendered_value_calc_size_ext (Cell const *cell, MStyle *mstyle)
 		rv->height_pixel = font_height;
 	}
 	style_font_unref (style_font);
-	mstyle_unref (mstyle);
 }
 
 /* Return the value as a single string without format infomation.
@@ -298,21 +308,14 @@ cell_get_entered_text (Cell const *cell)
 		return ret;
 	}
 
-	/*
-	 * Return the value without parsing.
-	 */
-	if (cell->entered_text != NULL)
-		return g_strdup (cell->entered_text->str);
-
-	/* Getting desperate, no need to check for the rendered version.
-	 * This should not happen.
-	 */
-	else if (cell->value != NULL)
-		return value_get_as_string (cell->value);
-	else {
-		g_warning ("A cell with no expression, no value, and no entered_text ??");
-		return g_strdup ("<ERROR>");
+	if (cell->value != NULL) {
+		if (cell->value->type == VALUE_STRING)
+			return g_strconcat ("\'", cell->value->v_str.val->str, NULL);
+		return format_value (cell->format, cell->value, NULL, -1);
 	}
+
+	g_warning ("A cell with no expression, and no value ??");
+	return g_strdup ("<ERROR>");
 }
 
 int
