@@ -26,6 +26,8 @@
 #include "expr.h"
 #include "formats.h"
 #include "datetime.h"
+#include "mstyle.h"
+#include "errno.h"
 
 #include <math.h>
 
@@ -87,6 +89,7 @@ typedef struct _FillItem {
 	FillType     type;
 	Cell        *reference;
 	StyleFormat *fmt;
+	MStyle	    *style;
 
 	union {
 		ExprTree *formula;
@@ -170,14 +173,19 @@ matches_list (const String *str, int *n, int *is_i18n)
 static int
 string_has_number (String *str, int *num, int *pos)
 {
-	char *s = str->str, *p;
+	char *s = str->str, *p, *end;
 	int l = strlen (s);
 	gboolean found_number = FALSE;
+	long val;
 
-	if (isdigit ((unsigned char)*s)){
-		*num = atoi (s);
-		*pos = 0;
-		return TRUE;
+	errno = 0;
+	val = strtol (s, &end, 10);
+	if (s != end) {
+		if (errno != ERANGE) {
+			*num = val;
+			*pos = 0;
+			return TRUE;
+		}
 	}
 	if (l <= 1)
 		return FALSE;
@@ -189,10 +197,17 @@ string_has_number (String *str, int *num, int *pos)
 		return FALSE;
 
 	p++;
-	*num = atoi (p);
-	*pos = p - str->str;
+	errno = 0;
+	val = strtol (p, &end, 10);
+	if (p != end) {
+		if (errno != ERANGE) {
+			*num = val;
+			*pos = p - str->str;
+			return TRUE;
+		}
+	}
 
-	return TRUE;
+	return FALSE;
 }
 
 static void
@@ -215,6 +230,10 @@ fill_item_destroy (FillItem *fi)
 	default:
 		break;
 	}
+	if (fi->style) {
+		mstyle_unref (fi->style);
+		fi->style = NULL;
+	}
 	g_free (fi);
 }
 
@@ -229,15 +248,18 @@ str_contains (char const *str, char c)
 }
 
 static FillItem *
-fill_item_new (Cell *cell)
+fill_item_new (Sheet *sheet, int col, int row)
 {
 	Value     *value;
 	ValueType  value_type;
 	FillItem  *fi;
+	Cell *cell; 
 
 	fi = g_new (FillItem, 1);
 	fi->type = FILL_EMPTY;
+	fi->style = sheet_style_compute (sheet, col, row);
 
+	cell = sheet_cell_get (sheet, col, row);
 	fi->reference = cell;
 	if (!cell)
 		return fi;
@@ -433,7 +455,7 @@ type_is_compatible (FillItem *last, FillItem *current)
 }
 
 static GList *
-autofill_create_fill_items (Sheet *sheet, int x, int y, int region_count, int col_inc, int row_inc)
+autofill_create_fill_items (Sheet *sheet, int col, int row, int region_count, int col_inc, int row_inc)
 {
 	FillItem *last;
 	GList *item_list, *all_items, *l;
@@ -442,15 +464,11 @@ autofill_create_fill_items (Sheet *sheet, int x, int y, int region_count, int co
 	last = NULL;
 	item_list = all_items = NULL;
 
-	for (i = 0; i < region_count; i++){
-		FillItem *fi;
-		Cell *cell;
+	for (i = 0; i < region_count; i++) {
+		FillItem *fi = fill_item_new (sheet, col, row);
 
-		cell = sheet_cell_get (sheet, x, y);
-		fi = fill_item_new (cell);
-
-		if (!type_is_compatible (last, fi)){
-			if (last){
+		if (!type_is_compatible (last, fi)) {
+			if (last) {
 				all_items = g_list_append (all_items, item_list);
 				item_list = NULL;
 			}
@@ -460,8 +478,8 @@ autofill_create_fill_items (Sheet *sheet, int x, int y, int region_count, int co
 
 		item_list = g_list_append (item_list, fi);
 
-		x += col_inc;
-		y += row_inc;
+		col += col_inc;
+		row += row_inc;
 	}
 
 	if (item_list)
@@ -510,11 +528,6 @@ autofill_destroy_fill_items (GList *all_items)
 static void
 autofill_cell (Cell *cell, int idx, FillItem *fi)
 {
-	MStyle *mstyle = cell_get_mstyle (fi->reference);
-	sheet_style_attach_single (cell->base.sheet,
-				   cell->pos.col,
-				   cell->pos.row, mstyle);
-
 	switch (fi->type) {
 	case FILL_EMPTY:
 	case FILL_INVALID:
@@ -522,7 +535,7 @@ autofill_cell (Cell *cell, int idx, FillItem *fi)
 		return;
 
 	case FILL_STRING_CONSTANT:
-		sheet_cell_set_value (cell, value_new_string (fi->v.str->str), NULL);
+		cell_set_value (cell, value_new_string (fi->v.str->str), NULL);
 		return;
 
 	case FILL_STRING_WITH_NUMBER: {
@@ -547,7 +560,7 @@ autofill_cell (Cell *cell, int idx, FillItem *fi)
 			v = g_strconcat (n, buffer, NULL);
 			g_free (n);
 		}
-		/* FIXME : Can we set this as a value ? */
+		/* FIXME : Should we set this as a value ? */
 		sheet_cell_set_text (cell, v);
 		g_free (v);
 		return;
@@ -563,7 +576,7 @@ autofill_cell (Cell *cell, int idx, FillItem *fi)
 		} else
 			v = value_new_int (last->v.value->v_int.val +
 					   idx * last->delta.d_int);
-		sheet_cell_set_value (cell, v, fi->fmt);
+		cell_set_value (cell, v, fi->fmt);
 		return;
 	}
 
@@ -591,7 +604,7 @@ autofill_cell (Cell *cell, int idx, FillItem *fi)
 		res -= floor (res);
 		v = (res < 1e-6) ? value_new_int (d)
 			: value_new_float (((float_t)d) + res);
-		sheet_cell_set_value (cell, v, fi->fmt);
+		cell_set_value (cell, v, fi->fmt);
 		return;
 	}
 
@@ -614,7 +627,7 @@ autofill_cell (Cell *cell, int idx, FillItem *fi)
 		if (*text == '*')
 			text++;
 
-		sheet_cell_set_value (cell, value_new_string(text), NULL);
+		cell_set_value (cell, value_new_string(text), NULL);
 
 		return;
 	}
@@ -638,7 +651,7 @@ autofill_cell (Cell *cell, int idx, FillItem *fi)
 		 * relative references that will fall off the
 		 * edge ?? */
 		func = expr_rewrite (fi->v.formula, &rwinfo);
-		sheet_cell_set_expr (cell, (func == NULL) ? fi->v.formula : func);
+		cell_set_expr (cell, (func == NULL) ? fi->v.formula : func, NULL);
 		return;
 	}
 	}
@@ -652,8 +665,8 @@ sheet_autofill_dir (Sheet *sheet,
 		    int col_inc,      int row_inc)
 {
 	GList *all_items, *l, *m;
-	int x = base_col;
-	int y = base_row;
+	int col = base_col;
+	int row = base_row;
 	int pos, sub_index, loops, group_count;
 
 	/*
@@ -663,8 +676,8 @@ sheet_autofill_dir (Sheet *sheet,
 		sheet, base_col, base_row,
 		region_count, col_inc, row_inc);
 
-	x = base_col + region_count * col_inc;
-	y = base_row + region_count * row_inc;
+	col = base_col + region_count * col_inc;
+	row = base_row + region_count * row_inc;
 
 	/* Do the autofill */
 	l = all_items;
@@ -689,20 +702,22 @@ sheet_autofill_dir (Sheet *sheet,
 		}
 		fi = m->data;
 
-		cell = sheet_cell_get (sheet, x, y);
+		mstyle_ref (fi->style);
+		sheet_style_attach_single (sheet, col, row, fi->style);
 
+		cell = sheet_cell_get (sheet, col, row);
 		if (fi->type == FILL_EMPTY){
 			if (cell)
 				sheet_cell_remove (sheet, cell, TRUE);
 		} else {
 			if (!cell)
-				cell = sheet_cell_new (sheet, x, y);
-
-			autofill_cell (cell, loops * group_count + sub_index, fi);
+				cell = sheet_cell_new (sheet, col, row);
+			autofill_cell (cell,
+				       loops * group_count + sub_index, fi);
 		}
 
-		x += col_inc;
-		y += row_inc;
+		col += col_inc;
+		row += row_inc;
 	}
 
 	autofill_destroy_fill_items (all_items);
@@ -717,14 +732,17 @@ autofill_init (void)
 	autofill_register_list (month_long);
 }
 
-/*
- * Autofills a range of cells in the horizontal or the vertical direction
+/**
+ * sheet_autofill :
+ *
+ * An internal routine to autofill a region.  It does NOT
+ * queue a recalc, flag a status update, or regen spans.
  */
 void
 sheet_autofill (Sheet *sheet, int base_col, int base_row, int w, int h, int end_col, int end_row)
 {
-	int range;
 	static int autofill_inited;
+	int series;
 
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet));
@@ -735,12 +753,10 @@ sheet_autofill (Sheet *sheet, int base_col, int base_row, int w, int h, int end_
 	}
 
 	if (end_col != base_col + w - 1){
-		for (range = 0; range < h; range++)
-			sheet_autofill_dir (sheet, base_col, base_row+range, w, base_col, end_col+1, 1, 0);
+		for (series = 0; series < h; series++)
+			sheet_autofill_dir (sheet, base_col, base_row+series, w, base_col, end_col+1, 1, 0);
 	} else {
-		for (range = 0; range < w; range++)
-			sheet_autofill_dir (sheet, base_col+range, base_row, h, base_row, end_row+1, 0, 1);
+		for (series = 0; series < w; series++)
+			sheet_autofill_dir (sheet, base_col+series, base_row, h, base_row, end_row+1, 0, 1);
 	}
-
-	workbook_recalc (sheet->workbook);
 }
