@@ -103,7 +103,7 @@ sheet_object_bonobo_load_from_file (SheetObjectBonobo *sob, const char *fname)
 	
 	CORBA_exception_init (&ev);
 
-	pf = Bonobo_Unknown_query_interface (
+	pf = Bonobo_Unknown_queryInterface (
 		bonobo_object_corba_objref (BONOBO_OBJECT (sob->object_server)),
 		"IDL:Bonobo/PersistFile:1.0", &ev);
 
@@ -114,17 +114,19 @@ sheet_object_bonobo_load_from_file (SheetObjectBonobo *sob, const char *fname)
 			file = get_file_name ();
 		else
 			file = g_strdup (fname);
-		if (file)
+		if (file) {
 			Bonobo_PersistFile_load (pf, file, &ev);
+			if (BONOBO_EX (&ev))
+				g_warning ("Error '%s'", bonobo_exception_get_text (&ev));
+		}
 
-		Bonobo_Unknown_unref ((Bonobo_Unknown) pf, &ev);
-		CORBA_Object_release (pf, &ev);
+		bonobo_object_release_unref (pf, &ev);
 		g_free (file);
 
 		goto finish;
 	} 
 		
-	ps = Bonobo_Unknown_query_interface (
+	ps = Bonobo_Unknown_queryInterface (
 		bonobo_object_corba_objref (BONOBO_OBJECT (sob->object_server)),
 		"IDL:Bonobo/PersistStream:1.0", &ev);
 	
@@ -139,16 +141,19 @@ sheet_object_bonobo_load_from_file (SheetObjectBonobo *sob, const char *fname)
 		if (file) {
 			BonoboStream *stream;
 			
-			stream = bonobo_stream_fs_open (file, Bonobo_Storage_READ);
+			stream = bonobo_stream_open ("fs", file, Bonobo_Storage_READ, 0);
 			if (stream) {
 				Bonobo_PersistStream_load (
 					ps,
 					(Bonobo_Stream) bonobo_object_corba_objref (
 						BONOBO_OBJECT (stream)), "", &ev);
-			}
+				if (BONOBO_EX (&ev))
+					g_warning ("Error '%s'", bonobo_exception_get_text (&ev));
+			} else
+				g_warning ("Failed to open '%s'", file);
 		}
-		Bonobo_Unknown_unref ((Bonobo_Unknown) ps, &ev);
-		CORBA_Object_release (ps, &ev);
+
+		bonobo_object_release_unref (pf, &ev);
 		g_free (file);
 
 		goto finish;
@@ -157,7 +162,6 @@ sheet_object_bonobo_load_from_file (SheetObjectBonobo *sob, const char *fname)
 	return FALSE;
 
  finish:
-	sheet_object_realize (SHEET_OBJECT (sob));
 	CORBA_exception_free (&ev);
 	return TRUE;
 }
@@ -183,7 +187,7 @@ sheet_object_bonobo_load (SheetObjectBonobo *sob,
 	
 	CORBA_exception_init (&ev);
 	
-	ret = Bonobo_Unknown_query_interface (
+	ret = Bonobo_Unknown_queryInterface (
 		bonobo_object_corba_objref (BONOBO_OBJECT (sob->object_server)),
 		"IDL:Bonobo/PersistStream:1.0", &ev);
 	if (ev._major == CORBA_NO_EXCEPTION && ret != CORBA_OBJECT_NIL) {
@@ -206,7 +210,6 @@ sheet_object_bonobo_load (SheetObjectBonobo *sob,
 	return TRUE;
 }
 
-#ifdef ENABLE_BONOBO
 static void
 sheet_object_bonobo_print (SheetObject *so, SheetObjectPrintInfo *pi)
 {
@@ -224,7 +227,29 @@ sheet_object_bonobo_print (SheetObject *so, SheetObjectPrintInfo *pi)
 
 	bonobo_print_client_render (bpc, pi->pd);
 }
-#endif
+
+static void
+open_cb (GtkMenuItem *item, SheetObjectBonobo *sheet_object)
+{
+	sheet_object_bonobo_load_from_file (sheet_object, NULL);
+}
+
+static void
+sheet_object_bonobo_populate_menu (SheetObject *sheet_object,
+				   GnomeCanvasItem *obj_view,
+				   GtkMenu     *menu)
+{
+	GtkWidget *item = gtk_menu_item_new_with_label (_("Open"));
+
+	gtk_signal_connect (GTK_OBJECT (item), "activate",
+			    (GtkSignalFunc) open_cb, sheet_object);
+
+	gtk_menu_append (menu, item);
+
+	if (sheet_object_bonobo_parent_class->populate_menu)
+		sheet_object_bonobo_parent_class->
+			populate_menu (sheet_object, obj_view, menu);
+}
 
 static void
 sheet_object_bonobo_class_init (GtkObjectClass *object_class)
@@ -236,9 +261,8 @@ sheet_object_bonobo_class_init (GtkObjectClass *object_class)
 	/* Object class method overrides */
 	object_class->destroy = sheet_object_bonobo_destroy;
 
-#ifdef ENABLE_BONOBO
 	sheet_object_class->print = sheet_object_bonobo_print;
-#endif
+	sheet_object_class->populate_menu = sheet_object_bonobo_populate_menu;
 }
 
 GtkType
@@ -283,7 +307,7 @@ sheet_object_bonobo_construct (SheetObjectBonobo *sob, Sheet *sheet,
 		return NULL;
 	}
 
-	sob->client_site   = bonobo_client_site_new (sheet->workbook->priv->bonobo_container);
+	sob->client_site = bonobo_client_site_new (sheet->workbook->priv->bonobo_container);
 	
 	if (!bonobo_client_site_bind_embeddable (sob->client_site,
 						 sob->object_server)) {
