@@ -29,6 +29,7 @@
 #include "gnumeric-util.h"
 #include "gnumeric-type-util.h"
 #include "dependent.h"
+#include "sheet-view.h"
 #include "sheet-object-widget.h"
 #include "expr.h"
 #include "value.h"
@@ -122,7 +123,7 @@ sheet_object_widget_new_view (SheetObject *so, SheetView *sheet_view)
 }
 
 /*
- * This implemenationt moves the widget rather than
+ * This implemenation moves the widget rather than
  * destroying/updating/creating the views
  */
 static void
@@ -476,7 +477,7 @@ typedef struct {
 	GtkWidget *dialog;
 	GtkWidget *entry;
 
-	Workbook  *wb;
+	WorkbookControlGUI *wbcg;
 	SheetWidgetCheckbox *swc;
 } CheckboxConfigState;
 
@@ -486,12 +487,12 @@ cb_checkbox_config_destroy (GtkObject *w, CheckboxConfigState *state)
 	g_return_val_if_fail (w != NULL, FALSE);
 	g_return_val_if_fail (state != NULL, FALSE);
 
-	workbook_edit_detach_guru (state->wb);
+	workbook_edit_detach_guru (state->wbcg);
 
 	/* Handle window manger closing the dialog.
 	 * This will be ignored if we are being destroyed differently.
 	 */
-	workbook_finish_editing (state->wb, FALSE);
+	workbook_finish_editing (state->wbcg, FALSE);
 
 	state->dialog = NULL;
 
@@ -502,8 +503,8 @@ cb_checkbox_config_destroy (GtkObject *w, CheckboxConfigState *state)
 static void
 cb_checkbox_config_focus (GtkWidget *w, GdkEventFocus *ev, CheckboxConfigState *state)
 {
-	workbook_set_entry (state->wb, GTK_ENTRY (state->entry));
-	workbook_edit_select_absolute (state->wb);
+	workbook_set_entry (state->wbcg, GTK_ENTRY (state->entry));
+	workbook_edit_select_absolute (state->wbcg);
 }
 
 static void
@@ -527,11 +528,11 @@ cb_checkbox_config_clicked (GnomeDialog *dialog, gint button_number,
 		} else
 			checkbox_set_expr (&state->swc->dep, NULL);
 	}
-	workbook_finish_editing (state->wb, FALSE);
+	workbook_finish_editing (state->wbcg, FALSE);
 }
 
 static void
-sheet_widget_checkbox_user_config (SheetObject *so)
+sheet_widget_checkbox_user_config (SheetObject *so, SheetView *sheet_view)
 {
 	CheckboxConfigState *state;
 	SheetWidgetCheckbox *swc = SHEET_WIDGET_CHECKBOX (so);
@@ -540,7 +541,7 @@ sheet_widget_checkbox_user_config (SheetObject *so)
 
 	state = g_new (CheckboxConfigState, 1);
 	state->swc = swc;
-	state->wb = so->sheet->workbook;
+	state->wbcg = sheet_view->wbcg;
 	state->dialog = gnome_dialog_new (_("Checkbox Configure"),
 					  GNOME_STOCK_BUTTON_OK, 
 					  GNOME_STOCK_BUTTON_CANCEL, 
@@ -566,8 +567,8 @@ sheet_widget_checkbox_user_config (SheetObject *so)
  	gnumeric_editable_enters (GTK_WINDOW (state->dialog),
 				  GTK_EDITABLE(state->entry));
 
-	gnumeric_non_modal_dialog (state->wb, GTK_WINDOW (state->dialog));
-	workbook_edit_attach_guru (state->wb, state->dialog);
+	gnumeric_non_modal_dialog (state->wbcg, GTK_WINDOW (state->dialog));
+	workbook_edit_attach_guru (state->wbcg, state->dialog);
 	gtk_window_set_position (GTK_WINDOW (state->dialog), GTK_WIN_POS_MOUSE);
 	gtk_window_set_focus (GTK_WINDOW (state->dialog),
 			      GTK_WIDGET (state->entry));
