@@ -12,6 +12,7 @@
 #include <gdk/gdkkeysyms.h>
 #include "gnumeric.h"
 #include "gnumeric-sheet.h"
+#include "sheet-view.h"
 #include "gnumeric-util.h"
 #include "dialogs.h"
 #include "sheet-object.h"
@@ -23,19 +24,19 @@
 #include <bonobo.h>
 #endif
 
-/* Pulls the GnumericSheet from a SheetView */
-#define GNUMERIC_SHEET_VIEW(p) GNUMERIC_SHEET (SHEET_VIEW(p)->sheet_view);
-
 /* Returns the class for a SheetObject */
 #define SO_CLASS(so) SHEET_OBJECT_CLASS(GTK_OBJECT(so)->klass)
 
 static GtkObjectClass *sheet_object_parent_class;
 
-static void sheet_object_stop_editing    (SheetObject *so);
-static void sheet_object_populate_menu   (SheetObject *so, GtkMenu *menu);
-static int  control_point_handle_event   (GnomeCanvasItem *item,
-					  GdkEvent *event, SheetObject *so);
-static void update_bbox 		 (SheetObject *so);
+static void sheet_object_stop_editing  (SheetObject *so);
+static void sheet_object_populate_menu (SheetObject *so,
+				        GnomeCanvasItem *obj_view,
+				        GtkMenu *menu);
+static int  control_point_handle_event (GnomeCanvasItem *item,
+				        GdkEvent *event,
+				        SheetObject *so);
+static void update_bbox 	       (SheetObject *so);
 
 static void
 sheet_object_destroy (GtkObject *object)
@@ -194,14 +195,12 @@ sheet_object_item_destroyed (GnomeCanvasItem *item, SheetObject *so)
  * Creates a GnomeCanvasItem for a SheetView and sets up the event
  * handlers.
  */
-static GnomeCanvasItem *
+GnomeCanvasItem *
 sheet_object_new_view (SheetObject *so, SheetView *sheet_view)
 {
 	GnomeCanvasItem *item;
 
-	g_return_val_if_fail (sheet_view != NULL, NULL);
 	g_return_val_if_fail (IS_SHEET_VIEW (sheet_view), NULL);
-	g_return_val_if_fail (so != NULL, NULL);
 	g_return_val_if_fail (IS_SHEET_OBJECT (so), NULL);
 
 	item = SO_CLASS (so)->new_view (so, sheet_view);
@@ -211,6 +210,12 @@ sheet_object_new_view (SheetObject *so, SheetView *sheet_view)
 		return NULL;
 	}
 
+	/* Store some useful information */
+	gtk_object_set_data (GTK_OBJECT (item), 
+			     SHEET_OBJ_VIEW_OBJECT_KEY, so);
+	gtk_object_set_data (GTK_OBJECT (item), 
+			     SHEET_OBJ_VIEW_SHEET_VIEW_KEY, sheet_view);
+
 	gtk_signal_connect (GTK_OBJECT (item), "event",
 			    GTK_SIGNAL_FUNC (sheet_object_canvas_event), so);
 	gtk_signal_connect (GTK_OBJECT (item), "destroy",
@@ -481,8 +486,12 @@ sheet_mode_edit	(Sheet *sheet)
 
 	sheet_mode_clear (sheet);
 	sheet_show_cursor (sheet);
+
+#warning FIXME : this function has no business operating on the model.
+#if 0
 	if (workbook_edit_has_guru (sheet->workbook))
 		workbook_finish_editing (sheet->workbook, FALSE);
+#endif
 }
 
 /*
@@ -739,11 +748,19 @@ sheet_object_remove_cb (GtkWidget *widget, SheetObject *so)
 }
 
 static void
-cb_sheet_object_configure (GtkWidget *widget, SheetObject *so)
+cb_sheet_object_configure (GtkWidget *widget, GnomeCanvasItem *obj_view)
 {
-	g_return_if_fail (so != NULL);
+	SheetView *sheet_view;
+	SheetObject *so;
 
-	SO_CLASS(so)->user_config (so);
+	g_return_if_fail (obj_view != NULL);
+
+	so = gtk_object_get_data (GTK_OBJECT (obj_view),
+				  SHEET_OBJ_VIEW_OBJECT_KEY);
+	sheet_view = gtk_object_get_data (GTK_OBJECT (obj_view),
+					  SHEET_OBJ_VIEW_SHEET_VIEW_KEY);
+
+	SO_CLASS(so)->user_config (so, sheet_view);
 }
 
 /**
@@ -754,7 +771,9 @@ cb_sheet_object_configure (GtkWidget *widget, SheetObject *so)
  * Add standard items to the object's popup menu.
  **/
 static void
-sheet_object_populate_menu (SheetObject *so, GtkMenu *menu)
+sheet_object_populate_menu (SheetObject *so,
+			    GnomeCanvasItem *obj_view,
+			    GtkMenu *menu)
 {
 	GtkWidget *item = gnome_stock_menu_item (GNOME_STOCK_MENU_CLOSE,
 						 _("Delete"));
@@ -767,7 +786,7 @@ sheet_object_populate_menu (SheetObject *so, GtkMenu *menu)
 		item = gnome_stock_menu_item (GNOME_STOCK_MENU_PROP,
 					      _("Configure"));
 		gtk_signal_connect (GTK_OBJECT (item), "activate",
-				    GTK_SIGNAL_FUNC (cb_sheet_object_configure), so);
+				    GTK_SIGNAL_FUNC (cb_sheet_object_configure), obj_view);
 		gtk_menu_append (menu, item);
 	}
 }
@@ -777,16 +796,20 @@ menu_unrealize_cb (GtkMenu *menu, SheetObject *so)
 {
 }
 
-static GtkMenu *
-create_popup_menu (SheetObject *so)
+static void
+display_object_menu (SheetObject *so, GnomeCanvasItem *item,
+		     GdkEvent *event)
 {
-	GtkMenu *menu = GTK_MENU (gtk_menu_new ());
+	GtkMenu *menu;
 
-	SO_CLASS (so)->populate_menu (so, menu);
+	sheet_mode_edit_object (so);
+	menu = GTK_MENU (gtk_menu_new ());
+	SO_CLASS (so)->populate_menu (so, item, menu);
 	gtk_signal_connect (GTK_OBJECT (menu), "unrealize",
 			    GTK_SIGNAL_FUNC (menu_unrealize_cb), so);
 
-	return menu;
+	gtk_widget_show_all (GTK_WIDGET (menu));
+	gnumeric_popup_menu (menu, &event->button);
 }
 
 /*
@@ -822,24 +845,19 @@ control_point_handle_event (GnomeCanvasItem *item, GdkEvent *event,
 		switch (event->button.button) {
 		case 1:
 		case 2:
-		so->dragging = TRUE;
-		gnome_canvas_item_grab (item,
-					GDK_POINTER_MOTION_MASK |
-					GDK_BUTTON_RELEASE_MASK,
-					NULL, event->button.time);
-		last_x = event->button.x;
-		last_y = event->button.y;
-		break;
-		case 3:
-		{
-			GtkMenu *menu;
-
-			sheet_mode_edit_object (so);
-			menu = create_popup_menu (so);
-			gtk_widget_show_all (GTK_WIDGET (menu));
-			gnumeric_popup_menu (menu, &event->button);
+			so->dragging = TRUE;
+			gnome_canvas_item_grab (item,
+						GDK_POINTER_MOTION_MASK |
+						GDK_BUTTON_RELEASE_MASK,
+						NULL, event->button.time);
+			last_x = event->button.x;
+			last_y = event->button.y;
 			break;
-		}
+
+		case 3:
+			display_object_menu (so, item, event);
+			break;
+
 		default:
 			/* Ignore mouse wheel events */
 			return FALSE;
@@ -962,15 +980,9 @@ sheet_object_canvas_event (GnomeCanvasItem *item, GdkEvent *event,
 			event_total_y = 0;
 			break;
 		case 3:
-		{
-			GtkMenu *menu;
-
-			sheet_mode_edit_object (so);
-			menu = create_popup_menu (so);
-			gtk_widget_show_all (GTK_WIDGET (menu));
-			gnumeric_popup_menu (menu, &event->button);
+			display_object_menu (so, item, event);
 			break;
-		}
+
 		default:
 			/* Ignore mouse wheel events */
 			return FALSE;
