@@ -7,13 +7,11 @@
  */
 #include <config.h>
 
-#include <gnome.h>
-#include <gdk/gdkkeysyms.h>
-#include <gdk/gdkx.h>
-#include <string.h>
-#include "gnumeric.h"
+#include "sheet-view.h"
+#include "item-bar.h"
 #include "gnumeric-sheet.h"
 #include "workbook.h"
+#include "workbook-view.h"
 #include "workbook-cmd-format.h"
 #include "cell.h"
 #include "selection.h"
@@ -28,6 +26,10 @@
 #include "cmd-edit.h"
 #include "commands.h"
 
+#include <gdk/gdkkeysyms.h>
+#include <gdk/gdkx.h>
+#include <string.h>
+
 static GtkTableClass *sheet_view_parent_class;
 
 void
@@ -37,7 +39,7 @@ sheet_view_redraw_all (SheetView *sheet_view)
 	g_return_if_fail (IS_SHEET_VIEW (sheet_view));
 
 	gnome_canvas_request_redraw (
-		GNOME_CANVAS (sheet_view->sheet_view),
+		GNOME_CANVAS (sheet_view->canvas),
 		0, 0, INT_MAX, INT_MAX);
 	gnome_canvas_request_redraw (
 		GNOME_CANVAS (sheet_view->col_canvas),
@@ -63,7 +65,7 @@ sheet_view_redraw_cell_region (SheetView *sheet_view,
 	g_return_if_fail (sheet_view != NULL);
 	g_return_if_fail (IS_SHEET_VIEW (sheet_view));
 
-	gsheet = GNUMERIC_SHEET (sheet_view->sheet_view);
+	gsheet = GNUMERIC_SHEET (sheet_view->canvas);
 	g_return_if_fail (GNUMERIC_IS_SHEET (gsheet));
 	canvas = GNOME_CANVAS (gsheet);
 
@@ -108,7 +110,7 @@ sheet_view_redraw_headers (SheetView *sheet_view,
 	g_return_if_fail (sheet_view != NULL);
 	g_return_if_fail (IS_SHEET_VIEW (sheet_view));
 
-	gsheet = GNUMERIC_SHEET (sheet_view->sheet_view);
+	gsheet = GNUMERIC_SHEET (sheet_view->canvas);
 
 	if (col) {
 		int left = 0, right = INT_MAX-1;
@@ -166,7 +168,7 @@ sheet_view_update_cursor_pos (SheetView *sheet_view)
 	g_return_if_fail (sheet_view != NULL);
 	g_return_if_fail (IS_SHEET_VIEW (sheet_view));
 
-	gsheet = GNUMERIC_SHEET (sheet_view->sheet_view);
+	gsheet = GNUMERIC_SHEET (sheet_view->canvas);
 
 	/* Repsition the selection cursor */
 	item_cursor_reposition (gsheet->item_cursor);
@@ -186,7 +188,7 @@ sheet_view_set_zoom_factor (SheetView *sheet_view, double factor)
 	g_return_if_fail (sheet_view != NULL);
 	g_return_if_fail (IS_SHEET_VIEW (sheet_view));
 
-	gsheet = GNUMERIC_SHEET (sheet_view->sheet_view);
+	gsheet = GNUMERIC_SHEET (sheet_view->canvas);
 	col_item = ITEM_BAR (sheet_view->col_item);
 	row_item = ITEM_BAR (sheet_view->row_item);
 
@@ -209,11 +211,11 @@ sheet_view_set_zoom_factor (SheetView *sheet_view, double factor)
 
 	gtk_widget_set_usize (GTK_WIDGET (sheet_view->col_canvas), -1, h);
 	gnome_canvas_set_scroll_region (GNOME_CANVAS (sheet_view->col_canvas), 0, 0,
-					1000000*factor, h);
+					GNUMERIC_SHEET_FACTOR_X * factor, h);
 
 	gtk_widget_set_usize (GTK_WIDGET (sheet_view->row_canvas), w, -1);
 	gnome_canvas_set_scroll_region (GNOME_CANVAS (sheet_view->row_canvas), 0, 0,
-					w, 1200000*factor);
+					w, GNUMERIC_SHEET_FACTOR_Y * factor);
 
 	/* Recalibrate the starting offsets */
 	gsheet->col_offset.first =
@@ -268,7 +270,7 @@ sheet_view_scrollbar_config (SheetView const *sheet_view)
 {
 	GtkAdjustment *va = GTK_ADJUSTMENT (sheet_view->va);
 	GtkAdjustment *ha = GTK_ADJUSTMENT (sheet_view->ha);
-	GnumericSheet *gsheet = GNUMERIC_SHEET (sheet_view->sheet_view);
+	GnumericSheet *gsheet = GNUMERIC_SHEET (sheet_view->canvas);
 	Sheet         *sheet = sheet_view->sheet;
 	int const last_col = gsheet->col.last_full;
 	int const last_row = gsheet->row.last_full;
@@ -303,7 +305,7 @@ sheet_view_scrollbar_config (SheetView const *sheet_view)
  * Make the cell at the edit position visible.
  *
  * To be called from the "size_allocate" signal handler when the geometry of a
- * new sheet view has been configured. 
+ * new sheet view has been configured.
  */
 static void
 sheet_view_make_edit_pos_visible (SheetView const *sheet_view)
@@ -339,7 +341,7 @@ static void
 sheet_view_col_selection_changed (ItemBar *item_bar, int col, int modifiers, SheetView *sheet_view)
 {
 	Sheet *sheet = sheet_view->sheet;
-	GnumericSheet *gsheet = GNUMERIC_SHEET (sheet_view->sheet_view);
+	GnumericSheet *gsheet = GNUMERIC_SHEET (sheet_view->canvas);
 
 	if (modifiers){
 		if ((modifiers & GDK_SHIFT_MASK) && sheet->selections){
@@ -380,19 +382,20 @@ sheet_view_col_size_changed (ItemBar *item_bar, int col, int new_size_pixels,
 	 * If all cols in the selection are completely selected (top to bottom)
 	 * then resize all of them, otherwise just resize the selected col.
 	 */
- 	if (!sheet_selection_full_cols (sheet)) {
+ 	if (!sheet_selection_full_cols (sheet, col)) {
 		ColRowIndexList *sel = col_row_get_index_list (col, col, NULL);
-		cmd_resize_row_col (workbook_command_context_gui (sheet->workbook),
+		cmd_resize_row_col (WORKBOOK_CONTROL (sheet_view->wbcg),
 				    sheet, TRUE, sel, new_size_pixels);
 	} else
-		workbook_cmd_format_column_width (sheet, new_size_pixels);
+		workbook_cmd_format_column_width (WORKBOOK_CONTROL (sheet_view->wbcg),
+						  sheet, new_size_pixels);
 }
 
 static void
 sheet_view_row_selection_changed (ItemBar *item_bar, int row, int modifiers, SheetView *sheet_view)
 {
 	Sheet *sheet = sheet_view->sheet;
-	GnumericSheet *gsheet = GNUMERIC_SHEET (sheet_view->sheet_view);
+	GnumericSheet *gsheet = GNUMERIC_SHEET (sheet_view->canvas);
 
 	if (modifiers){
 		if ((modifiers & GDK_SHIFT_MASK) && sheet->selections){
@@ -433,12 +436,13 @@ sheet_view_row_size_changed (ItemBar *item_bar, int row, int new_size_pixels,
 	 * If all rows in the selection are completely selected (left to right)
 	 * then resize all of them, otherwise just resize the selected row.
 	 */
- 	if (!sheet_selection_full_rows (sheet)) {
+ 	if (!sheet_selection_full_rows (sheet, row)) {
 		ColRowIndexList *sel = col_row_get_index_list (row, row, NULL);
-		cmd_resize_row_col (workbook_command_context_gui (sheet->workbook),
+		cmd_resize_row_col (WORKBOOK_CONTROL (sheet_view->wbcg),
 				    sheet, FALSE, sel, new_size_pixels);
 	} else
-		workbook_cmd_format_row_height (sheet, new_size_pixels);
+		workbook_cmd_format_row_height (WORKBOOK_CONTROL (sheet_view->wbcg),
+						sheet, new_size_pixels);
 }
 
 /***************************************************************************/
@@ -480,7 +484,7 @@ horizontal_scroll_event (GtkScrollbar *scroll, GdkEvent *event, SheetView *sheet
 		gtk_widget_show_all (gtk_widget_get_toplevel (sheet_view->tip));
 
 	} else if (event->type == GDK_BUTTON_RELEASE) {
-		GnumericSheet  *gsheet = GNUMERIC_SHEET (sheet_view->sheet_view);
+		GnumericSheet  *gsheet = GNUMERIC_SHEET (sheet_view->canvas);
 		int col;
 
 		/* A button release can be generated without a press by people
@@ -509,7 +513,7 @@ vertical_scroll_event (GtkScrollbar *scroll, GdkEvent *event, SheetView *sheet_v
 		gtk_widget_show_all (gtk_widget_get_toplevel (sheet_view->tip));
 
 	} else if (event->type == GDK_BUTTON_RELEASE) {
-		GnumericSheet  *gsheet = GNUMERIC_SHEET (sheet_view->sheet_view);
+		GnumericSheet  *gsheet = GNUMERIC_SHEET (sheet_view->canvas);
 		int row;
 
 		/* A button release can be generated without a press by people
@@ -533,6 +537,7 @@ sheet_view_init (SheetView *sheet_view)
 {
 	GtkTable *table = GTK_TABLE (sheet_view);
 
+	sheet_view->sheet = NULL;
 	sheet_view->slide_handler = NULL;
 	sheet_view->slide_data = NULL;
 	sheet_view->sliding = -1;
@@ -576,9 +581,8 @@ sheet_view_construct (SheetView *sheet_view)
 			    GTK_SIGNAL_FUNC (sheet_view_row_size_changed),
 			    sheet_view);
 
-
 	/* Create the gnumeric sheet canvas */
-	sheet_view->sheet_view = gnumeric_sheet_new (
+	sheet_view->canvas = gnumeric_sheet_new (
 		sheet_view,
 		ITEM_BAR (sheet_view->col_item),
 		ITEM_BAR (sheet_view->row_item));
@@ -589,7 +593,7 @@ sheet_view_construct (SheetView *sheet_view)
 
 	/* Create the object group inside the GnumericSheet */
 	root_group = GNOME_CANVAS_GROUP (
-		GNOME_CANVAS (sheet_view->sheet_view)->root);
+		GNOME_CANVAS (sheet_view->canvas)->root);
 	sheet_view->object_group = GNOME_CANVAS_GROUP (
 		gnome_canvas_item_new (
 			root_group,
@@ -599,12 +603,12 @@ sheet_view_construct (SheetView *sheet_view)
 			NULL));
 
 	/* Attach the GnumericSheet */
-	gtk_table_attach (table, sheet_view->sheet_view,
+	gtk_table_attach (table, sheet_view->canvas,
 			  1, 2, 1, 2,
 			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			  0, 0);
-	gtk_widget_show (sheet_view->sheet_view);
+	gtk_widget_show (sheet_view->canvas);
 
 	i = sizeof (sheet_view->control_points)/sizeof(GnomeCanvasItem *);
 	while (i-- > 0)
@@ -707,6 +711,9 @@ sheet_view_destroy (GtkObject *object)
 	if (sheet_view->tip)
 		gtk_object_unref (GTK_OBJECT (sheet_view->tip));
 
+	if (sheet_view->sheet)
+		sheet_detach_sheet_view (sheet_view);
+
 	/* FIXME : Should we be pedantic and
 	 * 1) clear the control points
 	 * 2) remove ourselves from the sheets list of views ?
@@ -753,7 +760,7 @@ sheet_view_get_type (void)
 void
 sheet_view_hide_cursor (SheetView *sheet_view)
 {
-	GnumericSheet *gsheet = GNUMERIC_SHEET (sheet_view->sheet_view);
+	GnumericSheet *gsheet = GNUMERIC_SHEET (sheet_view->canvas);
 
 	item_cursor_set_visibility (gsheet->item_cursor, FALSE);
 }
@@ -761,7 +768,7 @@ sheet_view_hide_cursor (SheetView *sheet_view)
 void
 sheet_view_show_cursor (SheetView *sheet_view)
 {
-	GnumericSheet *gsheet = GNUMERIC_SHEET (sheet_view->sheet_view);
+	GnumericSheet *gsheet = GNUMERIC_SHEET (sheet_view->canvas);
 
 	item_cursor_set_visibility (gsheet->item_cursor, TRUE);
 }
@@ -787,7 +794,7 @@ sheet_view_comment_get_points (SheetView *sheet_view, int col, int row)
 	points->coords [5] = y + TRIANGLE_WIDTH;
 
 	for (i = 0; i < 3; i++){
-		gnome_canvas_c2w (GNOME_CANVAS (sheet_view->sheet_view),
+		gnome_canvas_c2w (GNOME_CANVAS (sheet_view->canvas),
 				  points->coords [i*2],
 				  points->coords [i*2+1],
 				  &(points->coords [i*2]),
@@ -806,7 +813,7 @@ sheet_view_comment_create_marker (SheetView *sheet_view, int col, int row)
 	g_return_val_if_fail (sheet_view != NULL, NULL);
 	g_return_val_if_fail (IS_SHEET_VIEW (sheet_view), NULL);
 
-	group = GNOME_CANVAS_GROUP (GNOME_CANVAS (sheet_view->sheet_view)->root);
+	group = GNOME_CANVAS_GROUP (GNOME_CANVAS (sheet_view->canvas)->root);
 	points = sheet_view_comment_get_points (sheet_view, col, row);
 
 	i = gnome_canvas_item_new (
@@ -814,6 +821,7 @@ sheet_view_comment_create_marker (SheetView *sheet_view, int col, int row)
 		"points",     points,
 		"fill_color", "red",
 		NULL);
+	gnome_canvas_points_free (points);
 
 	return i;
 }
@@ -866,7 +874,7 @@ sheet_view_selection_ant (SheetView *sheet_view)
 		sheet_view_selection_unant (sheet_view);
 
 	group = sheet_view->selection_group;
-	grid = GNUMERIC_SHEET (sheet_view->sheet_view)->item_grid;
+	grid = GNUMERIC_SHEET (sheet_view->canvas)->item_grid;
 
 	for (l = sheet_view->sheet->selections; l; l = l->next){
 		SheetSelection *ss = l->data;
@@ -891,7 +899,7 @@ void
 sheet_view_adjust_preferences (SheetView *sheet_view)
 {
 	Sheet *sheet = sheet_view->sheet;
-	Workbook *wb = sheet->workbook;
+	WorkbookView *wbv = wb_control_view (WORKBOOK_CONTROL (sheet_view->wbcg));
 
 	if (sheet->show_col_header)
 		gtk_widget_show (GTK_WIDGET (sheet_view->col_canvas));
@@ -908,12 +916,12 @@ sheet_view_adjust_preferences (SheetView *sheet_view)
 	else
 		gtk_widget_hide (sheet_view->select_all_btn);
 
-	if (wb->show_horizontal_scrollbar)
+	if (wbv->show_horizontal_scrollbar)
 		gtk_widget_show (sheet_view->hs);
 	else
 		gtk_widget_hide (sheet_view->hs);
 
-	if (wb->show_vertical_scrollbar)
+	if (wbv->show_vertical_scrollbar)
 		gtk_widget_show (sheet_view->vs);
 	else
 		gtk_widget_hide (sheet_view->vs);
@@ -926,17 +934,10 @@ sheet_view_get_style_font (const Sheet *sheet, MStyle const * const mstyle)
 	 * display.  72dpi is base size
 	 */
 	double const res  = application_dpi_to_pixels ();
-	double zoom;
 
-	/*
-	 * If sheet is NULL we default to 1
-	 */
-	if (sheet)
-		zoom = sheet->last_zoom_factor_used;
-	else
-		zoom = 1;
-		
-		
+	/* When previewing sheet can == NULL */
+	double const zoom = (sheet) ? sheet->last_zoom_factor_used : 1.;
+
 	return mstyle_get_font (mstyle, zoom * res);
 }
 
@@ -958,7 +959,7 @@ static gint
 sheet_view_sliding_callback (gpointer data)
 {
 	SheetView *sheet_view = data;
-	GnumericSheet *gsheet = GNUMERIC_SHEET (sheet_view->sheet_view);
+	GnumericSheet *gsheet = GNUMERIC_SHEET (sheet_view->canvas);
 	gboolean change = FALSE;
 	int col, row;
 
@@ -1050,7 +1051,7 @@ sheet_view_start_sliding (SheetView *sheet_view,
 			  gpointer user_data,
 			  int col, int row, int dx, int dy)
 {
-	GnumericSheet *gsheet = GNUMERIC_SHEET (sheet_view->sheet_view);
+	GnumericSheet *gsheet = GNUMERIC_SHEET (sheet_view->canvas);
 
 	/* Do not slide off the edge */
 	if (((dx == 0) ||
