@@ -1,18 +1,14 @@
 #ifndef GNUMERIC_SHEET_VIEW_H
 #define GNUMERIC_SHEET_VIEW_H
 
-#include "sheet.h"
+#include "gui-gnumeric.h"
 #include <gtk/gtktable.h>
-#include <gnome.h>
 
 #define SHEET_VIEW_TYPE        (sheet_view_get_type ())
 #define SHEET_VIEW(obj)        (GTK_CHECK_CAST((obj), SHEET_VIEW_TYPE, SheetView))
 #define SHEET_VIEW_CLASS(k)    (GTK_CHECK_CLASS_CAST((k), SHEET_VIEW_TYPE))
 #define IS_SHEET_VIEW(o)       (GTK_CHECK_TYPE((o), SHEET_VIEW_TYPE))
 
-struct _SheetView;
-typedef struct _SheetView SheetView;
-
 typedef gboolean (*SheetViewSlideHandler) (SheetView *sheet_view, int col, int row,
 					   gpointer user_data);
 
@@ -20,7 +16,8 @@ struct _SheetView {
 	GtkTable  table;
 
 	Sheet            *sheet;
-	GtkWidget        *sheet_view;
+	WorkbookControlGUI     *wbcg;	/* FIXME : How does sheet-view relate to workbook-control */
+	GtkWidget        *canvas;
 	GtkWidget	 *select_all_btn;
 	GnomeCanvas      *col_canvas, *row_canvas;
 	GnomeCanvasItem  *col_item, *row_item;
@@ -106,7 +103,7 @@ typedef struct {
  * These actually belong in sheet.h, but the structure dependency
  * forces me to put them here
  */
-SheetView *sheet_new_sheet_view     (Sheet *sheet);
-void       sheet_destroy_sheet_view (Sheet *sheet, SheetView *sheet_view);
+SheetView *sheet_new_sheet_view    (Sheet *sheet);
+void       sheet_detach_sheet_view (SheetView *sheet_view);
 
 #endif /* GNUMERIC_SHEET_VIEW_H */
