@@ -17,9 +17,6 @@ struct _SheetSelection {
         Range user;
 };
 
-#define SHEET_MAX_ROWS (64 * 1024)
-#define SHEET_MAX_COLS 256	/* 0 - 255 inclusive */
-
 /* The size, mask, and shift must be kept in sync */
 #define COLROW_SEGMENT_SIZE	0x80
 #define COLROW_SUB_INDEX(i)	((i) & 0x7f)
@@ -98,7 +95,7 @@ struct _Sheet {
 };
 
 #define SHEET_SIGNATURE 0x12349876
-#define IS_SHEET(x) ((x)->signature == SHEET_SIGNATURE)
+#define IS_SHEET(x) (((x) != NULL) && ((x)->signature == SHEET_SIGNATURE))
 
 Sheet      *sheet_new                  	 (Workbook *wb, const char *name);
 Sheet      *sheet_duplicate		 (Sheet const *source_sheet);
@@ -143,8 +140,9 @@ void        sheet_cell_comment_unlink    (Cell *cell);
 
 void        sheet_recompute_spans_for_col     (Sheet *sheet, int col);
 
-gboolean    sheet_is_region_empty_or_selected (Sheet *sheet, int start_col, int start_row,
-					       int end_col, int end_row);
+gboolean    sheet_is_region_empty_or_selected (Sheet *sheet, Range const *r);
+gboolean    sheet_is_region_empty 	      (Sheet *sheet, Range const *r);
+gboolean    sheet_is_cell_empty 	      (Sheet *sheet, int col, int row);
 
 /* Create new ColRowInfos from the default sheet style */
 ColRowInfo *sheet_col_new                  (Sheet *sheet);
@@ -161,12 +159,12 @@ ColRowInfo *sheet_col_get_info            (Sheet const *sheet, int const col);
 ColRowInfo *sheet_row_get_info            (Sheet const *sheet, int const row);
 
 /* Returns a pointer to a ColRowInfo: existed or NULL */
-ColRowInfo *sheet_row_get                 (Sheet const *sheet, int const pos);
 ColRowInfo *sheet_col_get                 (Sheet const *sheet, int const pos);
+ColRowInfo *sheet_row_get                 (Sheet const *sheet, int const pos);
 
 /* Returns a pointer to a ColRowInfo: existed or freshly created */
-ColRowInfo *sheet_row_fetch               (Sheet *sheet, int pos);
 ColRowInfo *sheet_col_fetch               (Sheet *sheet, int pos);
+ColRowInfo *sheet_row_fetch               (Sheet *sheet, int pos);
 
 /* Add a ColRowInfo to the Sheet */
 void        sheet_col_add                 (Sheet *sheet, ColRowInfo *cp);
@@ -265,18 +263,17 @@ void	    sheet_redraw_headers          (Sheet const *sheet,
 
 void	    sheet_flag_status_update_cell (Cell const *cell);
 void	    sheet_flag_status_update_range(Sheet const *sheet, Range const *range);
+void        sheet_flag_format_update_range(Sheet const *sheet, Range const *range);
 void        sheet_flag_selection_change   (Sheet const *sheet);
 void        sheet_update                  (Sheet const *sheet);
 void        sheet_compute_visible_ranges  (Sheet const *sheet);
 
-void        sheet_update_auto_expr        (Sheet const *sheet);
-
 void        sheet_mark_clean              (Sheet *sheet);
 void        sheet_set_dirty               (Sheet *sheet, gboolean is_dirty);
 gboolean    sheet_is_pristine             (Sheet *sheet);
 
 /* Sheet information manipulation */
-void        sheet_move_range              (CommandContext *context,
+void        sheet_move_range              (WorkbookControl *context,
 					   ExprRelocateInfo const * rinfo,
 					   GSList **reloc_storage);
 
@@ -303,6 +300,10 @@ void sheet_range_set_text (EvalPos const *pos, Range const *r, char const *str);
 void sheet_calc_spans	    (Sheet const *sheet,	SpanCalcFlags flags);
 void sheet_range_calc_spans (Sheet *sheet, Range r,	SpanCalcFlags flags);
 void sheet_cell_calc_span   (Cell const *cell,		SpanCalcFlags flags);
+void sheet_regen_adjacent_spans (Sheet *sheet,
+			     int start_col, int start_row,
+			     int end_col, int end_row,
+			     int *min_col, int *max_col);
 SpanCalcFlags required_updates_for_style (MStyle *style);
 
 /*
@@ -316,13 +317,13 @@ void sheet_corba_shutdown    (Sheet *);
  * These have undo/redo capabilities
  * and will route error messages to the caller appropriately.
  */
-gboolean  sheet_insert_cols (CommandContext *context, Sheet *sheet,
+gboolean  sheet_insert_cols (WorkbookControl *context, Sheet *sheet,
 			     int col, int count, GSList **reloc_storage);
-gboolean  sheet_delete_cols (CommandContext *context, Sheet *sheet,
+gboolean  sheet_delete_cols (WorkbookControl *context, Sheet *sheet,
 			     int col, int count, GSList **reloc_storage);
-gboolean  sheet_insert_rows (CommandContext *context, Sheet *sheet,
+gboolean  sheet_insert_rows (WorkbookControl *context, Sheet *sheet,
 			     int row, int count, GSList **reloc_storage);
-gboolean  sheet_delete_rows (CommandContext *context, Sheet *sheet,
+gboolean  sheet_delete_rows (WorkbookControl *context, Sheet *sheet,
 			     int row, int count, GSList **reloc_storage);
 
 void sheet_adjust_preferences (Sheet const *sheet);
@@ -335,7 +336,7 @@ typedef enum
 	CLEAR_NOCHECKARRAY = 0x8,
 } SheetClearFlags;
 
-void  sheet_clear_region (CommandContext *context,
+void  sheet_clear_region (WorkbookControl *context,
 			  Sheet *sheet,
 			  int start_col, int start_row,
 			  int end_col, int end_row,
