@@ -47,6 +47,12 @@ typedef struct {
 	int linepos;           /* Position on the current line */
 } Source_t;
 
+/* Struct used for autodiscovery */
+typedef struct {
+	int start;
+	int stop;
+} AutoDiscovery_t;
+
 /*******************************************************************************************************
  * STF PARSE OPTIONS : StfParseOptions related
  *******************************************************************************************************/
@@ -203,8 +209,10 @@ stf_parse_options_before_modification (StfParseOptions_t *parseoptions)
 	/* Copy the old splitpositions into a temporary array so we can compare them later */
 	parseoptions->oldsplitpositions = g_array_new (FALSE, FALSE, sizeof (int));
 
-	for (i = 0; i < parseoptions->splitpositions->len; i++)
-		g_array_append_val (parseoptions->oldsplitpositions, (int) g_array_index (parseoptions->splitpositions, int, i));
+	for (i = 0; i < parseoptions->splitpositions->len; i++) {
+		int idx = g_array_index (parseoptions->splitpositions, int, i);
+		g_array_append_val (parseoptions->oldsplitpositions, idx);
+	}
 }
 
 /**
@@ -218,7 +226,7 @@ gboolean
 stf_parse_options_after_modification (StfParseOptions_t *parseoptions)
 {
 	g_return_val_if_fail (parseoptions != NULL, FALSE);
-	g_return_val_if_fail (parseoptions->modificationmode == TRUE, FALSE);
+	g_return_val_if_fail (parseoptions->modificationmode, FALSE);
 
 	if (parseoptions->splitpositions->len == parseoptions->oldsplitpositions->len) {
 		int i;
@@ -836,7 +844,7 @@ stf_parse_csv_line (Source_t *src, StfParseOptions_t *parseoptions)
 
 		if (++col >= SHEET_MAX_COLS) {
 			g_warning (WARN_TOO_MANY_COLS, col);
-			return NULL;
+			return list;
 		}
 	}
 
@@ -917,7 +925,7 @@ stf_parse_fixed_line (Source_t *src, StfParseOptions_t *parseoptions)
 
 	while (*src->position && *src->position != parseoptions->terminator) {
 		char *field;
-		field = (char *)stf_parse_fixed_cell (src, parseoptions);
+		field = stf_parse_fixed_cell (src, parseoptions);
 
 		trim_spaces_inplace (field, parseoptions);
 
@@ -930,7 +938,7 @@ stf_parse_fixed_line (Source_t *src, StfParseOptions_t *parseoptions)
 
 		if (++col >= SHEET_MAX_COLS) {
 			g_warning (WARN_TOO_MANY_COLS, col);
-			return NULL;
+			return list;
 		}
 
 		src->splitpos++;
@@ -1102,6 +1110,7 @@ stf_parse_get_rowcount (StfParseOptions_t *parseoptions, const char *data)
 {
 	const char *iterator;
 	int rowcount = 0;
+	gboolean last_row_empty = TRUE;
 
 	g_return_val_if_fail (parseoptions != NULL, 0);
 	g_return_val_if_fail (data != NULL, 0);
@@ -1109,9 +1118,10 @@ stf_parse_get_rowcount (StfParseOptions_t *parseoptions, const char *data)
 	iterator = data;
 
 	while (*iterator) {
-
 		if (*iterator == parseoptions->terminator)
-			rowcount++;
+			rowcount++, last_row_empty = TRUE;
+		else
+			last_row_empty = FALSE;
 
 		if (parseoptions->parselines != -1)
 			if (rowcount > parseoptions->parselines)
@@ -1120,7 +1130,7 @@ stf_parse_get_rowcount (StfParseOptions_t *parseoptions, const char *data)
 		iterator++;
 	}
 
-	return (rowcount + 1);
+	return last_row_empty ? rowcount : rowcount + 1;
 }
 
 /**
@@ -1332,43 +1342,32 @@ stf_parse_get_colwidth (StfParseOptions_t *parseoptions, const char *data, int i
  * unix line-terminated format. this means that CRLF (windows) will be converted to LF
  * and CR (Macintosh) to LF.
  * In addition to that form feed (\F) characters will be removed.
- * NOTE : This will not resize the buffer
+ * NOTE: This will not resize the buffer
  *
- * returns : TRUE on success, FALSE otherwise.
+ * returns: TRUE on success, FALSE otherwise.
  **/
 gboolean
-stf_parse_convert_to_unix (const char *data)
+stf_parse_convert_to_unix (char *data)
 {
 	const char *iterator = data;
-	char *dest = (char *) data;
-	int len = 0;
+	char *dest = data;
 
 	if (!iterator)
 		return FALSE;
 
 	while (*iterator) {
-
 		if (*iterator == '\r') {
-			const char *temp = iterator;
-
-			temp++;
-
-			if (*temp != '\n') {
-				*dest = '\n';
-				dest++;
-				len++;
-			}
+			*dest++ = '\n';
 			iterator++;
+			if (*iterator == '\n')
+				iterator++;
+			continue;
 		} else if (*iterator == '\f') {
-		
 			iterator++;
+			continue;
 		}
-		
-		*dest = *iterator;
 
-		iterator++;
-		dest++;
-		len++;
+		*dest++ = *iterator++;
 	}
 	*dest = '\0';
 
@@ -1382,15 +1381,13 @@ stf_parse_convert_to_unix (const char *data)
  * returns weather the input data is valid to import.
  * (meaning it checks weather it is text only)
  *
- * returns : true if valid, false otherwise
+ * returns : NULL if valid, a pointer to the invalid character otherwise
  **/
-gboolean
-stf_parse_is_valid_data (const char *data)
+char *
+stf_parse_is_valid_data (char *data)
 {
-	gboolean valid;
-	const char *iterator = data;
+	char *iterator = data;
 
-	valid = TRUE;
 	while (*iterator) {
 
 		if (!isprint ((unsigned char)*iterator) &&
@@ -1399,13 +1396,279 @@ stf_parse_is_valid_data (const char *data)
 		    *iterator != '\t' &&
 		    *iterator != '\f') {
 
-			valid = FALSE;
-			break;
+			
+			return iterator;
 		}
 		iterator++;
 	}
 
-	return valid;
+	return NULL;
+}
+
+/**
+ * stf_parse_options_fixed_autodiscover:
+ * @parseoptions: a Parse options struct.
+ * @data_lines : The number of lines to look at in @data.
+ * @data : The actual data.
+ * 
+ * Automatically try to discover columns in the text to be parsed.
+ * We ignore empty lines (only containing parseoptions->terminator)
+ * 
+ **/
+void
+stf_parse_options_fixed_autodiscover (StfParseOptions_t *parseoptions, int data_lines, const char *data)
+{
+	const char *iterator = data;
+	GSList *list = NULL;
+	GSList *list_start = NULL;
+	int lines = 0;
+	int effective_lines = 0;
+	int max_line_length = 0;
+	int *line_begin_hits = NULL;
+	int *line_end_hits = NULL;
+	int i;
+
+	stf_parse_options_fixed_splitpositions_clear (parseoptions);
+
+	/*
+	 * First take a look at all possible white space combinations
+	 */
+	while (*iterator) {
+		gboolean begin_recorded = FALSE;
+		AutoDiscovery_t *disc = NULL;
+		int position = 0;
+
+		while (*iterator && *iterator != parseoptions->terminator) {
+			
+			if (!begin_recorded && *iterator == ' ') {
+
+				disc = g_new0 (AutoDiscovery_t, 1);
+
+				disc->start = position;
+
+				begin_recorded = TRUE;
+			} else if (begin_recorded && *iterator != ' ') {
+			
+				disc->stop = position;
+				list = g_slist_prepend (list, disc);
+
+				begin_recorded = FALSE;
+				disc = NULL;
+			}
+
+			position++;
+			iterator++;
+		}
+
+		if (position > max_line_length)
+			max_line_length = position;
+
+		/*
+		 * If there are excess spaces at the end of
+		 * the line : ignore them
+		 */
+		if (disc)
+			g_free (disc);
+
+		/*
+		 * Hop over the terminator
+		 */
+		if (*iterator)
+			iterator++;
+
+		if (position != 0)
+			effective_lines++;
+			
+		lines++;
+
+		if (lines >= data_lines)
+			break;
+	}
+
+	list       = g_slist_reverse (list);
+	list_start = list;
+
+	/*
+	 * Kewl stuff :
+	 * Look at the number of hits at each line position
+	 * if the number of hits equals the number of lines
+	 * we can be pretty sure this is the start or end
+	 * of a column, we filter out empty columns
+	 * later
+	 */
+	line_begin_hits = g_new0 (int, max_line_length + 1);
+	line_end_hits   = g_new0 (int, max_line_length + 1);
+	
+	while (list) {
+		AutoDiscovery_t *disc = list->data;
+
+		line_begin_hits[disc->start]++;
+		line_end_hits[disc->stop]++;
+
+		g_free (disc);
+		
+		list = g_slist_next (list);
+	}
+	g_slist_free (list_start);
+
+	for (i = 0; i < max_line_length + 1; i++) {
+
+		if (line_begin_hits[i] == effective_lines || line_end_hits[i] == effective_lines) {
+
+			stf_parse_options_fixed_splitpositions_add (parseoptions, i);
+		}
+
+	}
+
+	/*
+	 * Do some corrections to the initial columns
+	 * detected here, we obviously don't need to
+	 * do this if there are no columns at all.
+	 */
+	if (parseoptions->splitpositions->len > 0) {
+
+		/*
+		 * Try to find columns that look like :
+		 *
+		 * Example     100
+		 * Example2      9
+		 *
+		 * (In other words : Columns with left & right justification with
+		 *  a minimum of 2 spaces in the middle)
+		 * Split these columns in 2
+		 */
+
+		for (i = 0; i < parseoptions->splitpositions->len - 1; i++) {
+			int begin = g_array_index (parseoptions->splitpositions, int, i);
+			int end   = g_array_index (parseoptions->splitpositions, int, i + 1);
+			int num_spaces   = -1;
+			int spaces_start = 0;
+			gboolean right_aligned = TRUE;
+			gboolean left_aligned  = TRUE;
+			gboolean has_2_spaces  = TRUE;
+
+			iterator = data;
+			lines = 0;
+			while (*iterator) {
+				gboolean trigger = FALSE;
+				gboolean space_trigger = FALSE;
+				int pos = 0;
+
+				num_spaces   = -1;
+				spaces_start = 0;
+				while (*iterator && *iterator != parseoptions->terminator) {
+
+					if (pos == begin) {
+					
+						if (*iterator == ' ')
+							left_aligned = FALSE;
+
+						trigger = TRUE;
+					} else if (pos == end - 1) {
+					
+						if (*iterator == ' ')
+							right_aligned = FALSE;
+
+						trigger = FALSE;
+					}
+
+					if (trigger || pos == end - 1) {
+						if (!space_trigger && *iterator == ' ') {
+			
+							space_trigger = TRUE;
+							spaces_start = pos;
+						} else if (space_trigger && *iterator != ' ') {
+						
+							space_trigger = FALSE;
+							num_spaces = pos - spaces_start;
+						}
+					}
+						    
+					iterator++;
+					pos++;
+				}
+
+				if (num_spaces < 2)
+					has_2_spaces = FALSE;
+					
+				if (*iterator)
+					iterator++;
+
+				lines++;
+			
+				if (lines >= data_lines)
+					break;
+			}
+
+			/*
+			 * If this column meets all the criteria
+			 * split it into two at the last measured
+			 * spaces_start + num_spaces
+			 */
+			if (has_2_spaces && right_aligned && left_aligned) {
+				int val = (((spaces_start + num_spaces) - spaces_start) / 2) + spaces_start;
+
+				g_array_insert_val (parseoptions->splitpositions, i + 1, val);
+
+				/*
+				 * Skip over the inserted column
+				 */
+				i++;
+			}
+		}
+		
+		/*
+		 * Remove empty columns here if needed
+		 */	 
+		for (i = 0; i < parseoptions->splitpositions->len - 1; i++) {
+			int begin = g_array_index (parseoptions->splitpositions, int, i);
+			int end = g_array_index (parseoptions->splitpositions, int, i + 1);
+			gboolean only_spaces = TRUE;
+
+			iterator = data;
+			lines = 0;
+			while (*iterator) {
+				gboolean trigger = FALSE;
+				int pos = 0;
+			
+				while (*iterator && *iterator != parseoptions->terminator) {
+
+
+					if (pos == begin)
+						trigger = TRUE;
+					else if (pos == end)
+						trigger = FALSE;
+
+					if (trigger) {
+				
+						if (*iterator != ' ')
+							only_spaces = FALSE;
+					}
+
+					iterator++;
+					pos++;
+				}
+
+				if (*iterator)
+					iterator++;
+
+				lines++;
+			
+				if (lines >= data_lines)
+					break;
+			}
+
+			/*
+			 * The column only contains spaces
+			 * remove it
+			 */
+			if (only_spaces)
+				g_array_remove_index (parseoptions->splitpositions, i);
+		}
+	}
+
+	g_free (line_begin_hits);
+	g_free (line_end_hits);
 }
 
 /*******************************************************************************************************
