@@ -241,18 +241,28 @@ value_new_from_string (ValueType t, const char *str)
 	case VALUE_INTEGER:
 	{
 		char *end;
-		long l = strtol (str, &end, 10);
-		if (*end == '\0' && errno != ERANGE)
-			return value_new_int ((int)l);
+		long l;
+
+		errno = 0;
+		l = strtol (str, &end, 10);
+		if (str != end && *end == '\0') {
+			if (errno != ERANGE)
+				return value_new_int ((int)l);
+		}
 		return NULL;
 	}
 
 	case VALUE_FLOAT:
 	{
 		char *end;
-		double d = strtod (str, &end);
-		if (*end == '\0' && errno != ERANGE)
-			return value_new_float ((float_t)d);
+		double d;
+
+		errno = 0;
+		d = strtod (str, &end);
+		if (str != end && *end == '\0') {
+			if (errno != ERANGE)
+				return value_new_float ((float_t)d);
+		}
 		return NULL;
 	}
 
@@ -694,15 +704,21 @@ compare_float_float (Value const *va, Value const *vb)
 		return IS_GREATER;
 }
 
-/*
+/**
+ * value_compare :
+ *
+ * @a : value a
+ * @b : value b
+ * @case_sensitive : are string comparisons case sensitive.
+ *
  * Compares two (Value *) and returns one of ValueCompare
  *
  * if pos is non null it will perform implict intersection for
- * cellranges. if cs is true, be case sensitive on string 
+ * cellranges. if case_sensitive is true, be case sensitive on string 
  * comparisons.
  */
 ValueCompare
-value_compare (Value const *a, Value const *b, gboolean cs)
+value_compare (Value const *a, Value const *b, gboolean case_sensitive)
 {
 	ValueType ta, tb;
 
@@ -733,7 +749,7 @@ value_compare (Value const *a, Value const *b, gboolean cs)
 		{
 			int t;
 
-			if (cs) {
+			if (case_sensitive) {
 				t = strcmp (a->v_str.val->str, 
 					    b->v_str.val->str);
 			} else {
@@ -792,18 +808,3 @@ value_compare (Value const *a, Value const *b, gboolean cs)
 		return TYPE_MISMATCH;
 	}
 }
-
-StyleHAlignFlags
-value_get_default_halign (Value const *v, MStyle const *mstyle)
-{
-	StyleHAlignFlags align = mstyle_get_align_h (mstyle);
-	g_return_val_if_fail (v != NULL, HALIGN_RIGHT);
-
-	if (align == HALIGN_GENERAL) {
-		if (v->type == VALUE_FLOAT || v->type == VALUE_INTEGER)
-			return HALIGN_RIGHT;
-		return HALIGN_LEFT;
-	}
-
-	return align;
-}
