@@ -6,7 +6,6 @@
  *   Miguel de Icaza <miguel@gnu.org>
  *   Jody Goldberg <jgoldberg@home.com>
  */
-
 #include <config.h>
 #include <gnome.h>
 #include <locale.h>
@@ -31,23 +30,19 @@
 #include "expr-name.h"
 #include "cell.h"
 #include "cell-comment.h"
-#include "workbook.h"
+#include "io-context.h"
+#include "command-context.h"
+#include "workbook-control.h"
 #include "workbook-view.h"
+#include "workbook.h"
 #include "selection.h"
 #include "clipboard.h"
-#include "command-context.h"
 #include "format.h"
+#include "ranges.h"
 
 /*
  * A parsing context.
  */
-typedef enum
-{
-    GNUM_XML_V1,
-    GNUM_XML_V2,
-    GNUM_XML_V3,	/* >= 0.52 */
-    GNUM_XML_V4,	/* >= 0.57 */
-} GnumericXMLVersion;
 
 struct _XmlParseContext {
 	xmlDocPtr doc;		/* Xml document */
@@ -70,12 +65,14 @@ struct _XmlParseContext {
 XmlParseContext *
 xml_parse_ctx_new_full (xmlDocPtr             doc,
 			xmlNsPtr              ns,
+			GnumericXMLVersion    version,
 			XmlSheetObjectReadFn  read_fn,
 			XmlSheetObjectWriteFn write_fn,
 			gpointer              user_data)
 {
 	XmlParseContext *ctxt = g_new0 (XmlParseContext, 1);
 
+	ctxt->version   = version;
 	ctxt->doc       = doc;
 	ctxt->ns        = ns;
 	ctxt->expr_map  = g_hash_table_new (g_direct_hash, g_direct_equal);
@@ -91,7 +88,8 @@ XmlParseContext *
 xml_parse_ctx_new (xmlDocPtr doc,
 		   xmlNsPtr  ns)
 {
-	return xml_parse_ctx_new_full (doc, ns, NULL, NULL, NULL);
+	return xml_parse_ctx_new_full (
+		doc, ns, GNUM_XML_V5, NULL, NULL, NULL);
 }
        
 void
@@ -629,13 +627,20 @@ xml_search_child (xmlNodePtr node, const char *name)
 }
 
 static gboolean
-xml_read_range (xmlNodePtr tree, Range *res)
+xml_read_range (xmlNodePtr tree, Range *r)
 {
-	return
-	    xml_get_value_int (tree, "startCol", &res->start.col) &&
-	    xml_get_value_int (tree, "startRow", &res->start.row) &&
-	    xml_get_value_int (tree, "endCol",   &res->end.col) &&
-	    xml_get_value_int (tree, "endRow",   &res->end.row);
+	gboolean res =
+	    xml_get_value_int (tree, "startCol", &r->start.col) &&
+	    xml_get_value_int (tree, "startRow", &r->start.row) &&
+	    xml_get_value_int (tree, "endCol",   &r->end.col) &&
+	    xml_get_value_int (tree, "endRow",   &r->end.row);
+
+	/* Add some silent sanity checking to cleanup problems
+	 * with older versions of gnumeric that had some boundary problems
+	 */
+	range_check_sanity (r);
+
+	return res;
 }
 
 static void
@@ -1264,7 +1269,6 @@ xml_read_attributes (XmlParseContext *ctxt, xmlNodePtr tree, GList **list)
 
 	g_return_if_fail (ctxt != NULL);
 	g_return_if_fail (tree != NULL);
-	g_return_if_fail (ctxt->wb != NULL);
 
 	child = tree->childs;
 	while (child) {
@@ -1296,11 +1300,36 @@ xml_read_attributes (XmlParseContext *ctxt, xmlNodePtr tree, GList **list)
 	}
 }
 
+static xmlNodePtr
+xml_write_print_repeat_range (XmlParseContext *ctxt, char *name, PrintRepeatRange *range)
+{
+	xmlNodePtr cur;
+	String *str;
+	char *s;
+
+	g_return_val_if_fail (name != NULL, NULL);
+	g_return_val_if_fail (range != NULL, NULL);
+	
+	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, name, NULL);
+	if (range->use) {
+		s = value_cellrange_get_as_string ((Value *)&range->range, FALSE);
+	} else {
+		s = g_strdup ("");
+	}
+	str = string_get (s);
+	xml_set_value_string  (cur, "value", str);
+	
+	string_unref (str);
+	g_free (s);
+
+	return cur;
+}
+
 static xmlNodePtr
 xml_write_print_info (XmlParseContext *ctxt, PrintInformation *pi)
 {
 	xmlNodePtr cur, child;
-
+	
 	g_return_val_if_fail (pi != NULL, NULL);
 
 	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "PrintInformation", NULL);
@@ -1333,6 +1362,12 @@ xml_write_print_info (XmlParseContext *ctxt, PrintInformation *pi)
 	xml_set_value_int  (child, "value",    pi->print_titles);
 	xmlAddChild (cur, child);
 
+	child = xml_write_print_repeat_range (ctxt, "repeat_top", &pi->repeat_top);
+	xmlAddChild (cur, child);
+
+	child = xml_write_print_repeat_range (ctxt, "repeat_left", &pi->repeat_left);
+	xmlAddChild (cur, child);
+
 	if (pi->print_order == PRINT_ORDER_DOWN_THEN_RIGHT)
 		child = xmlNewDocNode (ctxt->doc, ctxt->ns, "order", "d_then_r");
 	else
@@ -1354,6 +1389,38 @@ xml_write_print_info (XmlParseContext *ctxt, PrintInformation *pi)
 	return cur;
 }
 
+static void
+xml_read_print_repeat_range (XmlParseContext *ctxt, xmlNodePtr tree, char *name, PrintRepeatRange *range)
+{
+	xmlNodePtr child;
+	
+	g_return_if_fail (ctxt != NULL);
+	g_return_if_fail (tree != NULL);
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (range != NULL);
+	
+	if (ctxt->version > GNUM_XML_V4) {
+		if ((child = xml_search_child (tree, name))) {
+			String *s = xml_get_value_string  (child, "value");
+
+			if (strcmp (s->str, "") != 0) {
+				Value *cellrange = range_parse (NULL, s->str, TRUE);
+
+				if (cellrange) {
+					range->range = cellrange->v_range;
+					range->use   = TRUE;
+					value_release (cellrange);
+				}
+			} else {
+				range->use = FALSE;
+			}
+			string_unref (s);
+		}
+	} else {
+		range->use = FALSE;
+	}
+}
+
 static void
 xml_read_print_info (XmlParseContext *ctxt, xmlNodePtr tree)
 {
@@ -1408,6 +1475,9 @@ xml_read_print_info (XmlParseContext *ctxt, xmlNodePtr tree)
 		pi->print_titles          = (b == 1);
 	}
 
+	xml_read_print_repeat_range (ctxt, tree, "repeat_top", &pi->repeat_top);
+	xml_read_print_repeat_range (ctxt, tree, "repeat_left", &pi->repeat_left);
+			
 	if ((child = xml_search_child (tree, "order"))) {
 		char *txt;
 		txt = xmlNodeGetContent (child);
@@ -1709,12 +1779,14 @@ xml_write_sheet_object (XmlParseContext *ctxt, SheetObject *object)
 			cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Bonobo", NULL);
 			ok = ctxt->write_fn (cur, object,
 					     ctxt->user_data);
-			xml_set_gnome_canvas_points (cur, "Points",
-						     object->bbox_points);
+			xml_set_gnome_canvas_points (
+				cur, "Points", object->bbox_points);
+
 			if (!ok) {
 				g_warning ("Error serializing bonobo sheet object");
-				g_warning ("Since libxml fails to implement xmlRemoveNode we"
-					   " are pretty stuffed");
+				xmlUnlinkNode (cur);
+				xmlFreeNode (cur);
+				cur = NULL;
 			}
 		} else
 			cur = NULL;
@@ -1798,14 +1870,18 @@ xml_read_sheet_object (XmlParseContext *ctxt, xmlNodePtr tree)
 		xml_get_coordinates (tree, "Points", &x1, &y1, &x2, &y2);
 
 		if (ctxt->read_fn) {
-			ret = ctxt->read_fn (tree, ctxt->sheet, x1, y1, x2, y2,
-					     ctxt->user_data);
+			ret = ctxt->read_fn (
+				tree, ctxt->sheet, x1, y1, x2, y2,
+				ctxt->user_data);
+
 			if (!ret)
 				g_warning ("Error hydrating sheet object");
 
 			return ret;
-		} else
+		} else {
+			g_warning ("Internal error, no read fn");
 			return NULL;
+		}
 #endif
 	} else {
 		fprintf (stderr,
@@ -2846,7 +2922,7 @@ xml_sheet_read (XmlParseContext *ctxt, xmlNodePtr tree)
 	 */
 	val = xml_value_get (tree, "Name");
 	if (val != NULL){
-		ret = workbook_sheet_lookup (ctxt->wb, (const char *) val);
+		ret = workbook_sheet_by_name (ctxt->wb, (const char *) val);
 		if (ret == NULL)
 			ret = sheet_new (ctxt->wb, (const char *) val);
 		g_free (val);
@@ -2973,7 +3049,7 @@ xml_read_selection_clipboard (XmlParseContext *ctxt, xmlNodePtr tree)
  * Create an XML subtree of doc equivalent to the given Workbook.
  */
 xmlNodePtr
-xml_workbook_write (XmlParseContext *ctxt, Workbook *wb)
+xml_workbook_write (XmlParseContext *ctxt, WorkbookView *wb_view)
 {
 	xmlNodePtr cur;
 	xmlNsPtr gmr;
@@ -2981,8 +3057,8 @@ xml_workbook_write (XmlParseContext *ctxt, Workbook *wb)
 	GtkArg *args;
 	guint n_args;
 	GList *sheets, *sheets0;
-	char *old_num_locale, *old_msg_locale;
-	GtkWidget *toplevel;
+	char *old_num_locale, *old_monetary_locale, *old_msg_locale;
+	Workbook *wb = wb_view_workbook (wb_view);
 
 	/*
 	 * General informations about the Sheet.
@@ -2991,17 +3067,19 @@ xml_workbook_write (XmlParseContext *ctxt, Workbook *wb)
 	if (cur == NULL)
 		return NULL;
 	if (ctxt->ns == NULL) {
-		gmr = xmlNewNs (cur, "http://www.gnome.org/gnumeric/v4", "gmr");
+		gmr = xmlNewNs (cur, "http://www.gnome.org/gnumeric/v5", "gmr");
 		xmlSetNs(cur, gmr);
 		ctxt->ns = gmr;
 	}
 
 	old_num_locale = g_strdup (gnumeric_setlocale (LC_NUMERIC, NULL));
 	gnumeric_setlocale (LC_NUMERIC, "C");
+	old_monetary_locale = g_strdup (gnumeric_setlocale (LC_MONETARY, NULL));
+	gnumeric_setlocale (LC_MONETARY, "C");
 	old_msg_locale = g_strdup (textdomain (NULL));
 	textdomain ("C");
 
-	args = workbook_get_attributev (wb, &n_args);
+	args = wb_view_get_attributev (wb_view, &n_args);
 	child = xml_write_attributes (ctxt, n_args, args);
 	if (child)
 		xmlAddChild (cur, child);
@@ -3020,9 +3098,8 @@ xml_workbook_write (XmlParseContext *ctxt, Workbook *wb)
 	xmlAddChild (cur, child);*/
 
 	child = xmlNewDocNode (ctxt->doc, ctxt->ns, "Geometry", NULL);
-	toplevel = workbook_get_toplevel (wb);
-	xml_set_value_int (child, "Width", toplevel->allocation.width);
-	xml_set_value_int (child, "Height", toplevel->allocation.height);
+	xml_set_value_int (child, "Width", wb_view->preferred_width);
+	xml_set_value_int (child, "Height", wb_view->preferred_height);
 	xmlAddChild (cur, child);
 
 	/*
@@ -3046,12 +3123,13 @@ xml_workbook_write (XmlParseContext *ctxt, Workbook *wb)
 	g_list_free (sheets0);
 
 	child = xmlNewDocNode (ctxt->doc, ctxt->ns, "UIData", NULL);
-	xml_set_value_int (child, "SelectedTab",
-			   gtk_notebook_get_current_page
-			   (GTK_NOTEBOOK (wb->notebook)));
+	xml_set_value_int (child, "SelectedTab", workbook_sheet_index_get (wb,
+		wb_view_cur_sheet (wb_view)));
 	xmlAddChild (cur, child);
 
 	textdomain (old_msg_locale);
+	gnumeric_setlocale (LC_MONETARY, old_monetary_locale);
+	g_free (old_monetary_locale);
 	g_free (old_msg_locale);
 	gnumeric_setlocale (LC_NUMERIC, old_num_locale);
 	g_free (old_num_locale);
@@ -3077,7 +3155,7 @@ xml_sheet_create (XmlParseContext *ctxt, xmlNodePtr tree)
 		Sheet *sheet;
 
 		sheet = sheet_new (ctxt->wb, (const char *) val);
-		workbook_attach_sheet (ctxt->wb, sheet);
+		workbook_sheet_attach (ctxt->wb, sheet, NULL);
 		g_free (val);
 	}
 	return;
@@ -3087,12 +3165,14 @@ xml_sheet_create (XmlParseContext *ctxt, xmlNodePtr tree)
  * Create a Workbook equivalent to the XML subtree of doc.
  */
 gboolean
-xml_workbook_read (Workbook *wb, XmlParseContext *ctxt, xmlNodePtr tree)
+xml_workbook_read (IOContext *context, WorkbookView *wb_view,
+		   XmlParseContext *ctxt, xmlNodePtr tree)
 {
 	Sheet *sheet;
 	xmlNodePtr child, c;
-	char *old_num_locale, *old_msg_locale;
+	char *old_num_locale, *old_monetary_locale, *old_msg_locale;
 	GList *list = NULL;
+	Workbook *wb = wb_view_workbook (wb_view);
 
 	if (strcmp (tree->name, "Workbook")){
 		fprintf (stderr,
@@ -3104,6 +3184,8 @@ xml_workbook_read (Workbook *wb, XmlParseContext *ctxt, xmlNodePtr tree)
 
 	old_num_locale = g_strdup (gnumeric_setlocale (LC_NUMERIC, NULL));
 	gnumeric_setlocale (LC_NUMERIC, "C");
+	old_monetary_locale = g_strdup (gnumeric_setlocale (LC_MONETARY, NULL));
+	gnumeric_setlocale (LC_MONETARY, "C");
 	old_msg_locale = g_strdup (textdomain (NULL));
 	textdomain ("C");
 
@@ -3112,12 +3194,12 @@ xml_workbook_read (Workbook *wb, XmlParseContext *ctxt, xmlNodePtr tree)
 		xml_read_summary (ctxt, child, wb->summary_info);
 
 	child = xml_search_child (tree, "Geometry");
-	if (child){
+	if (child) {
 		int width, height;
 
 		xml_get_value_int (child, "Width", &width);
 		xml_get_value_int (child, "Height", &height);
-		workbook_view_set_size (wb, width, height);
+		wb_view_preferred_size	  (wb_view, width, height);
 	}
 
 /*	child = xml_search_child (tree, "Style");
@@ -3158,21 +3240,24 @@ xml_workbook_read (Workbook *wb, XmlParseContext *ctxt, xmlNodePtr tree)
 	}
 
 	child = xml_search_child (tree, "Attributes");
-	if (child) {
+	if (child && ctxt->version >= GNUM_XML_V5) {
 		xml_read_attributes (ctxt, child, &list);
-		workbook_set_attributev (ctxt->wb, list);
+		wb_view_set_attributev (wb_view, list);
 		xml_free_arg_list (list);
 		g_list_free (list);
 	}
 
 	child = xml_search_child (tree, "UIData");
 	if (child) {
-		int tmp = 0;
-		if (xml_get_value_int (child, "SelectedTab", &tmp))
-			gtk_notebook_set_page (GTK_NOTEBOOK (wb->notebook), tmp);
+		int sheet_index = 0;
+		if (xml_get_value_int (child, "SelectedTab", &sheet_index))
+			wb_view_sheet_focus (wb_view, 
+				workbook_sheet_by_index (ctxt->wb, sheet_index));
 	}
 
 	textdomain (old_msg_locale);
+	gnumeric_setlocale (LC_MONETARY, old_monetary_locale);
+	g_free (old_monetary_locale);
 	g_free (old_msg_locale);
 	gnumeric_setlocale (LC_NUMERIC, old_num_locale);
 	g_free (old_num_locale);
@@ -3185,6 +3270,7 @@ static const struct {
 	char const * const id;
 	GnumericXMLVersion const version;
 } GnumericVersions [] = {
+	{ "http://www.gnome.org/gnumeric/v5", GNUM_XML_V5 },
 	{ "http://www.gnome.org/gnumeric/v4", GNUM_XML_V4 },
 	{ "http://www.gnome.org/gnumeric/v3", GNUM_XML_V3 },
 	{ "http://www.gnome.org/gnumeric/v2", GNUM_XML_V2 },
@@ -3192,7 +3278,7 @@ static const struct {
 	{ NULL }
 };
 
-static xmlNsPtr
+xmlNsPtr
 xml_check_version (xmlDocPtr doc, GnumericXMLVersion *version)
 {
 	xmlNsPtr gmr;
@@ -3272,7 +3358,7 @@ xml_probe (const char *filename)
  * returns 0 in case of success, -1 otherwise.
  */
 int
-gnumeric_xml_write_selection_clipboard (CommandContext *context, Sheet *sheet,
+gnumeric_xml_write_selection_clipboard (WorkbookControl *wbc, Sheet *sheet,
 					xmlChar **buffer, int *size)
 {
 	xmlDocPtr xml;
@@ -3284,8 +3370,8 @@ gnumeric_xml_write_selection_clipboard (CommandContext *context, Sheet *sheet,
 	 * Create the tree
 	 */
 	xml = xmlNewDoc ("1.0");
-	if (xml == NULL){
-		gnumeric_error_save (context, "");
+	if (xml == NULL) {
+		gnumeric_error_save (COMMAND_CONTEXT (wbc), "");
 		return -1;
 	}
 	ctxt.doc = xml;
@@ -3313,7 +3399,7 @@ gnumeric_xml_write_selection_clipboard (CommandContext *context, Sheet *sheet,
  * returns 0 on success and -1 otherwise.
  */
 int
-gnumeric_xml_read_selection_clipboard (CommandContext *context, CellRegion **cr,
+gnumeric_xml_read_selection_clipboard (WorkbookControl *wbc, CellRegion **cr,
 				       xmlChar *buffer)
 {
 	xmlDocPtr res;
@@ -3327,13 +3413,13 @@ gnumeric_xml_read_selection_clipboard (CommandContext *context, CellRegion **cr,
 	 */
 	res = xmlParseDoc (buffer);
 	if (res == NULL) {
-		gnumeric_error_read (context, "");
+		gnumeric_error_read (COMMAND_CONTEXT (wbc), "");
 		return -1;
 	}
 	if (res->root == NULL) {
 		xmlFreeDoc (res);
-		gnumeric_error_read
-			(context, _("Invalid xml clipboard data. Tree is empty ?"));
+		gnumeric_error_read (COMMAND_CONTEXT (wbc),
+			_("Invalid xml clipboard data. Tree is empty ?"));
 		return -1;
 	}
 
@@ -3354,7 +3440,8 @@ gnumeric_xml_read_selection_clipboard (CommandContext *context, CellRegion **cr,
  * the actual in-memory structure.
  */
 int
-gnumeric_xml_read_workbook (CommandContext *context, Workbook *wb,
+gnumeric_xml_read_workbook (IOContext *context,
+			    WorkbookView *wbv,
 			    const char *filename)
 {
 	xmlDocPtr res;
@@ -3369,13 +3456,13 @@ gnumeric_xml_read_workbook (CommandContext *context, Workbook *wb,
 	 */
 	res = xmlParseFile (filename);
 	if (res == NULL) {
-		gnumeric_error_read (context, "");
+		gnumeric_io_error_read (context, "");
 		return -1;
 	}
 	if (res->root == NULL) {
 		xmlFreeDoc (res);
-		gnumeric_error_read
-			(context, _("Invalid xml file. Tree is empty ?"));
+		gnumeric_io_error_read (context,
+			_("Invalid xml file. Tree is empty ?"));
 		return -1;
 	}
 
@@ -3383,15 +3470,16 @@ gnumeric_xml_read_workbook (CommandContext *context, Workbook *wb,
 	gmr = xml_check_version (res, &version);
 	if (gmr == NULL) {
 		xmlFreeDoc (res);
-		gnumeric_error_read (context, _("Is not an Gnumeric Workbook file"));
+		gnumeric_io_error_read (context,
+			_("Is not an Gnumeric Workbook file"));
 		return -1;
 	}
 
 	/* Parse the file */
 	ctxt = xml_parse_ctx_new (res, gmr);
 	ctxt->version = version;
-	xml_workbook_read (wb, ctxt, res->root);
-	workbook_set_saveinfo (wb, filename, FILE_FL_AUTO,
+	xml_workbook_read (context, wbv, ctxt, res->root);
+	workbook_set_saveinfo (wb_view_workbook (wbv), filename, FILE_FL_AUTO,
 			       &gnumeric_xml_write_workbook);
 	xml_parse_ctx_destroy (ctxt);
 	xmlFreeDoc (res);
@@ -3404,26 +3492,26 @@ gnumeric_xml_read_workbook (CommandContext *context, Workbook *wb,
  * returns 0 in case of success, -1 otherwise.
  */
 int
-gnumeric_xml_write_workbook (CommandContext *context, Workbook *wb,
+gnumeric_xml_write_workbook (IOContext *context, WorkbookView *wb_view,
 			     const char *filename)
 {
 	xmlDocPtr xml;
 	XmlParseContext *ctxt;
 	int ret;
 
-	g_return_val_if_fail (wb != NULL, -1);
+	g_return_val_if_fail (wb_view != NULL, -1);
 	g_return_val_if_fail (filename != NULL, -1);
 
 	/*
 	 * Create the tree
 	 */
 	xml = xmlNewDoc ("1.0");
-	if (xml == NULL){
-		gnumeric_error_save (context, "");
+	if (xml == NULL) {
+		gnumeric_io_error_save (context, "");
 		return -1;
 	}
 	ctxt = xml_parse_ctx_new (xml, NULL);
-	xml->root = xml_workbook_write (ctxt, wb);
+	xml->root = xml_workbook_write (ctxt, wb_view);
 	xml_parse_ctx_destroy (ctxt);
 
 	/*
@@ -3433,7 +3521,7 @@ gnumeric_xml_write_workbook (CommandContext *context, Workbook *wb,
 	ret = xmlSaveFile (filename, xml);
 	xmlFreeDoc (xml);
 	if (ret < 0) {
-		gnumeric_error_save (context, "");
+		gnumeric_io_error_save (context, "");
 		return -1;
 	}
 	return 0;
